@@ -26,17 +26,10 @@
 (define-group fragment-tag
   framed-fragment shell-fragment scm-fragment cpp-fragment mmx-fragment)
 
-;; override variant function for verbatim and scm
-;; FIXME: find a nicer solution
+(define-group tmdoc-prog-tag
+  verbatim scm mmx cpp)
 
-(tm-define (variant-circulate forward?)
+(tm-define (variant-circulate t forward?)
   (:mode in-tmdoc?)
-  (:inside verbatim)
-  (with-innermost t 'verbatim
-    (tree-assign-node! t 'scm)))
-
-(tm-define (variant-circulate forward?)
-  (:mode in-tmdoc?)
-  (:inside scm)
-  (with-innermost t 'scm
-    (tree-assign-node! t 'verbatim)))
+  (:require (tree-in? t (tmdoc-prog-tag-list)))
+  (variant-circulate-in t (tmdoc-prog-tag-list) forward?))
