@@ -30,75 +30,66 @@
 ;; Inserting rows and columns
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(tm-define (kbd-return)
-  (:inside table)
+(tm-define (kbd-enter t shift?)
+  (:require (table-markup-context? t))
   (let ((x (inside-which '(table document))))
     (cond ((== x 'document)
 	   (insert-return))
 	  (else
-	   (table-insert-row #t)
-	   (table-go-to (table-which-row) 1)))))
+           (table-insert-row #t)
+           (table-go-to (table-which-row) 1)))))
 
-(tm-define (structured-insert forwards?)
-  (:inside table)
+(tm-define (structured-insert-horizontal t forwards?)
+  (:require (table-markup-context? t))
   (table-insert-column forwards?))
 
-(tm-define (structured-insert-up)
-  (:inside table)
-  (table-insert-row #f))
+(tm-define (structured-insert-vertical t downwards?)
+  (:require (table-markup-context? t))
+  (table-insert-row downwards?))
 
-(tm-define (structured-insert-down)
-  (:inside table)
-  (table-insert-row #t))
-
-(tm-define (structured-remove forwards?)
-  (:inside table)
+(tm-define (structured-remove-horizontal t forwards?)
+  (:require (table-markup-context? t))
   (table-remove-column forwards?))
 
+(tm-define (structured-remove-vertical t downwards?)
+  (:require (table-markup-context? t))
+  (table-remove-row downwards?))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Posititioning
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(tm-define (positioning-default)
-  (:inside table)
-  (cell-del-format ""))
+(tm-define (geometry-default t)
+  (:require (table-markup-context? t))
+  (with-focus-after t
+    (cell-del-format "")))
 
-(tm-define (positioning-left)
-  (:inside table)
-  (cell-halign-left))
+(tm-define (geometry-horizontal t forward?)
+  (:require (table-markup-context? t))
+  (with-focus-after t
+    (if forward? (cell-halign-right) (cell-halign-left))))
 
-(tm-define (positioning-right)
-  (:inside table)
-  (cell-halign-right))
-
-(tm-define (positioning-up)
-  (:inside table)
-  (cell-valign-up))
-
-(tm-define (positioning-down)
-  (:inside table)
-  (cell-valign-down))
+(tm-define (geometry-vertical t down?)
+  (:require (table-markup-context? t))
+  (with-focus-after t
+    (if down? (cell-valign-down) (cell-valign-up))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Structured traversal
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(tm-define (similar-complex-context? t)
-  (:case cell row table tformat)
-  #f)
+(define (cell-search-downwards t)
+  (if (tree-is? t 'cell) t
+      (and (tree-down t)
+           (cell-search-downwards (tree-down t)))))
 
-(define (cell-context? t)
-  (and (tree-is? t 'cell)
-       (tree-is? t :up 'row)
-       (tree-is? t :up :up 'table)))
-
-(define (cell-not-at-top-context? t)
-  (and (cell-context? t)
-       (> (tree-index (tree-up t)) 0)))
-
-(define (cell-not-at-bottom-context? t)
-  (and (cell-context? t)
-       (< (tree-index (tree-up t)) (- (tree-arity (tree-up t 2)) 1))))
+(define (table-non-extremal-context? t downwards?)
+  (and (table-markup-context? t)
+       (and-with c (cell-search-downwards t)
+         (and-with i (tree-index (tree-up c))
+           (if downwards?
+               (< i (- (tree-arity (tree-up c 2)) 1))
+               (> i 0))))))
 
 (define (cell-move-absolute c row col)
   (let* ((r (tree-up c))
@@ -116,58 +107,32 @@
 	 (col (+ (tree-index c) dcol)))
     (cell-move-absolute c row col)))
 
-(tm-define (traverse-up)
-  (:context cell-not-at-top-context?)
-  (with-innermost c cell-not-at-top-context?
-    (cell-move-relative c -1 0)))
-
-(tm-define (traverse-down)
-  (:context cell-not-at-bottom-context?)
-  (with-innermost c cell-not-at-bottom-context?
-    (cell-move-relative c 1 0)))
+(tm-define (traverse-vertical t downwards?)
+  (:require (table-non-extremal-context? t downwards?))
+  (and-with c (cell-search-downwards t)
+    (cell-move-relative c (if downwards? 1 -1) 0)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Structured movements
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(define (cell-simple-context? t)
-  (and (nleaf? t)
-       (simple-context? (tree-down t))
-       (cell-context? t)))
+(tm-define (structured-horizontal t forwards?)
+  (:require (table-markup-context? t))
+  (with-focus-after t
+    (and-with c (cell-search-downwards t)
+      (cell-move-relative c 0 (if forwards? 1 -1)))))
 
-(tm-define (structured-left)
-  (:context cell-simple-context?)
-  (with-innermost c cell-simple-context?
-    (cell-move-relative c 0 -1)))
+(tm-define (structured-vertical t downwards?)
+  (:require (table-markup-context? t))
+  (with-focus-after t
+    (and-with c (cell-search-downwards t)
+      (cell-move-relative c (if downwards? 1 -1) 0))))
 
-(tm-define (structured-right)
-  (:context cell-simple-context?)
-  (with-innermost c cell-simple-context?
-    (cell-move-relative c 0 1)))
-
-(tm-define (structured-up)
-  (:context cell-simple-context?)
-  (with-innermost c cell-simple-context?
-    (cell-move-relative c -1 0)))
-
-(tm-define (structured-down)
-  (:context cell-simple-context?)
-  (with-innermost c cell-simple-context?
-    (cell-move-relative c 1 0)))
-
-(tm-define (structured-exit-left)
-  (:context cell-simple-context?)
-  (with-innermost c cell-simple-context?
-    (with t (tree-ref c :up :up)
-      (while (tree-in? t :up '(tformat document)) (set! t (tree-up t)))
-      (tree-go-to t :up :start))))
-
-(tm-define (structured-exit-right)
-  (:context cell-simple-context?)
-  (with-innermost c cell-simple-context?
-    (with t (tree-ref c :up :up)
-      (while (tree-in? t :up '(tformat document)) (set! t (tree-up t)))
-      (tree-go-to t :up :end))))
+(tm-define (structured-inner-extremal t forwards?)
+  (:require (table-markup-context? t))
+  (with-focus-after t
+    (and-with c (cell-search-downwards t)
+      (tree-go-to c (if forwards? :end :start)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Commands for tables
@@ -306,8 +271,7 @@
     (position-set sp (selection-get-start))
     (position-set ep (selection-get-end))
     (map (lambda (var val)
-	   (selection-set-start-path (position-get sp))
-	   (selection-set-end-path (position-get ep))
+	   (selection-set (position-get sp) (position-get ep))
 	   (cell-set-format var val))
 	 vars vals)
     (position-delete sp)
@@ -528,15 +492,16 @@
 	 (st  (table-cell-tree row -1)))
     (tree-search-subtree st (stree->tree '(eq-number)))))
 
-(tm-define (numbered?)
-  (:inside eqnarray eqnarray*)
-  (if (table-search-number-equation) #t #f))
+(tm-define (numbered-numbered? t)
+  (:require (tree-in? t '(eqnarray eqnarray*)))
+  (and (== t (tree-innermost '(eqnarray eqnarray*)))
+       (if (table-search-number-equation) #t #f)))
 
 (tm-define (table-number-equation)
   (let* ((row (table-which-row))
 	 (st  (table-cell-tree row -1)))
     (tree-go-to st :end)
-    (insert '(eq-number))))
+    (insert-go-to '(eq-number) '(0))))
 
 (tm-define (table-nonumber-equation)
   (let ((r (table-search-number-equation)))
@@ -554,8 +519,9 @@
 	 (tree-inside? t2 t1)
 	 (table-inside-sub? t1 (tree-up t2)))))
 
-(tm-define (toggle-number)
-  (:inside eqnarray eqnarray*)
-  (if (table-search-number-equation)
-      (table-nonumber-equation)
-      (table-number-equation)))
+(tm-define (numbered-toggle t)
+  (:require (tree-in? t '(eqnarray eqnarray*)))
+  (when (== t (tree-innermost '(eqnarray eqnarray*)))
+    (if (table-search-number-equation)
+        (table-nonumber-equation)
+        (table-number-equation))))
