@@ -5,37 +5,35 @@
 ;; DESCRIPTION : the Table menu
 ;; COPYRIGHT   : (C) 1999  Joris van der Hoeven
 ;;
-;; This software falls under the GNU general public license and comes WITHOUT
-;; ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for details.
-;; If you don't have this file, write to the Free Software Foundation, Inc.,
-;; 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+;; This software falls under the GNU general public license version 3 or later.
+;; It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+;; in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (texmacs-module (table table-menu)
-  (:use (table table-edit)))
+  (:use (table table-edit)
+        (generic generic-menu)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Inserting tables
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (menu-bind insert-table-menu
-  (if (style-has? "std-dtd")
-      (when (and (in-text?) (style-has? "env-float-dtd"))
-	    ("Small table" (make 'small-table))
-	    ("Big table" (make 'big-table))
-	    ---))
+  (if (and (style-has? "std-dtd") (in-text?) (style-has? "env-float-dtd"))
+      ("Small table" (make 'small-table))
+      ("Big table" (make 'big-table))
+      ---)
   ("Plain tabular" (make 'tabular))
   ("Centered tabular" (make 'tabular*))
   ("Plain block" (make 'block))
   ("Centered block" (make 'block*))
-  (if (style-has? "std-dtd")
-      (when (in-math?)
-	    ---
-	    ("Matrix" (make 'matrix))
-	    ("Determinant" (make 'det))
-	    ("Choice" (make 'choice))
-	    ("Stack" (make 'stack)))))
+  (if (and (style-has? "std-dtd") (in-math?))
+      ---
+      ("Matrix" (make 'matrix))
+      ("Determinant" (make 'det))
+      ("Choice" (make 'choice))
+      ("Stack" (make 'stack))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Submenus of the Table menu
@@ -170,19 +168,9 @@
   ("None" (cell-set-background ""))
   ("Foreground" (cell-set-background "foreground"))
   ---
-  ("Black" (cell-set-background "black"))
-  ("White" (cell-set-background "white"))
-  ("Grey" (cell-set-background "grey"))
-  ("Red" (cell-set-background "red"))
-  ("Blue" (cell-set-background "blue"))
-  ("Yellow" (cell-set-background "yellow"))
-  ("Green" (cell-set-background "green"))
-  ("Orange" (cell-set-background "orange"))
-  ("Magenta" (cell-set-background "magenta"))
-  ("Brown" (cell-set-background "brown"))
-  ("Pink" (cell-set-background "pink"))
+  (pick-background (cell-set-background answer))
   ---
-  ("Other"  (interactive cell-set-background)))
+  ("Other" (interactive cell-set-background)))
 
 (menu-bind cell-special-menu
   ("Set span" (interactive cell-set-span))
@@ -215,29 +203,30 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (menu-bind table-menu
-  (-> "Insert"
-      ("Row above" (table-insert-row #f))
-      ("Row below" (table-insert-row #t))
-      ("Column to the left" (table-insert-column #f))
-      ("Column to the right" (table-insert-column #t))
-      ("Blank row" (interactive table-insert-blank-row))
-      ("Blank column" (interactive table-insert-blank-column)))
-  (-> "Remove"
-      ("This row" (table-remove-row #f))
-      ("This column" (table-remove-column #f)))
-  ---
+  (assuming #f
+    (-> "Insert"
+        ("Row above" (table-insert-row #f))
+        ("Row below" (table-insert-row #t))
+        ("Column to the left" (table-insert-column #f))
+        ("Column to the right" (table-insert-column #t))
+        ("Blank row" (interactive table-insert-blank-row))
+        ("Blank column" (interactive table-insert-blank-column)))
+    (-> "Remove"
+        ("This row" (table-remove-row #f))
+        ("This column" (table-remove-column #f)))
+    ---)
   (-> "Width" (link table-width-menu))
   (-> "Height" (link table-height-menu))
   (-> "Border" (link table-border-menu))
   (-> "Padding" (link table-padding-menu))
   (-> "Horizontal alignment" (link table-halign-menu))
   (-> "Vertical alignment" (link table-valign-menu))
-  ---
+  ;;---
   (-> "Special properties" (link table-special-menu)))
 
 (menu-bind cell-menu
   (-> "Operation mode" (link cell-mode-menu))
-  ---
+  ;;---
   (-> "Width" (link cell-width-menu))
   (-> "Height" (link cell-height-menu))
   (-> "Border" (link cell-border-menu))
@@ -245,7 +234,7 @@
   (-> "Horizontal alignment" (link cell-halign-menu))
   (-> "Vertical alignment" (link cell-valign-menu))
   (-> "Background color" (link cell-color-menu))
-  ---
+  ;;---
   (-> "Special properties" (link cell-special-menu)))
 
 (menu-bind vertical-table-cell-menu
@@ -262,172 +251,191 @@
   (if (== (get-cell-mode) "column") (-> "Column" (link cell-menu)))
   (if (== (get-cell-mode) "table") (-> "Cells" (link cell-menu))))
 
+(tm-menu (standard-focus-menu t)
+  (:require (table-markup-context? t))
+  (dynamic (focus-tag-menu t))
+  (-> "Move" (dynamic (focus-move-menu t)))
+  (-> "Resize" (dynamic (focus-insert-menu t)))
+  ---
+  (group "Table")
+  (link table-menu)
+  ---
+  (if (== (get-cell-mode) "cell") (group "Cell"))
+  (if (== (get-cell-mode) "row") (group "Row"))
+  (if (== (get-cell-mode) "column") (group "Column"))
+  (if (== (get-cell-mode) "table") (group "Cells"))
+  (link cell-menu)
+  (dynamic (focus-extra-menu t))
+  (dynamic (focus-hidden-menu t)))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Icons for manipulation of tables
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (menu-bind table-insert-icons
   ((balloon (icon "tm_table_arow.xpm")
-	    "Insert row above (A-up)")
+	    "Insert row above")
    (table-insert-row #f))
   ((balloon (icon "tm_table_brow.xpm")
-	    "Insert row below (A-down)")
+	    "Insert row below")
    (table-insert-row #t))
   ((balloon (icon "tm_table_lcol.xpm")
-	    "Insert column to the left (A-left)")
+	    "Insert column to the left")
    (table-insert-column #f))
   ((balloon (icon "tm_table_rcol.xpm")
-	    "Insert column to the right (A-right)")
+	    "Insert column to the right")
    (table-insert-column #t))
   ---
   ((balloon (icon "tm_table_drow.xpm")
-	    "Delete row (M-t h delete)")
+	    "Delete row")
    (table-remove-row #t))
   ((balloon (icon "tm_table_dcol.xpm")
-	    "Delete column (M-t v delete)")
+	    "Delete column")
    (table-remove-column #t)))
 
 (menu-bind table-hpos-icons
   ((balloon (icon "tm_table_left.xpm")
-	    "Align table at the left (M-t H l)")
+	    "Align table at the left")
    (table-set-halign "l"))
   ((balloon (icon "tm_table_hcenter.xpm")
-	    "Align table at the horizontal center (M-t H c)")
+	    "Align table at the horizontal center")
    (table-set-halign "c"))
   ((balloon (icon "tm_table_right.xpm")
-	    "Align table to the right (M-t H r)")
+	    "Align table to the right")
    (table-set-halign "r"))
   ((icon "tm_empty.xpm") (noop))
   ((balloon (icon "tm_table_bleft.xpm")
-	    "Align table at base of left column (M-t H L)")
+	    "Align table at base of left column")
    (table-set-halign "L"))
   ((balloon (icon "tm_table_bhcenter.xpm")
-	    "Align table at base of middle column (M-t H C)")
+	    "Align table at base of middle column")
    (table-set-halign "C"))
   ((balloon (icon "tm_table_bright.xpm")
-	    "Align table at base of right column (M-t H R)")
+	    "Align table at base of right column")
    (table-set-halign "R"))
   ((balloon (icon "tm_table_hbase.xpm")
-	    "Align table at base of origin column (M-t H O)")
+	    "Align table at base of origin column")
    (table-set-halign "O")))
 
 (menu-bind table-vpos-icons
   ((balloon (icon "tm_table_bottom.xpm")
-	    "Align table at the bottom (M-t V b)")
+	    "Align table at the bottom")
    (table-set-valign "b"))
   ((balloon (icon "tm_table_vcenter.xpm")
-	    "Align table at the center (M-t V c)")
+	    "Align table at the center")
    (table-set-valign "c"))
   ((balloon (icon "tm_table_top.xpm")
-	    "Align table at the top (M-t V t)")
+	    "Align table at the top")
    (table-set-valign "t"))
   ((balloon (icon "tm_table_vfrac.xpm")
-	    "Align table at axis (M-t V f)")
+	    "Align table at axis")
    (table-set-valign "f"))
   ((balloon (icon "tm_table_bbottom.xpm")
-	    "Align table at the base of bottom row (M-t V B)")
+	    "Align table at the base of bottom row")
    (table-set-valign "B"))
   ((balloon (icon "tm_table_bvcenter.xpm")
-	    "Align table at the base of middle row (M-t V C)")
+	    "Align table at the base of middle row")
    (table-set-valign "C"))
   ((balloon (icon "tm_table_btop.xpm")
-	    "Align table at the base of top row (M-t V T)")
+	    "Align table at the base of top row")
    (table-set-valign "T"))
   ((balloon (icon "tm_table_vbase.xpm")
-	    "Align table at the base of origin row (M-t V O)")
+	    "Align table at the base of origin row")
    (table-set-valign "O")))
 
 (menu-bind cell-mode-icons
   ((balloon (icon "tm_cell_by_cell.xpm")
-	    "Perform operations on cells (M-t m c)")
+	    "Perform operations on cells")
    (set-cell-mode "cell"))
   ((balloon (icon "tm_cell_by_row.xpm")
-	    "Perform operations on rows (M-t m h)")
+	    "Perform operations on rows")
    (set-cell-mode "row"))
   ((balloon (icon "tm_cell_by_column.xpm")
-	    "Perform operations on columns (M-t m v)")
+	    "Perform operations on columns")
    (set-cell-mode "column"))
   ((balloon (icon "tm_cell_by_table.xpm")
-	    "Perform operations on entire table (M-t m t)")
+	    "Perform operations on entire table")
    (set-cell-mode "table")))
 
 (menu-bind cell-pos-icons
   ((balloon (icon "tm_cell_left.xpm")
-	    "Align cell to the left (M-t h l)")
+	    "Align cell to the left")
    (cell-set-halign "l"))
   ((balloon (icon "tm_cell_hcenter.xpm")
-	    "Horizontally center the cell (M-t h c)")
+	    "Horizontally center the cell")
    (cell-set-halign "c"))
   ((balloon (icon "tm_cell_right.xpm")
-	    "Align cell to the right (M-t h r)")
+	    "Align cell to the right")
    (cell-set-halign "r"))
   ((balloon (icon "tm_cell_hbase.xpm")
-	    "Horizontally align cell to the base (M-t h R)")
+	    "Horizontally align cell to the base")
    (cell-set-halign "R"))
   ((balloon (icon "tm_cell_hdot.xpm")
-	    "Align cell to the decimal dot (M-t h .)")
+	    "Align cell to the decimal dot")
    (cell-set-halign "L."))
   ((balloon (icon "tm_cell_hcomma.xpm")
-	    "Align cell to the decimal comma (M-t h ,)")
+	    "Align cell to the decimal comma")
    (cell-set-halign "L,"))
   ((balloon (icon "tm_cell_bottom.xpm")
-	    "Align cell to the bottom (M-t v b)")
+	    "Align cell to the bottom")
    (cell-set-valign "b"))
   ((balloon (icon "tm_cell_vcenter.xpm")
-	    "Vertically center the cell (M-t v c)")
+	    "Vertically center the cell")
    (cell-set-valign "c"))
   ((balloon (icon "tm_cell_top.xpm")
-	    "Align cell to the top (M-t v t)")
+	    "Align cell to the top")
    (cell-set-valign "t"))
   ((balloon (icon "tm_cell_vbase.xpm")
-	    "Vertically align cell to the base (M-t v B)")
+	    "Vertically align cell to the base")
    (cell-set-valign "B")))
 
 (menu-bind cell-size-icons
   ((balloon (icon "tm_cell_width.xpm")
-	    "Set width of cell (M-t h s)")
+	    "Set width of cell")
    (cell-interactive-set "cell-width"))
   ((balloon (icon "tm_cell_hmin.xpm")
-	    "Width is minimum of specified width and box width (M-t h m)")
+	    "Width is minimum of specified width and box width")
    (cell-set-hmode "min"))
   ((balloon (icon "tm_cell_hexact.xpm")
-	    "Width is exactly the specified width (M-t h e)")
+	    "Width is exactly the specified width")
    (cell-set-hmode "exact"))
   ((balloon (icon "tm_cell_hmax.xpm")
-	    "Width is maximum of specified width and box width (M-t h M)")
+	    "Width is maximum of specified width and box width")
    (cell-set-hmode "max"))
   ((balloon (icon "tm_cell_height.xpm")
-	    "Set height of cell (M-t v s)")
+	    "Set height of cell")
    (cell-interactive-set "cell-height"))
   ((balloon (icon "tm_cell_vmin.xpm")
-	    "Height is minimum of specified height and box height (M-t v m)")
+	    "Height is minimum of specified height and box height")
    (cell-set-vmode "min"))
   ((balloon (icon "tm_cell_vexact.xpm")
-	    "Height is exactly the specified height (M-t v e)")
+	    "Height is exactly the specified height")
    (cell-set-vmode "exact"))
   ((balloon (icon "tm_cell_vmax.xpm")
-	    "Height is maximum of specified height and box height (M-t v M)")
+	    "Height is maximum of specified height and box height")
    (cell-set-vmode "max")))
 
 (menu-bind cell-decoration-icons
   ((balloon (icon "tm_decorate_left.xpm")
-	    "Use left hand column as border (M-t b left)")
+	    "Use left hand column as border")
    (table-column-decoration #f))
   ((balloon (icon "tm_decorate_right.xpm")
-	    "Use right hand column as border (M-t b right)")
+	    "Use right hand column as border")
    (table-column-decoration #t))
   ((balloon (icon "tm_decorate_up.xpm")
-	    "Use row above as border (M-t b up)")
+	    "Use row above as border")
    (table-row-decoration #f))
   ((balloon (icon "tm_decorate_down.xpm")
-	    "Use row below as border (M-t b down)")
+	    "Use row below as border")
    (table-row-decoration #t)))
 
-(menu-bind table-icons
-  |
-  (=> (balloon (icon "tm_table_insert.xpm")
-	       "Insert or delete rows or columns")
-      (link table-insert-icons))
+(tm-menu (focus-extra-icons t)
+  (:require (table-markup-context? t))
+  (glue #f #f 10 0)
+  (assuming #f
+    (=> (balloon (icon "tm_table_insert.xpm")
+	         "Insert or delete rows or columns")
+        (link table-insert-icons)))
   (=> (balloon (icon "tm_table_size.xpm") "Specify the size of the table")
       (group "Width")
       (link table-width-menu)
@@ -446,7 +454,7 @@
       (tile 4 (link table-vpos-icons)))
   (=> (balloon (icon "tm_table_special.xpm") "Set special table properties")
       (link table-special-menu))
-  |
+  (glue #f #f 10 0)
   (if (== (get-cell-mode) "cell")
       (=> (balloon (icon "tm_cell_cell.xpm")
 		   "Change cell operation mode")
