@@ -4,18 +4,42 @@
 * DESCRIPTION: conversion of tex strings into texmacs trees
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include "Tex/convert_tex.hpp"
 #include "scheme.hpp"
 #include "vars.hpp"
+#include "tree_correct.hpp"
+#include "url.hpp"
 
 tree upgrade_tex (tree t);
 static bool textm_appendices= false;
+static bool textm_unicode   = false;
+static bool textm_natbib    = false;
+
+/*
+bool
+check_tree (tree t) {
+  if (is_atomic (t)) {
+    for (int i=0; i<N(t->label); i++)
+      if (((int) t->label[i]) == 0) {
+	cout << "t= " << t << "\n";
+	return true;
+      }
+  }
+  else {
+    for (int i=0; i<N(t); i++)
+      if (check_tree (t[i])) {
+	if (is_atomic (t[i])) cout << "t= " << t << "\n";
+	return true;
+      }
+  }
+  return false;
+}
+*/
 
 /******************************************************************************
 * Preprocess preamble
@@ -26,13 +50,18 @@ filter_preamble (tree t) {
   int i, n=N(t);
   bool in_preamble= true;
   tree r (CONCAT);
-  array<tree> preamble;
+  tree preamble (CONCAT);
+  tree title_info (CONCAT);
 
   for (i=0; i<n; i++) {
     tree u= t[i];
     if (in_preamble) {
       if (u == tuple ("\\begin-document")) {
-	r << u << preamble;
+	r << u;
+	if (N(preamble) > 0)
+	  r << tuple ("\\begin-hide-preamble") << A(preamble)
+	    << tuple ("\\end-hide-preamble");
+	r << A(title_info);
 	in_preamble= false;
       }
       else if (is_tuple (u, "\\documentclass") ||
@@ -40,10 +69,18 @@ filter_preamble (tree t) {
 	       is_tuple (u, "\\documentstyle") ||
 	       is_tuple (u, "\\documentstyle*"))
 	r << u;
-      else if (is_tuple (u, "\\def") || is_tuple (u, "\\def*") ||
-	       is_tuple (u, "\\title") || is_tuple (u, "\\author") ||
+      else if (is_tuple (u, "\\def") ||
+	       is_tuple (u, "\\def*"))
+	preamble << u << "\n" << "\n";
+      else if (is_tuple (u, "\\newtheorem"))
+	preamble << u << "\n" << "\n";
+      else if (is_tuple (u, "\\newenvironment") ||
+	       is_tuple (u, "\\newenvironment*"))
+	preamble << u << "\n" << "\n";
+      else if (is_tuple (u, "\\title") ||
+	       is_tuple (u, "\\author") ||
 	       is_tuple (u, "\\address"))
-	preamble << u;
+	title_info << u;
     }
     else r << u;
   }
@@ -59,6 +96,7 @@ filter_preamble (tree t) {
 #define t2e parsed_text_to_tree
 #define m2e latex_modifier_to_tree
 #define var_m2e var_latex_modifier_to_tree
+#define v2e latex_verbarg_to_string
 tree l2e (tree);
 tree latex_command_to_tree (tree t);
 
@@ -66,23 +104,39 @@ tree
 latex_symbol_to_tree (string s) {
   if (s == "") return "";
   if (s[0] == '\\') {
-    if (latex_type[s] == "command") {
+    if (latex_type (s) == "command") {
       if (s == "\\ ") return " ";
       if (s == "\\-") return "";
       if (s == "\\/") return "";
-      if (s == "\\i") return "\020";
-      if (s == "\\j") return "\021";
-      if (s == "\\oe") return "\367";
-      if (s == "\\ae") return "\346";
-      if (s == "\\ss") return "\377";
-      if (s == "\\OE") return "\327";
-      if (s == "\\AE") return "\306";
-      if (s == "\\SS") return "\337";
+      if (s == "\\AA") return "\xC5";
+      if (s == "\\AE") return "\xC6";
+      if (s == "\\DH") return "\xD0";
+      if (s == "\\L") return "\x8A";
+      if (s == "\\NG") return "\x8D";
+      if (s == "\\O") return "\xD8";
+      if (s == "\\OE") return "\xD7";
+      if (s == "\\S") return "\x9F";
+      if (s == "\\SS") return "\xDF";
+      if (s == "\\TH") return "\xDE";
+      if (s == "\\aa") return "\xE5";
+      if (s == "\\ae") return "\xE6";
+      if (s == "\\dh") return "\xF0";
+      if (s == "\\dj") return "\x9E";
+      if (s == "\\i") return "\x19";
+      if (s == "\\j") return "\x1A";
+      if (s == "\\l") return "\xAA";
+      if (s == "\\ng") return "\xAD";
+      if (s == "\\o") return "\xF8";
+      if (s == "\\oe") return "\xF7";
+      if (s == "\\ss") return "\xFF";
+      if (s == "\\th") return "\xFE";
+      if (s == "\\pounds") return "\xBF";
       if (s == "\\\\") return tree (FORMAT, "next line");
       if (s == "\\cr") return tree (FORMAT, "next line");
       if (s == "\\noindent")  return tree (FORMAT, "no first indentation");
       if (s == "\\linebreak")  return tree (FORMAT, "line break");
       if (s == "\\newline")  return tree (FORMAT, "new line");
+      if (s == "\\nobreak")  return tree (FORMAT, "no line break");
       if (s == "\\nolinebreak")  return tree (FORMAT, "no line break");
       if (s == "\\pagebreak")  return tree (FORMAT, "page break");
       if (s == "\\nopagebreak")  return tree (FORMAT, "no page break after");
@@ -110,13 +164,20 @@ latex_symbol_to_tree (string s) {
       if (s == "\\Vert") return "<||>";
       if (s == "\\notin") return "<nin>";
       if (s == "\\addots") return "<udots>";
+      if (s == "\\dots") return "<ldots>";
+      if (s == "\\infin") return "<infty>";
+      if (s == "\\rang") return "<rangle>";
+      if (s == "\\tableofcontents")
+	return compound ("table-of-contents", "toc", tree (DOCUMENT, ""));
+      if (s == "\\bgroup") return "";
+      if (s == "\\egroup") return "";
     }
 
-    if (latex_type[s] == "texmacs") {
+    if (latex_type (s) == "texmacs") {
       if (s == "\\tmdummy")  return "";
     }
 
-    if ((latex_type[s] == "modifier") && (latex_arity[s] == 0)) {
+    if ((latex_type (s) == "modifier") && (latex_arity (s) == 0)) {
       s= s(1,N(s));
       if (s == "rmfamily") return tree (SET, FONT_FAMILY, "rm");
       if (s == "ttfamily") return tree (SET, FONT_FAMILY, "tt");
@@ -181,16 +242,16 @@ latex_symbol_to_tree (string s) {
       if (s == "pink") return tree (SET, COLOR, "pink");
 
       cerr << "The symbol was " << s << "\n";
-      fatal_error ("unexpected situation", "latex_symbol_to_tree");
+      FAILED ("unexpected situation");
     }
-    if (latex_type[s] == "operator")
+    if (latex_type (s) == "operator")
       return s(1,N(s));
-    if (latex_type[s] == "control") return s(1,N(s));
+    if (latex_type (s) == "control") return s(1,N(s));
     if ((s == "\\ldots") && (command_type ("!mode") != "math")) return "...";
-    if (latex_type[s] == "symbol")  return "<" * s(1,N(s)) * ">";
-    if (latex_type[s] == "big-symbol") {
-      if (s == "\\bignone") return tree (BIG, ".");
-      else if (s(0,4)=="\\big") return tree (BIG, s(4,N(s)));
+    if (s == "\\bignone") return tree (BIG, ".");
+    if (latex_type (s) == "symbol")  return "<" * s(1,N(s)) * ">";
+    if (latex_type (s) == "big-symbol") {
+      if (s(0,4)=="\\big") return tree (BIG, s(4,N(s)));
       else return tree (BIG, s(1,N(s)));
     }
 
@@ -199,6 +260,10 @@ latex_symbol_to_tree (string s) {
     if ((N(s) > 5) && (s(0,5) == "\\end-"))
       return tree (END, s(5,N(s)));
 
+    if (starts (s, "\\#") && s != "\\#") {
+      textm_unicode= true;
+      return "<" * s (1, N(s)) * ">";
+    }
     return tree (APPLY, s(1,N(s)));
   }
   if ((N(s) == 2) && (s[0] == '#') && (s[1] >= '0') && (s[1] <= '9'))
@@ -226,6 +291,28 @@ test_alpha_on_end (tree t) {
   return false;
 }
 
+string
+string_arg (tree t) {
+  if (is_atomic (t)) return t->label;
+  else if (is_concat (t)) {
+    string r;
+    int i, n= N(t);
+    for (i=0; i<n; i++)
+      r << string_arg (t[i]);
+    return r;
+  }
+  else if (is_func (t, RSUB, 1))
+    return "_" * string_arg (t[0]);
+  else if (is_func (t, RSUP, 1))
+    return "^" * string_arg (t[0]);
+  else if (is_func (t, APPLY, 1) && t[0] == "nbsp")
+    return " ";
+  else {
+    //cout << "t= " << t << "\n";
+    return "";
+  }
+}
+
 tree
 latex_concat_to_tree (tree t, bool& new_flag) {
   int i, n=N(t);
@@ -247,7 +334,7 @@ latex_concat_to_tree (tree t, bool& new_flag) {
   for (i=0; i<n; i++) {
     if (is_tuple (t[i]) && (N(t[i])==1)) {
       string s= t[i][0]->label;
-      if (latex_type[s] == "math-environment") {
+      if (latex_type (s) == "math-environment") {
 	if (s(0,4)=="\\end") command_type ("!mode") = "text";
 	else command_type ("!mode") = "math";
       }
@@ -261,23 +348,34 @@ latex_concat_to_tree (tree t, bool& new_flag) {
 
     bool operator_flag=
       is_tuple (t[i]) && (N(t[i])==1) &&
-      (latex_type[t[i][0]->label]=="operator");
+      (latex_type (t[i][0]->label) == "operator");
     bool cc_flag= is_concat (t[i]);
     tree u= (cc_flag? latex_concat_to_tree (t[i], new_flag): l2e (t[i]));
     if (is_atomic (u)) {
       if (u == " ") {
 	if (command_type ["!mode"] == "math") {
 	  if ((i==0) || (!is_tuple (t[i-1])) || (N(t[i-1])!=1) ||
-	      (latex_type[t[i-1][0]->label] != "operator"))
+	      (latex_type (t[i-1][0]->label) != "operator"))
 	    continue;
 	}
 	else {
-	  if ((t[i] != tree (TUPLE, "\\ ")) && (i>0) && (is_tuple (t[i-1]))) {
-	    string s= t[i-1][0]->label;
-	    if ((s[0]=='\\') && (latex_type[s]=="command") &&
-		(s!="\\end-math") && (s!="\\end-displaymath"))
-	      if ((arity(t[i-1])==1) || (s=="\\label"))
+	  if ((t[i] != tree (TUPLE, "\\ "))) {
+	    if (i>0 && is_tuple (t[i-1])) {
+	      string s= t[i-1][0]->label;
+	      if ((s[0] == '\\') && (latex_type (s) == "command") &&
+		  (s!="\\end-math") && (s!="\\end-displaymath"))
+		if ((arity(t[i-1])==1) || (s=="\\label"))
+		  continue;
+	      if (starts (s, "\\begin-") &&
+		  (command_type["!verbatim"] != "true"))
 		continue;
+	    }
+	    if (i+1<N(t) && is_tuple (t[i+1])) {
+	      string s= t[i+1][0]->label;
+	      if (starts (s, "\\end-") &&
+		  (command_type["!verbatim"] != "true"))
+		continue;
+	    }
 	  }
 	}
       }
@@ -328,19 +426,9 @@ var_m2e (tree t, string var, string val) {
 	       tree (RESET, copy (var)));
 }
 
-tree
-latex_verbarg_to_tree (tree t) {
-  t= l2e (t);
-  if (is_concat (t)) {
-    string s;
-    int i, n= N(t);
-    for (i=0; i<n; i++)
-      if (is_atomic (t[i])) s << t[i]->label;
-      else if (is_func (t[i], RSUB, 1) && is_atomic (t[i][0]))
-	s << "_" << t[i][0]->label;
-    return s;
-  }
-  else return "";
+string
+v2e (tree t) {
+  return string_arg (t2e (t, false));
 }
 
 static bool
@@ -362,7 +450,8 @@ is_right_type (tree t) {
 
 static bool
 is_large_delimiter (tree t, int& type) {
-  if ((!is_tuple (t)) || (N(t) != 2) || is_compound (t[0])) return false;
+  if ((!is_tuple (t)) || (N(t) != 2) ||
+      is_compound (t[0]) || is_compound (t[1])) return false;
   string s= t[0]->label;
   if (starts (s, "\\Big")) s= "\\big" * s(4,N(s));
   if (starts (s, "\\bigg")) s= "\\big" * s(5,N(s));
@@ -398,15 +487,62 @@ latex_cite_to_tree (string cite_type, string s) {
   return r;
 }
 
+tree
+latex_index_to_tree (string s) {
+  int i, start, n= N(s);
+  array<tree> a (0);
+  for (start= i= 0; i<n; i++)
+    if (s[i] == '!' && N(a) < 3) {
+      a << tree (s (start, i));
+      start= i+1;
+    }
+  a << tree (s (start, i));
+  if (N(a) == 1) return compound ("index", a);
+  if (N(a) == 2) return compound ("subindex", a);
+  if (N(a) == 3) return compound ("subsubindex", a);
+  return compound ("subsubsubindex", a);
+}
+
+tree
+latex_accent (tree t, string acc) {
+  return tree (WITH, MODE, "math",
+	       tree (WIDE, tree (WITH, MODE, "text", l2e (t)), acc));
+}
+
+tree
+latex_eps_get (tree t, string var) {
+  if (!is_atomic (t)) return "";
+  string s= t->label;
+  int start=0, i, n=N(s);
+  for (i=0; i <= n; i++)
+    if (i == n || s[i] == ',') {
+      string ss= s (start, i);
+      while (starts (ss, " ")) ss= ss (1, N(ss));
+      while (ends (ss, " ")) ss= ss (0, N(ss) - 1);
+      int j, k= N(ss);
+      for (j=0; j<k; j++)
+	if (ss[j] == '=') break;
+      string v= ss (0, j);
+      while (ends (v, " ")) v= v (0, N(v) - 1);
+      if (j < k && v == var) {
+	string val= ss (j+1, N(ss));
+	while (starts (val, " ")) val= val (1, N(val));
+	return val;
+      }
+      start= i+1;
+    }
+  return "";
+}
+
 tree
 latex_command_to_tree (tree t) {
   if (is_tuple (t, "\\def", 2)) {
-    string var= as_string (t[1]);
+    string var= string_arg (t[1]);
     if ((N(var)>0) && (var[0]=='\\')) var= var (1, N(var));
     return tree (ASSIGN, var, tree (FUNC, l2e (t[2])));
   }
   if (is_tuple (t, "\\def*", 3)) {
-    string var= as_string (t[1]);
+    string var= string_arg (t[1]);
     if ((N(var)>0) && (var[0]=='\\')) var= var (1, N(var));
     int i, arity= as_int (l2e(t[2]));
     tree f (FUNC);
@@ -415,6 +551,12 @@ latex_command_to_tree (tree t) {
     return tree (ASSIGN, var, f);
   }
 
+  if (is_tuple (t, "\\newtheorem", 2)) {
+    string var= l2e(t[1])->label;
+    string val= l2e(t[2])->label;
+    return compound ("new-theorem", var, val);
+  }
+
   if (is_tuple (t, "\\newenvironment", 3)) {
     string var= l2e(t[1])->label;
     return tree (ASSIGN, var, tree (ENV, l2e (t[2]), l2e (t[3])));
@@ -429,11 +571,6 @@ latex_command_to_tree (tree t) {
     return tree (ASSIGN, var, e);
   }
 
-  if (is_tuple (t, "\\setcounter", 2)) {
-    tree u= l2e (t[1]);
-    if (is_compound (u)) return "";
-    else return tree (ASSIGN, u->label * "nr", l2e (t[2]));
-  }
   if (is_tuple (t, "\\arabic", 1)) {
     tree u= l2e (t[1]);
     if (is_compound (u)) return "";
@@ -447,6 +584,17 @@ latex_command_to_tree (tree t) {
   if (textm_appendices && is_tuple (t, "\\chapter", 1))
     return tree (APPLY, "appendix", l2e (t[1]));
 
+  if (is_tuple (t, "\\^", 1)) return latex_accent (t[1], "^");
+  if (is_tuple (t, "\\~", 1)) return latex_accent (t[1], "~");
+  if (is_tuple (t, "\\`", 1)) return latex_accent (t[1], "<grave>");
+  if (is_tuple (t, "\\'", 1)) return latex_accent (t[1], "<acute>");
+  if (is_tuple (t, "\\\"", 1)) return latex_accent (t[1], "<ddot>"); // diaeresis
+  if (is_tuple (t, "\\.", 1)) return latex_accent (t[1], "<dot>");
+  if (is_tuple (t, "\\u", 1)) return latex_accent (t[1], "<breve>");
+  if (is_tuple (t, "\\v", 1)) return latex_accent (t[1], "<check>"); // caron
+  if (is_tuple (t, "\\=", 1)) return latex_accent (t[1], "<bar>");   // macron
+
+
   if (is_tuple (t, "\\textrm", 1)) return m2e (t, FONT_FAMILY, "rm");
   if (is_tuple (t, "\\texttt", 1)) return m2e (t, FONT_FAMILY, "tt");
   if (is_tuple (t, "\\textsf", 1)) return m2e (t, FONT_FAMILY, "ss");
@@ -456,6 +604,15 @@ latex_command_to_tree (tree t) {
   if (is_tuple (t, "\\textit", 1)) return m2e (t, FONT_SHAPE, "italic");
   if (is_tuple (t, "\\textsl", 1)) return m2e (t, FONT_SHAPE, "slanted");
   if (is_tuple (t, "\\textsc", 1)) return m2e (t, FONT_SHAPE, "small-caps");
+  if (is_tuple (t, "\\tmtextrm", 1)) return m2e (t, FONT_FAMILY, "rm");
+  if (is_tuple (t, "\\tmtexttt", 1)) return m2e (t, FONT_FAMILY, "tt");
+  if (is_tuple (t, "\\tmtextsf", 1)) return m2e (t, FONT_FAMILY, "ss");
+  if (is_tuple (t, "\\tmtextmd", 1)) return m2e (t, FONT_SERIES, "medium");
+  if (is_tuple (t, "\\tmtextbf", 1)) return m2e (t, FONT_SERIES, "bold");
+  if (is_tuple (t, "\\tmtextup", 1)) return m2e (t, FONT_SHAPE, "right");
+  if (is_tuple (t, "\\tmtextit", 1)) return m2e (t, FONT_SHAPE, "italic");
+  if (is_tuple (t, "\\tmtextsl", 1)) return m2e (t, FONT_SHAPE, "slanted");
+  if (is_tuple (t, "\\tmtextsc", 1)) return m2e (t, FONT_SHAPE, "small-caps");
   if (is_tuple (t, "\\emph", 1))   return m2e (t, FONT_SHAPE, "italic");
   if (is_tuple (t, "\\operatorname", 1))
     return var_m2e (t, MATH_FONT_FAMILY, "rm");
@@ -469,31 +626,33 @@ latex_command_to_tree (tree t) {
   if (is_tuple (t, "\\mathit", 1)) return var_m2e (t, MATH_FONT_FAMILY, "it");
   if (is_tuple (t, "\\mathsl", 1)) return var_m2e (t, MATH_FONT_FAMILY, "sl");
   if (is_tuple (t, "\\mathup", 1)) return var_m2e (t, MATH_FONT_FAMILY, "up");
-  if (is_tuple (t, "\\mathcal", 1))  return m2e (t, MATH_FONT, "cal");
+  if (is_tuple (t, "\\mathcal", 1)) return m2e (t, MATH_FONT, "cal");
   if (is_tuple (t, "\\mathfrak", 1)) return m2e (t, MATH_FONT, "Euler");
-  if (is_tuple (t, "\\mathbb", 1))   return m2e (t, MATH_FONT, "Bbb");
-  if (is_tuple (t, "\\mathbbm", 1))   return m2e (t, MATH_FONT, "Bbb*");
+  if (is_tuple (t, "\\mathbb", 1)) return m2e (t, MATH_FONT, "Bbb");
+  if (is_tuple (t, "\\mathbbm", 1)) return m2e (t, MATH_FONT, "Bbb*");
 
-  if (is_tuple (t, "\\prime", 1))  return tree (RPRIME, as_string (t[1]));
-  if (is_tuple (t, "\\frac", 2))  return tree (FRAC, l2e (t[1]), l2e (t[2]));
+  if (is_tuple (t, "\\prime", 1)) return tree (RPRIME, string_arg (t[1]));
+  if (is_tuple (t, "\\frac", 2)) return tree (FRAC, l2e (t[1]), l2e (t[2]));
+  if (is_tuple (t, "\\atop", 2))
+    return compound ("atop", l2e (t[1]), l2e (t[2]));
   if (is_tuple (t, "\\sqrt", 1))  return tree (SQRT, l2e (t[1]));
   if (is_tuple (t, "\\sqrt*", 2)) return tree (SQRT, l2e (t[2]), l2e (t[1]));
   if (is_tuple (t, "\\<sub>", 1)) return tree (RSUB, l2e (t[1]));
-  if (is_tuple (t, "\\not", 1))   return tree (NEG, l2e (t[1]));
-  if (is_tuple (t, "\\bar", 1))  return tree (WIDE, l2e (t[1]), "<bar>");
-  if (is_tuple (t, "\\hat", 1))  return tree (WIDE, l2e (t[1]), "^");
-  if (is_tuple (t, "\\tilde", 1))  return tree (WIDE, l2e (t[1]), "~");
-  if (is_tuple (t, "\\widehat", 1))  return tree (WIDE, l2e (t[1]), "^");
-  if (is_tuple (t, "\\widetilde", 1))  return tree (WIDE, l2e (t[1]), "~");
-  if (is_tuple (t, "\\dot", 1))  return tree (WIDE, l2e (t[1]), "<dot>");
-  if (is_tuple (t, "\\ddot", 1))  return tree (WIDE, l2e (t[1]), "<ddot>");
-  if (is_tuple (t, "\\dddot", 1))  return tree (WIDE, l2e (t[1]), "<dddot>");
-  if (is_tuple (t, "\\ddddot", 1))  return tree (WIDE, l2e (t[1]), "<ddddot>");
-  if (is_tuple (t, "\\check", 1))  return tree (WIDE, l2e (t[1]), "<check>");
-  if (is_tuple (t, "\\grave", 1))  return tree (WIDE, l2e (t[1]), "<grave>");
-  if (is_tuple (t, "\\acute", 1))  return tree (WIDE, l2e (t[1]), "<acute>");
-  if (is_tuple (t, "\\vec", 1))  return tree (WIDE, l2e (t[1]), "<vect>");
-  if (is_tuple (t, "\\breve", 1))  return tree (WIDE, l2e (t[1]), "<breve>");
+  if (is_tuple (t, "\\not", 1)) return tree (NEG, l2e (t[1]));
+  if (is_tuple (t, "\\bar", 1)) return tree (WIDE, l2e (t[1]), "<bar>");
+  if (is_tuple (t, "\\hat", 1)) return tree (WIDE, l2e (t[1]), "^");
+  if (is_tuple (t, "\\tilde", 1)) return tree (WIDE, l2e (t[1]), "~");
+  if (is_tuple (t, "\\widehat", 1)) return tree (WIDE, l2e (t[1]), "^");
+  if (is_tuple (t, "\\widetilde", 1)) return tree (WIDE, l2e (t[1]), "~");
+  if (is_tuple (t, "\\dot", 1)) return tree (WIDE, l2e (t[1]), "<dot>");
+  if (is_tuple (t, "\\ddot", 1)) return tree (WIDE, l2e (t[1]), "<ddot>");
+  if (is_tuple (t, "\\dddot", 1)) return tree (WIDE, l2e (t[1]), "<dddot>");
+  if (is_tuple (t, "\\ddddot", 1)) return tree (WIDE, l2e (t[1]), "<ddddot>");
+  if (is_tuple (t, "\\check", 1)) return tree (WIDE, l2e (t[1]), "<check>");
+  if (is_tuple (t, "\\grave", 1)) return tree (WIDE, l2e (t[1]), "<grave>");
+  if (is_tuple (t, "\\acute", 1)) return tree (WIDE, l2e (t[1]), "<acute>");
+  if (is_tuple (t, "\\vec", 1)) return tree (WIDE, l2e (t[1]), "<vect>");
+  if (is_tuple (t, "\\breve", 1)) return tree (WIDE, l2e (t[1]), "<breve>");
   if (is_tuple (t, "\\abovering", 1))
     return tree (WIDE, l2e (t[1]), "<abovering>");
   if (is_tuple (t, "\\hspace", 1) || is_tuple (t, "\\hspace*", 1)) {
@@ -504,7 +663,22 @@ latex_command_to_tree (tree t) {
   if (is_tuple (t, "\\vspace", 1) || is_tuple (t, "\\vspace*", 1))
     return tree (VSPACE, t2e (t[1]));
   if (is_tuple (t, "\\label", 1)) return tree (LABEL, t2e (t[1]));
-  if (is_tuple (t, "\\ref", 1))   return tree (REFERENCE, t2e (t[1]));
+  if (is_tuple (t, "\\ref", 1)) return tree (REFERENCE, t2e (t[1]));
+  if (is_tuple (t, "\\newcounter", 1))
+    return compound ("new-counter", v2e (t[1]));
+  if (is_tuple (t, "\\value", 1))
+    return compound ("value-counter", v2e (t[1]));
+  if (is_tuple (t, "\\stepcounter", 1))
+    return compound ("inc-counter", v2e (t[1]));
+  if (is_tuple (t, "\\refstepcounter", 1))
+    return compound ("next-counter", v2e (t[1]));
+  if (is_tuple (t, "\\setcounter", 2)) // FIXME: only reset works
+    return compound ("reset-counter", v2e (t[1]));
+  if (is_tuple (t, "\\addtocounter", 2)) // FIXME: only inc works
+    return compound ("inc-counter", v2e (t[1]));
+  if (is_tuple (t, "\\setlength")) return "";
+  if (is_tuple (t, "\\addtolength")) return "";
+  if (is_tuple (t, "\\enlargethispage")) return "";
   if (is_tuple (t, "\\mathop", 1)) return l2e (t[1]);
   if (is_tuple (t, "\\mathrel", 1)) return l2e (t[1]);
   if (is_tuple (t, "\\overbrace", 1))
@@ -515,6 +689,14 @@ latex_command_to_tree (tree t) {
   if (is_tuple (t, "\\text", 1) ||
       is_tuple (t, "\\mbox", 1) || is_tuple (t, "\\hbox", 1))
     return var_m2e (t, MODE, "text");
+  if (is_tuple (t, "\\ensuremath", 1))
+    return var_m2e (t, MODE, "math");
+  if (is_tuple (t, "\\Mvariable", 1))
+    return compound ("Mvariable", var_m2e (t, MODE, "text"));
+  if (is_tuple (t, "\\Mfunction", 1))
+    return compound ("Mfunction", var_m2e (t, MODE, "text"));
+  if (is_tuple (t, "\\Muserfunction", 1))
+    return compound ("Muserfunction", var_m2e (t, MODE, "text"));
 
   if (is_tuple (t, "\\<sup>", 1)) {
     if (is_tuple (t[1], "\\prime", 0))
@@ -523,6 +705,10 @@ latex_command_to_tree (tree t) {
   }
   if (is_tuple (t, "\\stackrel", 2))
     return tree (ABOVE, l2e (t[2]), l2e (t[1]));
+  if (is_tuple (t, "\\overset", 2))
+    return tree (ABOVE, l2e (t[2]), l2e (t[1]));
+  if (is_tuple (t, "\\underset", 2))
+    return tree (BELOW, l2e (t[2]), l2e (t[1]));
 
   int dtype= 0;
   if (is_large_delimiter (t, dtype)) {
@@ -536,16 +722,51 @@ latex_command_to_tree (tree t) {
   }
   if (is_tuple (t, "\\cite", 1) || is_tuple (t, "\\nocite", 1)) {
     string cite_type= t[0]->label (1, N(t[0]->label));
-    string s= as_string (t2e(t[1]));
+    string s= v2e (t[1]);
     return latex_cite_to_tree (cite_type, s);
   }
-
   if (is_tuple (t, "\\cite*", 2)) {
     tree   ot= t2e(t[1])->label;
-    string s = as_string (t2e(t[2]));
+    string s = v2e (t[2]);
     tree   ct= latex_cite_to_tree ("cite", s);
+    if (N(ct) == 2) return compound ("cite-detail", ct[1], ot);
     return tree (CONCAT, ct, " (", ot, ")");
   }
+  if (is_tuple (t, "\\citedetail", 2))
+    return compound ("cite-detail", l2e (t[1]), l2e (t[2]));
+  if (is_tuple (t, "\\citet", 1) || is_tuple (t, "\\citep", 1) ||
+      is_tuple (t, "\\citet*", 1) || is_tuple (t, "\\citep*", 1) ||
+      is_tuple (t, "\\citealt", 1) || is_tuple (t, "\\citealp", 1) ||
+      is_tuple (t, "\\citealt*", 1) || is_tuple (t, "\\citealp*", 1))
+    {
+      textm_natbib= true;
+      string star= "";
+      string cite_type= t[0]->label (1, N(t[0]->label));
+      if (ends (cite_type, "*")) {
+	star= "*"; cite_type= cite_type (0, N (cite_type) - 1); }
+      if (cite_type == "citet") cite_type= "cite-textual" * star;
+      if (cite_type == "citep") cite_type= "cite-parenthesized" * star;
+      if (cite_type == "citealt") cite_type= "cite-raw" * star;
+      if (cite_type == "citealp") cite_type= "cite-raw" * star;
+      string s= v2e (t[1]);
+      return latex_cite_to_tree (cite_type, s);
+    }
+  if (is_tuple (t, "\\citetext", 1))
+    return compound ("render-cite", l2e (t[1]));
+  if (is_tuple (t, "\\citeauthor", 1)) {
+    textm_natbib= true; return compound ("cite-author-link", t2e (t[1])); }
+  if (is_tuple (t, "\\citeauthor*", 1)) {
+    textm_natbib= true; return compound ("cite-author*-link", t2e (t[1])); }
+  if (is_tuple (t, "\\citeyear", 1)) {
+    textm_natbib= true; return compound ("cite-year-link", t2e (t[1])); }
+  if (is_tuple (t, "\\bibitem", 1))
+    return compound ("bibitem", v2e (t[1]));
+  if (is_tuple (t, "\\bibitem*", 2))
+    return compound ("bibitem*", v2e (t[1]), v2e (t[2]));
+  if (is_tuple (t, "\\index", 1)) {
+    string s= v2e (t[1]);
+    return latex_index_to_tree (s);
+  }
   if (is_tuple (t, "\\displaylines", 1)) {
     tree u= l2e (t[1]);
     return tree (CONCAT, tree (BEGIN, "matrix"), u, tree (END, "matrix"));
@@ -558,14 +779,37 @@ latex_command_to_tree (tree t) {
     return r;
   }
   if (is_tuple (t, "\\includegraphics", 1)) {
-    tree name= latex_verbarg_to_tree (t[1]);
+    tree name= v2e (t[1]);
     if (name == "") return "";
     else {
-      tree g (POSTSCRIPT, 7);
+      tree g (IMAGE, 7);
       g[0]= name;
       return g;
     }
   }
+  if (is_tuple (t, "\\epsfig", 1)) {
+    tree data  = v2e (t[1]);
+    tree name  = latex_eps_get (data, "file");
+    tree width = latex_eps_get (data, "width");
+    tree height= latex_eps_get (data, "height");
+    if (name == "") return "";
+    else {
+      tree g (IMAGE, 5);
+      g[0]= name;
+      g[1]= width;
+      g[2]= height;
+      return g;
+    }
+  }
+  if (is_tuple (t, "\\noalign", 1))
+    return ""; // FIXME: for larger space in maple matrices
+  if (is_tuple (t, "\\etalchar", 1)) return t2e (t[1]);
+  if (is_tuple (t, "\\natexlab", 1)) return t2e (t[1]);
+  if (is_tuple (t, "\\penalty", 1)) return "";
+  if (is_tuple (t, "\\url", 1))
+    return tree (APPLY, "href", t2e (t[1]));
+  if (is_tuple (t, "\\href", 2))
+    return tree (APPLY, "hlink", l2e (t[2]), t2e (t[1]));
 
   // Start TeXmacs specific markup
   if (is_tuple (t, "\\tmmathbf", 1))
@@ -573,7 +817,7 @@ latex_command_to_tree (tree t) {
 		 tree (SET, MATH_FONT_SERIES, "bold"),
 		 l2e (t[1]),
 		 tree (RESET, MATH_FONT_SERIES));
-  if (is_tuple (t, "\\tmop", 1)) return tree (APPLY, "op", l2e (t[1]));
+  if (is_tuple (t, "\\tmop", 1)) return t2e (t[1]);
   if (is_tuple (t, "\\tmstrong", 1)) return tree (APPLY, "strong", l2e (t[1]));
   if (is_tuple (t, "\\tmem", 1)) return tree (APPLY, "em", l2e (t[1]));
   if (is_tuple (t, "\\tmtt", 1)) return tree (APPLY, "tt", l2e (t[1]));
@@ -591,6 +835,14 @@ latex_command_to_tree (tree t) {
     return tree (HLINK, l2e (t[1]), l2e (t[2]));
   if (is_tuple (t, "\\tmaction", 1))
     return tree (ACTION, l2e (t[1]), l2e (t[2]));
+  if (is_tuple (t, "\\foldtext", 2))
+    return compound ("fold-text", l2e (t[1]), l2e (t[2]));
+  if (is_tuple (t, "\\foldproof", 2))
+    return compound ("fold-proof", l2e (t[1]), l2e (t[2]));
+  if (is_tuple (t, "\\foldalgorithm", 2))
+    return compound ("fold-algorithm", l2e (t[1]), l2e (t[2]));
+  if (is_tuple (t, "\\foldexercise", 2))
+    return compound ("fold-exercise", l2e (t[1]), l2e (t[2]));
   // End TeXmacs specific markup
 
   int i;
@@ -606,7 +858,7 @@ latex_command_to_tree (tree t) {
 tree
 l2e (tree t) {
   if (is_atomic (t)) return latex_symbol_to_tree (t->label);
-  if (is_func (t, CONCAT)) {
+  if (L(t) == CONCAT) {
     bool new_flag= false;
     return latex_concat_to_tree (t, new_flag);
   }
@@ -656,7 +908,7 @@ static tree
 parse_matrix_params (tree t) {
   tree tformat (TFORMAT);
   if (N(t) <= 1) return tformat;
-  string s= as_string (t[1]);
+  string s= string_arg (t[1]);
   int i, n= N(s), col=1;
   for (i=0; i<n; i++) {
     switch (s[i]) {
@@ -688,9 +940,9 @@ parse_matrix_params (tree t) {
 }
 
 static void
-parse_pmatrix (tree& r, tree t, int& i, bool bracket) {
+parse_pmatrix (tree& r, tree t, int& i, string lb, string rb) {
   tree tformat= parse_matrix_params (t[i]);
-  if (bracket) r << tree (LEFT, "(");
+  if (lb != "") r << tree (LEFT, lb);
 
   int rows=0, cols=0;
   tree V (CONCAT);
@@ -712,17 +964,27 @@ parse_pmatrix (tree& r, tree t, int& i, bool bracket) {
       continue;
     }
     else if (is_func (v, BEGIN) && ((v[0] == "array") ||(v[0] == "tabular"))) {
-      parse_pmatrix (E, t, i, false);
+      parse_pmatrix (E, t, i, "", "");
       if (i<N(t)) continue;
       break;
     }
     else if (v == tree (BEGIN, "matrix")) {
-      parse_pmatrix (E, t, i, false);
+      parse_pmatrix (E, t, i, "", "");
       if (i<N(t)) continue;
       break;
     }
     else if (v == tree (BEGIN, "pmatrix")) {
-      parse_pmatrix (E, t, i, true);
+      parse_pmatrix (E, t, i, "(", ")");
+      if (i<N(t)) continue;
+      break;
+    }
+    else if (v == tree (BEGIN, "bmatrix")) {
+      parse_pmatrix (E, t, i, "{", "}");
+      if (i<N(t)) continue;
+      break;
+    }
+    else if (v == tree (BEGIN, "vmatrix")) {
+      parse_pmatrix (E, t, i, "|", "|");
       if (i<N(t)) continue;
       break;
     }
@@ -730,6 +992,8 @@ parse_pmatrix (tree& r, tree t, int& i, bool bracket) {
     else if (v == tree (END, "tabular")) break;
     else if (v == tree (END, "matrix")) break;
     else if (v == tree (END, "pmatrix")) break;
+    else if (v == tree (END, "bmatrix")) break;
+    else if (v == tree (END, "vmatrix")) break;
     else if (v == tree (APPLY, "hline")) {
       int    row  = N(V)+ (N(L)==0? 0: 1);
       string row_s= row==0? as_string (row+1): as_string (row);
@@ -762,7 +1026,7 @@ parse_pmatrix (tree& r, tree t, int& i, bool bracket) {
     M= tformat;
   }
   r << M;
-  if (bracket) r << tree (RIGHT, ")");
+  if (rb != "") r << tree (RIGHT, rb);
 }
 
 static tree
@@ -775,10 +1039,12 @@ finalize_pmatrix (tree t) {
     tree r (CONCAT);
     for (i=0; i<n; i++)
       if (is_func (u[i], BEGIN)) {
-	if (u[i][0] == "array") parse_pmatrix (r, u, i, false);
-	else if (u[i][0] == "tabular") parse_pmatrix (r, u, i, false);
-	else if (u[i][0] == "matrix") parse_pmatrix (r, u, i, false);
-	else if (u[i][0] == "pmatrix") parse_pmatrix (r, u, i, true);
+	if (u[i][0] == "array") parse_pmatrix (r, u, i, "", "");
+	else if (u[i][0] == "tabular") parse_pmatrix (r, u, i, "", "");
+	else if (u[i][0] == "matrix") parse_pmatrix (r, u, i, "", "");
+	else if (u[i][0] == "pmatrix") parse_pmatrix (r, u, i, "(", ")");
+	else if (u[i][0] == "bmatrix") parse_pmatrix (r, u, i, "[", "]");
+	else if (u[i][0] == "vmatrix") parse_pmatrix (r, u, i, "|", "|");
 	else r << u[i];
       }
       else r << u[i];
@@ -813,9 +1079,9 @@ space_eater (tree t) {
 static bool
 admissible_env (tree t) {
   string s= t[0]->label;
-  if (latex_type["\\begin-" * s] == "list") return true;
-  if (latex_type["\\begin-" * s] == "environment") return true;
-  if (latex_type["\\begin-" * s] == "math-environment") return true;
+  if (latex_type ("\\begin-" * s) == "list") return true;
+  if (latex_type ("\\begin-" * s) == "environment") return true;
+  if (latex_type ("\\begin-" * s) == "math-environment") return true;
   return false;
 }
 
@@ -855,7 +1121,7 @@ finalize_layout (tree t) {
 
       if (is_func (v, BEGIN, 1) && (v[0] == "picture")) {
 	for (; i<n; i++)
-	  if (is_func (u[i], POSTSCRIPT)) r << u[i];
+	  if (is_func (u[i], IMAGE)) r << u[i];
 	  else if (is_func (u[i], END, 1) && (u[i][0] == "picture"))
 	    break;
 	continue;
@@ -881,6 +1147,18 @@ finalize_layout (tree t) {
 	continue;
       }
 
+      /*
+      if (is_func (v, BEGIN) && (v[0] == "hide-preamble")) {
+	r << tree (BEGIN, "hide-preamble");
+	continue;
+      }
+
+      if (is_func (v, END) && (v[0] == "hide-preamble")) {
+	r << tree (END, "hide-preamble");
+	continue;
+      }
+      */
+
       if (is_func (v, BEGIN, 1) && admissible_env (v)) {
 	if (v == tree (BEGIN, "verbatim")) {
 	  r << v; i++;
@@ -912,7 +1190,7 @@ finalize_layout (tree t) {
 	insert_return (r);
 	r << tree (BEGIN, translate_list (v[0]->label));
 	spc_flag = true;
-	item_flag= (latex_type ["\\begin-" * v[0]->label] == "list");
+	item_flag= (latex_type ("\\begin-" * v[0]->label) == "list");
 	continue;
       }
 
@@ -986,7 +1264,7 @@ is_preamble_command (tree t, tree& doc, string& style) {
     if (t[0] == "usepackage") return true;
     if ((t[0] == "documentstyle") ||
 	(t[0] == "documentclass")) {
-      style= t[1]->label;
+      style= string_arg (t[1]);
       return true;
     }
   }
@@ -994,7 +1272,7 @@ is_preamble_command (tree t, tree& doc, string& style) {
     if (t[0] == "usepackage*") return true;
     if ((t[0] == "documentstyle*") ||
 	(t[0] == "documentclass*")) {
-      style= t[2]->label;
+      style= string_arg (t[2]);
       return true;
     }
   }
@@ -1065,6 +1343,75 @@ handle_improper_matches (tree& r, tree t, int& pos) {
   }
 }
 
+tree
+remove_env_spaces (tree t, bool& done) {
+  if (is_atomic (t)) {
+    done= done || (t != "" && t != " ");
+    return t;
+  }
+  else if (is_func (t, CONCAT)) {
+    array<tree> r;
+    for (int i=0; i<N(t); i++)
+      if (!done && t[i] == "");
+      else if (!done && t[i] == " ");
+      else r << remove_env_spaces (t[i], done);
+    return simplify_concat (tree (CONCAT, r));
+  }
+  else if (is_func (t, WITH)) {
+    tree r= t (0, N(t));
+    r[N(t)-1]= remove_env_spaces (r[N(t)-1], done);
+    return r;
+  }
+  else return t;
+}
+
+tree
+env_hacks (tree t) {
+  int i, n= N(t);
+  bool done= false;
+  tree beg= remove_env_spaces (t[n-2], done);
+  tree end= remove_env_spaces (t[n-1], done);
+  if (beg == "") beg= tree (CONCAT);
+  else if (!is_concat (beg)) beg= tree (CONCAT, beg);
+  if (end == "") end= tree (CONCAT);
+  else if (!is_concat (end)) end= tree (CONCAT, end);
+  array<tree> ba;
+  array<tree> ea;
+  for (i=0; i<N(beg); i++)
+    if (is_func (beg[i], VSPACE))
+      ba << tree (VAR_VSPACE, A(beg[i]));
+    else if (is_func (beg[i], PAGE_BREAK))
+      ba << tree (VAR_PAGE_BREAK, A(beg[i]));
+    else if (is_func (beg[i], NO_PAGE_BREAK))
+      ba << tree (VAR_NO_PAGE_BREAK, A(beg[i]));
+    else if (is_func (beg[i], NEW_PAGE))
+      ba << tree (VAR_NEW_PAGE, A(beg[i]));
+    else ba << beg[i];
+  for (i=0; i<N(end); i++)
+    if (is_func (end[i], NO_INDENT))
+      ea << tree (VAR_NO_INDENT, A(end[i]));
+    else if (is_func (end[i], YES_INDENT))
+      ea << tree (VAR_YES_INDENT, A(end[i]));
+    else ea << end[i];
+  beg= tree (CONCAT, ba);
+  end= tree (CONCAT, ea);
+  for (i=N(beg); i>0 && is_func (beg[i-1], RESET, 1); i--) ;
+  bool ok= (i<<1) >= N(beg);
+  for (int k=0; k<N(beg)-i; k++) {
+    ok= ok && is_func (beg[i-k-1], SET, 2) && beg[i-k-1][0] == beg[i+k][0];
+    //cout << "Matched " << beg[i-k-1] << " and " << beg[i+k] << "\n";
+  }
+  if (ok && i<N(beg)) {
+    tree r= t (0, n);
+    r[n-2]= simplify_concat (beg (0, i));
+    r[n-1]= simplify_concat (beg (i, N(beg)) * end);
+    //cout << "<< " << t << "\n";
+    //cout << ">> " << r << "\n";
+    t= r;
+  }
+  return t;
+}
+
 tree
 handle_improper_matches (tree t) {
   if (is_atomic (t)) return t;
@@ -1082,6 +1429,8 @@ handle_improper_matches (tree t) {
       if (N(u)==1) return u[0];
       return u;
     }
+    else if (is_func (r, ENV) && N(r) >= 2)
+      return env_hacks (r);
     return r;
   }
 }
@@ -1157,10 +1506,17 @@ finalize_textm (tree t) {
 
 tree
 latex_to_tree (tree t1) {
-  string style;
+  string style, lan= "";
   bool is_document= is_compound (t1, "!file", 1);
   if (is_document) t1= t1[0];
+  if (is_compound (t1, "!language", 2)) {
+    lan= t1[1]->label;
+    t1 = t1[0];
+  }
   textm_appendices= false;
+  textm_unicode   = false;
+  textm_natbib    = false;
+  command_type ("!em") = "false";
   // cout << "\n\nt1= " << t1 << "\n\n";
   tree t2= is_document? filter_preamble (t1): t1;
   // cout << "\n\nt2= " << t2 << "\n\n";
@@ -1171,7 +1527,7 @@ latex_to_tree (tree t1) {
   tree t5= is_document? finalize_preamble (t4, style): t4;
   // cout << "\n\nt5= " << t5 << "\n\n";
   tree t6= handle_improper_matches (t5);
-  // cout << "\n\nt6= " << t6 << "\n\n";
+  //cout << "\n\nt6= " << t6 << "\n\n";
   if ((!is_document) && is_func (t6, DOCUMENT, 1)) t6= t6[0];
   tree t7= upgrade_tex (t6);
   // cout << "\n\nt7= " << t7 << "\n\n";
@@ -1179,9 +1535,53 @@ latex_to_tree (tree t1) {
   // cout << "\n\nt8= " << t8 << "\n\n";
   tree t9= finalize_textm (t8);
   // cout << "\n\nt9= " << t9 << "\n\n";
-  tree t10= simplify_correct (t9);
+  tree t10= drd_correct (std_drd, t9);
   // cout << "\n\nt10= " << t10 << "\n\n";
-  if (is_document)
-    return tree (DOCUMENT, compound ("body", t10), compound ("style", style));
-  else return t10;
+  tree t11= simplify_correct (t10);
+  //cout << "\n\nt11= " << t11 << "\n\n";
+  tree t12= latex_correct (t11);
+  //cout << "\n\nt12= " << t12 << "\n\n";
+
+  if (!exists (url ("$TEXMACS_STYLE_PATH", style * ".ts")))
+    style= "generic";
+  tree initial (COLLECTION), mods (WITH);
+  if (textm_unicode) {
+    string name= "modern";
+    if (lan == "chinese") name= "fireflysung";
+    //if (lan == "japanese") name= "ipa";
+    //if (lan == "korean") name= "unbatang";
+    if (lan == "taiwanese") name= "fireflysung";
+    initial << tree (ASSOCIATE, FONT, name);
+    mods << tree (FONT) << tree (name);
+  }
+  if (lan != "") {
+    initial << tree (ASSOCIATE, LANGUAGE, lan);
+    mods << tree (LANGUAGE) << tree (lan);
+  }
+  if (is_document) {
+    tree the_body   = compound ("body", t12);
+    tree the_style  = compound ("style", style);
+    tree the_initial= compound ("initial", initial);
+    if (textm_natbib)
+      the_style= compound ("style", tuple (style, "cite-author-year"));
+    if (N (initial) == 0) return tree (DOCUMENT, the_style, the_body);
+    else return tree (DOCUMENT, the_style, the_body, the_initial);
+  }
+  else {
+    if (N (mods) > 0) { mods << t10; return mods; }
+    return t10;
+  }
+}
+
+tree
+latex_document_to_tree (string s) {
+  command_type ->extend ();
+  command_arity->extend ();
+  command_def  ->extend ();
+  tree t= parse_latex_document (s, true);
+  tree r= latex_to_tree (t);
+  command_type ->shorten ();
+  command_arity->shorten ();
+  command_def  ->shorten ();
+  return r;
 }
