@@ -4,10 +4,9 @@
 * DESCRIPTION: upgrade old TeXmacs formats
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include "convert.hpp"
@@ -16,6 +15,11 @@
 #include "vars.hpp"
 #include "drd_std.hpp"
 #include <stdio.h>
+#include "Scheme/object.hpp"
+#include "tree_correct.hpp"
+
+static bool upgrade_tex_flag= false;
+double get_magnification (string s);
 
 /******************************************************************************
 * Retrieve older operator hashmap
@@ -37,6 +41,25 @@ get_codes (string version) {
   hashmap<string,int> H (UNKNOWN);
   H->join (STD_CODE);
 
+  if (version_inf ("1.0.7.6", version)) return H;
+
+  rename_feature (H, "group", "rigid");
+  rename_feature (H, "postscript", "image");
+
+  if (version_inf ("1.0.6.9", version)) return H;
+
+  rename_feature (H, "frozen", "freeze");
+
+  if (version_inf ("1.0.6.2", version)) return H;
+
+  new_feature (H, "expand-as");
+  new_feature (H, "locus");
+  new_feature (H, "id");
+  new_feature (H, "hard-id");
+  new_feature (H, "link");
+  new_feature (H, "url");
+  new_feature (H, "script");
+
   if (version_inf ("1.0.4.1", version)) return H;
 
   new_feature (H, "copy");
@@ -395,7 +418,7 @@ upgrade_textual (tree t, path& mode_stack) {
 	else s << ss;
       }
       else if (((t->label[i] == '\'') || (t->label[i] == '`')) &&
-	       (!nil (mode_stack)) && (mode_stack->item == 1))
+	       (!is_nil (mode_stack)) && (mode_stack->item == 1))
 	{
 	  int start= i++;
 	  while ((i<n) && (t->label[i] == t->label[i-1])) i++;
@@ -421,7 +444,7 @@ upgrade_textual (tree t, path& mode_stack) {
       }
       else if (is_func (u, RESET)) {
 	if (u == tree (RESET, "mode"))
-	  if (!nil (mode_stack))
+	  if (!is_nil (mode_stack))
 	    mode_stack= mode_stack->next;
       }
       else if (is_func (u, BEGIN, 1)) {
@@ -436,7 +459,7 @@ upgrade_textual (tree t, path& mode_stack) {
 	    (u[0] == "equation*") ||
 	    (u[0] == "eqnarray*") ||
 	    (u[0] == "leqnarray*"))
-	  if (!nil (mode_stack))
+	  if (!is_nil (mode_stack))
 	    mode_stack= mode_stack->next;
       }
       if (is_concat (t) && is_concat (u)) r << A(u);
@@ -467,7 +490,7 @@ upgrade_apply_expand_value (tree t, hashset<string> H) {
   }
 }
 
-typedef char* charp;
+typedef const char* charp;
 static charp apply_expand_value_strings[]= {
   "part", "part*", "chapter", "chapter*", "appendix",
   "section", "section*", "subsection", "subsection*",
@@ -802,6 +825,8 @@ static tree
 upgrade_set_begin_surround (tree t, tree search, bool& found) {
   if (t == search) {
     found= true;
+    if (upgrade_tex_flag)
+      return tree (DOCUMENT, copy (t));
     return copy (t);
   }
   if (is_func (t, WITH) || is_func (t, EXPAND)) {
@@ -850,7 +875,7 @@ upgrade_env_args (tree t, tree env) {
 
 static tree
 upgrade_set_begin_env (tree t) {
-  // cout << "in  : " << t << "\n";
+  //cout << "in  : " << t << "\n";
   int i, n= N(t);
   tree u (MACRO, n);
   for (i=0; i<n-2; i++)
@@ -865,15 +890,15 @@ upgrade_set_begin_env (tree t) {
   else if (!is_concat (begin)) begin= tree (CONCAT, begin);
   if (end == "") end= tree (CONCAT);
   else if (!is_concat (end)) end= tree (CONCAT, end);
-  body << A (begin) << tree (ARG, copy (s)) << A (end);
-  // cout << "mid1: " << body << "\n";
+  body << A(begin) << tree (ARG, copy (s)) << A(end);
+  //cout << "mid1: " << body << "\n";
   body= upgrade_set_begin_concat (body);
   body= upgrade_env_args (body, t);
-  // cout << "mid2: " << body << "\n";
+  //cout << "mid2: " << body << "\n";
   bool found= false;
   u[n-1]= upgrade_set_begin_surround (body, tree (ARG, s), found);
-  // cout << "out : " << u << "\n";
-  // cout << "-------------------------------------------------------------\n";
+  //cout << "out : " << u << "\n";
+  //cout << "-------------------------------------------------------------\n";
   return u;
 }
 
@@ -924,7 +949,7 @@ expand_needs_surrounding (string s) {
     (s == "maketitle") || (s == "abstract") ||
     (s == "theorem") || (s == "proposition") || (s == "lemma") ||
     (s == "corollary") || (s == "proof") || (s == "axiom") ||
-    (s == "definition") || (s == "conjecture") ||
+    (s == "definition") || (s == "notation") || (s == "conjecture") ||
     (s == "remark") || (s == "note") || (s == "example") ||
     (s == "exercise") || (s == "warning") || (s == "convention") ||
     (s == "code") || (s == "quote") ||
@@ -2370,12 +2395,608 @@ upgrade_doc_info (tree t) {
   return t;
 }
 
+/******************************************************************************
+* Upgrade bibliographies
+******************************************************************************/
+
+tree
+upgrade_bibliography (tree t) {
+  if (is_atomic (t)) return t;
+  else {
+    int i, n= N(t);
+    tree r (t, n);
+    for (i=0; i<n; i++)
+      r[i]= upgrade_bibliography (t[i]);
+    if (is_compound (t, "bibliography") || is_compound (t, "bibliography*")) {
+      int l= N(r)-1;
+      if (is_func (r[l], DOCUMENT, 1) && is_compound (r[l][0], "bib-list"));
+      else if (is_compound (r[l], "bib-list"));
+      else r[l]= tree (DOCUMENT, compound ("bib-list", "[99]", r[l]));
+    }
+    return r;
+  }
+}
+
+/******************************************************************************
+* Upgrade switches
+******************************************************************************/
+
+tree
+upgrade_switch (tree t) {
+  if (is_atomic (t)) return t;
+  else {
+    int i, n= N(t);
+    tree r (t, n);
+    for (i=0; i<n; i++)
+      r[i]= upgrade_switch (t[i]);
+    if (is_compound (r, "switch", 2)) {
+      int i, n= N(r[1]);
+      tree u (make_tree_label ("switch"), n);
+      for (i=0; i<n; i++)
+	if (is_compound (r[1][i], "tmarker", 0)) u[i]= r[0];
+	else u[i]= compound ("hidden", r[1][i]);
+      return u;
+    }
+    if (is_compound (r, "fold", 2))
+      return compound ("folded", r[0], r[1]);
+    if (is_compound (r, "unfold", 2))
+      return compound ("unfolded", r[0], r[1]);
+    if (is_compound (r, "fold-bpr", 2) ||
+	is_compound (r, "fold-text", 2) ||
+	is_compound (r, "fold-proof", 2) ||
+	is_compound (r, "fold-exercise", 2))
+      return compound ("summarized", r[0], r[1]);
+    if (is_compound (r, "unfold-bpr", 2) ||
+	is_compound (r, "unfold-text", 2) ||
+	is_compound (r, "unfold-proof", 2) ||
+	is_compound (r, "unfold-exercise", 2))
+      return compound ("detailed", r[0], r[1]);
+    if (is_compound (r, "fold-algorithm", 2))
+      return compound ("summarized-algorithm", r[0], r[1]);
+    if (is_compound (r, "unfold-algorithm", 2))
+      return compound ("detailed-algorithm", r[0], r[1]);
+    if (is_func (r, ASSIGN, 2) && r[0] == "fold-algorithm")
+      return tree (ASSIGN, "summarized-algorithm", r[1]);
+    if (is_func (r, ASSIGN, 2) && r[0] == "unfold-algorithm")
+      return tree (ASSIGN, "detailed-algorithm", r[1]);
+    return r;
+  }
+}
+
+/******************************************************************************
+* Upgrade graphics
+******************************************************************************/
+
+static int
+find_attr_pos (tree t, string name) {
+  int i, n= N(t);
+  for (i=0; i<n; i+=2)
+    if (t[i] == name && i%2 == 0 && i+1<n) return i;
+  return -1;
+}
+
+static bool
+find_attr (tree t, string name) {
+  return find_attr_pos (t, name) != -1;
+}
+
+static tree
+get_attr (tree t, string name, tree ifnotfound) {
+  int i= find_attr_pos (t, name);
+  return i == -1 ? ifnotfound : t[i+1];
+}
+
+static tree
+add_attr (tree t, string name, tree value) {
+  int i, n= N(t);
+  tree r (t, n+2);
+  for (i=0; i<n-1; i++)
+    r[i]= t[i];
+  r[n-1]= name;
+  r[n]= value;
+  r[n+1]= t[n-1];
+  return r;
+}
+
+static tree
+set_attr (tree t, string name, tree value) {
+  int i= find_attr_pos (t, name);
+  if (i != -1)
+    t[i+1]= value;
+  else
+    t= add_attr (t, name, value);
+  return t;
+}
+
+static tree
+remove_attr (tree t, string name) {
+  int i= find_attr_pos (t, name);
+  if (i == -1)
+    return t;
+  else {
+    int j, n= N(t);
+    tree r (t, n-2);
+    for (j=0; j<i; j++)
+      r[j]= t[j];
+    for (j=i+2; j<n; j++)
+      r[j-2]= t[j];
+    return r;
+  }
+}
+
+tree
+upgrade_fill (tree t) {
+  int i;
+  if (is_atomic (t)) return t;
+  if (is_compound (t, "with")) {
+    if ((i= find_attr_pos (t, "gr-mode")) != -1
+     && is_tuple (t[i+1],"edit-prop"))
+      t[i+1]= tuple ("group-edit","props");
+
+    tree fm= get_attr (t, "fill-mode", tree ("none"));
+    t= remove_attr (t, "fill-mode");
+    t= remove_attr (t, "gr-fill-mode");
+    if (fm == "none")
+      t= remove_attr (t, "fill-color");
+    if (fm == "inside")
+      t= set_attr (t, "color", tree ("none"));
+  }
+  int n= N(t);
+  tree r (t, n);
+  for (i=0; i<n; i++)
+    r[i]= upgrade_fill (t[i]);
+  return r;
+}
+
+static void
+length_split (string &num, string &unit, string l) {
+  int i, n= N(l), nu= 0;
+  i= n-1;
+  while (i>=0 && is_alpha (l[i])) i--, nu++;
+  num= l (0, n-nu);
+  unit= l (n-nu, n);
+}
+
+static string
+length_minus (string l) {
+  if (l[0] == '+') l[0]= '-';
+  else
+  if (l[0] == '-') l[0]= '+';
+  else
+    l= "-" * l;
+  return l;
+}
+
+static string
+length_abs (string l) {
+  if (l[0] == '-') l[0]= '+';
+  return l;
+}
+
+static int length_add_error;
+
+static string
+length_add (string l1, string l2) {
+  length_add_error= 0;
+  string n1, u1, n2, u2;
+  length_split (n1, u1, l1);
+  length_split (n2, u2, l2);
+  double i1= as_double (n1);
+  double i2= as_double (n2);
+  if (i1 == 0.0) {
+    i1= i2;
+    i2= 0.0;
+    string u= u1;
+    u1= u2;
+    u2= u;
+  }
+  if (u1 == "par" && (u2 == "par" || i2 == 0.0))
+    return as_string (i1 + i2) * "par";
+  else
+  if ((u1 == "cm" || u1 == "mm")
+   && (u2 == "cm" || u2 == "mm" || i2 == 0.0)) {
+    if (u1 == "cm" && u2 == "mm") i2 /= 10;
+    if (u1 == "mm" && u2 == "cm") i2 *= 10;
+    return as_string (i1 + i2) * u1;
+  }
+  else {
+    length_add_error= 1;
+    return "0cm";
+  }
+}
+
+tree
+upgrade_graphics (tree t) {
+  int i;
+  if (is_atomic (t)) return t;
+  if (is_compound (t, "with") &&
+      (find_attr (t, "gr-frame") || find_attr (t, "gr-clip"))) {
+    tree fr= get_attr (t, "gr-frame",
+			  tuple ("scale", "1cm",
+				 tree (TUPLE, "0.5par", "0cm")));
+    tree clip= get_attr (t, "gr-clip",
+			    tuple ("clip",
+				   tuple ("0par", "-0.3par"),
+				   tuple ("1par", "0.3par")));
+    t= remove_attr (t, "gr-clip");
+
+    string ox= as_string (fr[2][0]), oy= as_string (fr[2][1]);
+    string cg= as_string (clip[1][0]), cb= as_string (clip[1][1]);
+    string cd= as_string (clip[2][0]), ch= as_string (clip[2][1]);
+    string w= length_add (cd, length_minus (cg));
+    string h= length_add (ch, length_minus (cb));
+
+    fr[2][0]= tree (length_add (length_abs (cg) , ox));
+    if (length_add_error)
+      fr[2][0]= tree (PLUS, length_abs (cg) , ox);
+    fr[2][1]= tree (length_add (length_abs (cb) , oy));
+    if (length_add_error)
+      fr[2][1]= tree (PLUS, length_abs (cb) , oy);
+    tree geom= tuple ("geometry", tree (w), tree (h));
+    t= add_attr (t, "gr-geometry", geom);
+    t= set_attr (t, "gr-frame", fr);
+  }
+  int n= N(t);
+  tree r (t, n);
+  for (i=0; i<n; i++)
+    r[i]= upgrade_graphics (t[i]);
+  return r;
+}
+
+tree
+upgrade_textat (tree t) {
+  int i;
+  if (is_atomic (t)) return t;
+  if (is_compound (t, "text-at") && N(t) == 4) {
+    tree t0= t;
+    t= tree (WITH, tree (TEXT_AT, t[0], t[1]));
+    t= set_attr (t, "text-at-halign", t0[2]);
+    t= set_attr (t, "text-at-valign", t0[3]);
+  }
+  int n= N(t);
+  tree r (t, n);
+  for (i=0; i<n; i++)
+    r[i]= upgrade_textat (t[i]);
+  return r;
+}
+
+/******************************************************************************
+* Upgrade cell alignment
+******************************************************************************/
+
+tree
+upgrade_cell_alignment (tree t) {
+  int i;
+  if (is_atomic (t)) return t;
+  if (is_func (t, CWITH) && (N(t) >= 2))
+    if (t[N(t)-2] == CELL_HALIGN)
+      if (t[N(t)-1] == "." || t[N(t)-1] == ",") {
+	tree r= copy (t);
+	r[N(t)-1]= "L" * t[N(t)-1]->label;
+	return r;
+      }
+  int n= N(t);
+  tree r (t, n);
+  for (i=0; i<n; i++)
+    r[i]= upgrade_cell_alignment (t[i]);
+  return r;
+}
+
+/******************************************************************************
+* Renaming primitives
+******************************************************************************/
+
+tree
+rename_primitive (tree t, string which, string by) {
+  int i;
+  if (is_atomic (t)) return t;
+  int n= N(t);
+  tree r (t, n);
+  if (is_compound (t, which))
+    r= tree (make_tree_label (by), n);
+  for (i=0; i<n; i++)
+    r[i]= rename_primitive (t[i], which, by);
+  return r;
+}
+
+/******************************************************************************
+* Upgrade label assignment
+******************************************************************************/
+
+tree
+upgrade_label_assignment (tree t) {
+  int i;
+  if (is_atomic (t)) return t;
+  else if (is_func (t, ASSIGN, 2) && t[0] == "the-label")
+    return tree (SET_BINDING, t[1]);
+  else {
+    int n= N(t);
+    tree r (t, n);
+    for (i=0; i<n; i++)
+      r[i]= upgrade_label_assignment (t[i]);
+    return r;
+  }
+}
+
+/******************************************************************************
+* Upgrade scheme documentation
+******************************************************************************/
+
+tree
+upgrade_scheme_doc (tree t) {
+  int i;
+  if (is_atomic (t)) return t;
+  else if (is_compound (t, "scm-fun", 1) ||
+	   is_compound (t, "scm-macro", 1))
+    return compound ("scm", t[0]);
+  else if (is_compound (t, "explain-scm-fun") ||
+	   is_compound (t, "explain-scm-macro"))
+    {
+      tree r (CONCAT);
+      r << "(" << t[0];
+      for (int i=1; i<N(t); i++)
+	r << " " << t[i];
+      r << ")";
+      return compound ("scm", simplify_concat (r));
+    }
+  else {
+    int n= N(t);
+    tree r (t, n);
+    for (i=0; i<n; i++)
+      r[i]= upgrade_scheme_doc (t[i]);
+    return r;
+  }
+}
+
+/******************************************************************************
+* Upgrade Mathemagix tag
+******************************************************************************/
+
+tree
+upgrade_mmx (tree t) {
+  int i;
+  if (is_atomic (t)) return t;
+  else if (is_compound (t, "mmx", 0) || t == tree (VALUE, "mmx"))
+    return compound ("mathemagix");
+  else if (is_compound (t, "mml", 0) || t == tree (VALUE, "mml"))
+    return compound ("mmxlib");
+  else if (is_compound (t, "scheme", 0) || t == tree (VALUE, "scheme"))
+    return compound ("scheme");
+  else if (is_compound (t, "cpp", 0) || t == tree (VALUE, "cpp"))
+    return compound ("c++");
+  else if (is_compound (t, "scheme-code", 1))
+    return compound ("scm", upgrade_mmx (t[0]));
+  else if (is_compound (t, "scheme-fragment", 1))
+    return compound ("scm-fragment", upgrade_mmx (t[0]));
+  else if (is_compound (t, "cpp-code", 1))
+    return compound ("cpp", upgrade_mmx (t[0]));
+  else {
+    int n= N(t);
+    tree r (t, n);
+    for (i=0; i<n; i++)
+      r[i]= upgrade_mmx (t[i]);
+    return r;
+  }
+}
+
+/******************************************************************************
+* Upgrade sessions
+******************************************************************************/
+
+static tree
+upgrade_session (tree t, tree lan, tree ses) {
+  if (is_atomic (t)) return t;
+  else if (is_compound (t, "session", 1))
+    return compound ("session", copy (lan), copy (ses),
+		     upgrade_session (t[0], lan, ses));
+  else if (is_func (t, WITH, 5) &&
+	   t[0] == PROG_LANGUAGE &&
+	   t[2] == PROG_SESSION)
+    return upgrade_session (t[4], t[1], t[3]);
+  else {
+    int i, n= N(t);
+    tree r (L(t));
+    for (i=0; i<n; i++) {
+      if (is_document (t) && is_compound (t[i], "input", 2)) {
+	bool m = is_compound (t[i][1], "math", 1);
+	tree in= (m? t[i][1][0]: t[i][1]);
+	if ((i+1)<n && is_compound (t[i+1], "output", 1)) {
+	  const char* op= (m? "unfolded-io-math": "unfolded-io");
+	  r << compound (op, t[i][0], in, t[i+1][0]);
+	  i++;
+	}
+	else {
+	  const char* op= (m? "input-math": "input");
+	  r << compound (op, t[i][0], in);
+	}
+      }
+      else r << upgrade_session (t[i], lan, ses);
+    }
+    return r;
+  }
+}
+
+/******************************************************************************
+* Upgrade presentation style
+******************************************************************************/
+
+tree
+upgrade_presentation (tree t) {
+  int i;
+  if (is_atomic (t)) return t;
+  else if (is_compound (t, "style") || is_compound (t, "tuple")) {
+    int n= N(t);
+    tree r (t, n);
+    for (i=0; i<n; i++)
+      if (t[i] == "presentation") r[i]= "presentation-ridged-paper";
+      else r[i]= upgrade_presentation (t[i]);
+    return r;
+  }
+  else {
+    int n= N(t);
+    tree r (t, n);
+    for (i=0; i<n; i++)
+      r[i]= upgrade_presentation (t[i]);
+    return r;
+  }
+}
+
+/******************************************************************************
+* Upgrade mathematical formulas
+******************************************************************************/
+
+bool
+is_non_style_document (tree doc) {
+  tree style= extract (doc, "style");
+  if (!is_tuple (style) || N(style) == 0 || !is_atomic (style[0]))
+    return false;
+  for (int i=0; i<N(style); i++)
+    if (style[i] == "source") return false;
+  string ms= style[0]->label;
+  return
+    (starts (ms, "tm")) ||
+    (starts (ms, "lycee")) ||
+    (ms == "article") ||
+    (ms == "beamer") ||
+    (ms == "book") ||
+    (ms == "exam") ||
+    (ms == "generic") ||
+    (ms == "letter") ||
+    (ms == "seminar") ||
+    (ms == "bibliography") ||
+    (ms == "browser") ||
+    (ms == "help") ||
+    (ms == "manual") ||
+    (ms == "mmxdoc") ||
+    (ms == "elsart") ||
+    (ms == "ifac") ||
+    (ms == "jsc") ||
+    (ms == "acmconf") ||
+    (ms == "svjour") ||
+    (ms == "svmono");
+}
+
+tree
+upgrade_math (tree t) {
+  int i;
+  if (is_atomic (t)) return t;
+  else if (is_func (t, WITH, 3) && t[0] == MODE && t[1] == "math")
+    return compound ("math", t[2]);
+  else if (is_func (t, WITH, 3) && t[0] == MODE && t[1] == "text")
+    return compound ("text", t[2]);
+  else {
+    int n= N(t);
+    tree r (t, n);
+    for (i=0; i<n; i++)
+      r[i]= upgrade_math (t[i]);
+    return r;
+  }
+}
+
+/******************************************************************************
+* Upgrade resize and clipped
+******************************************************************************/
+
+tree
+upgrade_resize_arg (tree t) {
+  if (is_func (t, MERGE, 2)) {
+    if (is_atomic (t[0]) && is_atomic (t[1]))
+      return upgrade_resize_arg (t[0]->label * t[1]->label);
+    tree u= upgrade_resize_arg (t[0]);
+    if (is_func (u, PLUS, 2) || is_func (u, MINUS, 2) ||
+	is_func (u, MINIMUM, 2) || is_func (u, MAXIMUM, 2))
+      if (u[1] == "") {
+	u[1]= upgrade_resize_arg (t[1]);
+	return u;
+      }
+    cout << "TeXmacs] warning, resize argument " << t << " not upgraded\n";
+    return t;
+  }
+  if (is_func (t, ARG, 1) ||
+      is_func (t, PLUS, 2) || is_func (t, MINUS, 2) ||
+      is_func (t, MINIMUM, 2) || is_func (t, MAXIMUM, 2))
+    return t;
+  if (!is_atomic (t)) {
+    cout << "TeXmacs] warning, resize argument " << t << " not upgraded\n";
+    return t;
+  }
+  string s= t->label;
+  if (starts (s, "c")) {
+    cout << "TeXmacs] warning, resize argument " << t << " not upgraded\n";
+    return t;
+  }
+  if (N(s) < 2) return t;
+  if (s[0] != 'l' && s[0] != 'b' && s[0] != 'r' && s[0] != 't') return t;
+  string s1= "1" * s (0, 1);
+  string s2= s (2, N(s));
+  if (s[1] == '+') return tree (PLUS, s1, s2);
+  if (s[1] == '-') return tree (MINUS, s1, s2);
+  if (s[1] == '[') return tree (MINIMUM, s1, s2);
+  if (s[1] == ']') return tree (MAXIMUM, s1, s2);
+  return t;
+}
+
+tree
+upgrade_resize_clipped (tree t) {
+  if (is_atomic (t)) return t;
+  else if (N(t) >= 5 && (is_func (t, RESIZE) || is_func (t, CLIPPED))) {
+    if (is_func (t, CLIPPED))
+      t= tree (CLIPPED, t[4], t[0], t[1], t[2], t[3]);
+    int i, n= N(t);
+    tree r (t, n);
+    r[0]= upgrade_resize_clipped (t[0]);
+    for (i=1; i<n; i++)
+      r[i]= upgrade_resize_arg (t[i]);
+    return r;
+  }
+  else {
+    int i, n= N(t);
+    tree r (t, n);
+    for (i=0; i<n; i++)
+      r[i]= upgrade_resize_clipped (t[i]);
+    return r;
+  }
+}
+
+/******************************************************************************
+* Upgrade images
+******************************************************************************/
+
+tree
+upgrade_image_length (tree t, string unit) {
+  if (!is_atomic (t)) return t;
+  string s= t->label;
+  if (starts (s, "*") || starts (s, "/")) {
+    double mag= get_magnification (s);
+    return as_string (mag) * unit;
+  }
+  else return t;
+}
+
+tree
+upgrade_image (tree t) {
+  if (is_atomic (t)) return t;
+  else if (is_func (t, IMAGE, 7))
+    return tree (IMAGE, t[0],
+		 upgrade_image_length (t[1], "w"),
+		 upgrade_image_length (t[2], "h"),
+		 "", "");
+  else {
+    int i, n= N(t);
+    tree r (t, n);
+    for (i=0; i<n; i++)
+      r[i]= upgrade_image (t[i]);
+    return r;
+  }
+}
+
 /******************************************************************************
 * Upgrade from previous versions
 ******************************************************************************/
 
 tree
 upgrade_tex (tree t) {
+  upgrade_tex_flag= true;
   t= upgrade_apply_expand_value (t);
   t= upgrade_new_environments (t);
   t= upgrade_items (t);
@@ -2397,6 +3018,19 @@ upgrade_tex (tree t) {
   t= upgrade_item_punct (t);
   t= substitute (t, tree (VALUE, "hrule"), compound ("hrule"));
   t= upgrade_doc_info (t);
+  t= upgrade_bibliography (t);
+  t= upgrade_math (t);
+  t= upgrade_resize_clipped (t);
+  if (call ("get-preference", "matching brackets") == object ("on"))
+    t= upgrade_brackets (t);
+  t= upgrade_image (t);
+  upgrade_tex_flag= false;
+  return t;
+}
+
+tree
+upgrade_mathml (tree t) {
+  t= upgrade_brackets (t, "math");
   return t;
 }
 
@@ -2462,5 +3096,37 @@ upgrade (tree t, string version) {
     t= substitute (t, tree (VALUE, "hrule"), compound ("hrule"));
     t= upgrade_doc_info (t);
   }
+  if (version_inf_eq (version, "1.0.4.6"))
+    t= upgrade_bibliography (t);
+  if (version_inf_eq (version, "1.0.5.4"))
+    t= upgrade_switch (t);
+  if (version_inf_eq (version, "1.0.5.7"))
+    t= upgrade_fill (t);
+  if (version_inf_eq (version, "1.0.5.8"))
+    t= upgrade_graphics (t);
+  if (version_inf_eq (version, "1.0.5.11"))
+    t= upgrade_textat (t);
+  if (version_inf_eq (version, "1.0.6.1"))
+    t= upgrade_cell_alignment (t);
+  if (version_inf_eq (version, "1.0.6.2"))
+    t= rename_primitive (t, "hyper-link", "hlink");
+  if (version_inf_eq (version, "1.0.6.2"))
+    t= upgrade_label_assignment (t);
+  if (version_inf_eq (version, "1.0.6.10"))
+    t= upgrade_scheme_doc (t);
+  if (version_inf_eq (version, "1.0.6.14"))
+    t= upgrade_mmx (t);
+  if (version_inf_eq (version, "1.0.7.1"))
+    t= upgrade_session (t, "scheme", "default");
+  if (version_inf_eq (version, "1.0.7.6"))
+    t= upgrade_presentation (t);
+  if (version_inf_eq (version, "1.0.7.6") && is_non_style_document (t))
+    t= upgrade_math (t);
+  if (version_inf_eq (version, "1.0.7.7"))
+    t= upgrade_resize_clipped (t);
+  if (version_inf_eq (version, "1.0.7.7"))
+    t= upgrade_image (t);
+  if (is_non_style_document (t))
+    t= automatic_correct (t, version);
   return t;
 }
