@@ -4,10 +4,9 @@
 * DESCRIPTION: routines for the editor
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include "edit_main.hpp"
@@ -21,6 +20,21 @@
 #include "drd_std.hpp"
 #include "message.hpp"
 #include <setjmp.h>
+#ifdef EXPERIMENTAL
+#include "../../Style/Memorizer/clean_copy.hpp"
+#endif
+
+#ifdef PDF_RENDERER
+#include "Pdf/pdf_renderer.hpp"
+#endif
+
+#ifdef USE_GS
+#include "Ghostscript/gs_utilities.hpp"
+#endif
+
+#ifdef QTTEXMACS
+#include "Qt/qt_utilities.hpp"
+#endif
 
 /******************************************************************************
 * Constructors and destructor
@@ -36,13 +50,19 @@ editor_rep::editor_rep (server_rep* sv2, tm_buffer buf2):
   et (the_et), rp (buf2->rp) {}
 
 edit_main_rep::edit_main_rep (server_rep* sv, tm_buffer buf):
-  editor_rep (sv, buf), props (UNKNOWN)
+  editor_rep (sv, buf), props (UNKNOWN), ed_obs (edit_observer (this))
 {
+#ifdef EXPERIMENTAL
+  cct= copy (subtree (et, rp));
+  copy_ip (subtree (et, rp), cct);
+#endif
+  attach_observer (subtree (et, rp), ed_obs);
   notify_change (THE_TREE);
   tp= correct_cursor (et, rp * 0);
 }
 
 edit_main_rep::~edit_main_rep () {
+  detach_observer (subtree (et, rp), ed_obs);
 #ifdef EXPERIMENTAL
   mem= memorizer ();
 #endif
@@ -50,7 +70,7 @@ edit_main_rep::~edit_main_rep () {
 
 editor
 new_editor (server_rep* sv, tm_buffer buf) {
-  return new edit_main_rep (sv, buf);
+  return tm_new<edit_main_rep> (sv, buf);
 }
 
 /******************************************************************************
@@ -145,8 +165,15 @@ void
 edit_main_rep::print (url name, bool conform, int first, int last) {
   bool pdf= (suffix (name) == "pdf");
   url orig= resolve (name, "");
-  if (pdf) name= url_temp (".ps");
 
+#ifndef PDF_RENDERER
+#ifdef USE_GS
+  if (pdf) name= url_temp (".ps");
+#endif
+#else
+  (void) pdf;  // avoid annoying warning
+#endif
+  
   string medium = env->get_string (PAGE_MEDIUM);
   if (conform && (medium != "paper")) conform= false;
   // FIXME: better command for conform printing
@@ -190,8 +217,15 @@ edit_main_rep::print (url name, bool conform, int first, int last) {
   // Print pages
 
   int i;
-  renderer ren=
-    printer (name, dpi, pages, page_type, landsc, w/cm, h/cm);
+  
+#ifdef PDF_RENDERER
+  renderer ren = (pdf ? 
+    pdf_renderer (name, dpi, pages, page_type, landsc, w/cm, h/cm) :
+    printer (name, dpi, pages, page_type, landsc, w/cm, h/cm) );
+#else
+  renderer ren = printer (name, dpi, pages, page_type, landsc, w/cm, h/cm);
+#endif
+  
   for (i=start; i<end; i++) {
     tree bg= env->read (BG_COLOR);
     ren->set_background_pattern (bg);
@@ -204,12 +238,16 @@ edit_main_rep::print (url name, bool conform, int first, int last) {
     the_box[0][i]->redraw (ren, path (0), rs);
     if (i<end-1) ren->next_page ();
   }
-  delete ren;
+  tm_delete (ren);
 
+#ifndef PDF_RENDERER
+#ifdef USE_GS
   if (pdf) {
-    ps2pdf (name, orig);
+    gs_to_pdf (name, orig);
     ::remove (name);
   }
+#endif
+#endif
 }
 
 void
@@ -217,14 +255,90 @@ edit_main_rep::print_to_file (url name, string first, string last) {
   print (name, false, as_int (first), as_int (last));
 }
 
+#ifdef _MBD_EXPERIMENTAL_PRINTER_WIDGET
+#include "qt_printer_widget.hpp" // Needed for PrinterSettings, could be elsewhere
+#endif
+
+/**
+ * In Qt this is the main entry point to the printing subsystem.
+ * the other routines (print_to_file, ...) are overriden since all fine tuning 
+ * is made here via the Qt print dialog
+ */
 void
 edit_main_rep::print_buffer (string first, string last) {
-  url temp= url_temp (".ps");
-  print (temp, false, as_int (first), as_int (last));
-  system (printing_cmd, temp);
-  ::remove (temp);
+  url target= url_temp (".ps");
+  
+#if (defined(_MBD_EXPERIMENTAL_PRINTER_WIDGET) && defined(QTTEXMACS))
+  command dummy;
+  widget wid = printer_widget(dummy, target);
+  
+    // Try to force the dialog to show our preferences. Platform dependent?
+  
+  // TODO: add missing preferences and reduce redundancy with print();
+  PrinterSettings settings;
+  settings.firstPage = as_int(first);
+  settings.lastPage  = as_int(last);
+  settings.dpi       = as_int(printing_dpi);
+  settings.landscape = env->page_landscape;
+  settings.paperSize = env->get_string(PAGE_TYPE);
+  
+  wid->send(SLOT_PRINTER_SETTINGS, close_box<PrinterSettings>(settings));
+  
+    // Show the dialog, read results
+  
+  send_keyboard_focus(wid);
+  settings = open_box<PrinterSettings> (wid->query(SLOT_PRINTER_SETTINGS,
+                                            type_helper<PrinterSettings>::id));
+  if (! settings.accepted) return;
+  
+  env->write(PAGE_TYPE, settings.paperSize);
+  env->page_landscape = settings.landscape;
+  
+    // FIXME?
+    // This is set to 72 dpi by the MacOS dialog and I see no way to change it.
+  //printing_dpi = from_qstring(QString("%1").arg(settings.dpi));
+  
+  // translate from the Qt convention.
+  if (settings.firstPage ==  0)   settings.firstPage = 1;
+  if (settings.lastPage == 0)     settings.lastPage  = 100000;   // ugly!
+
+  if (settings.printToFile) {
+    target = url_general(settings.fileName, URL_SYSTEM);
+    print (target, false, settings.firstPage, settings.lastPage);
+    return;  // No temporary file to remove nor to send to the printer.
+  }
+  print (target, false, settings.firstPage, settings.lastPage);
+#else
+  print (target, false, as_int (first), as_int (last));
+#endif
+
+    // Send the document to the printer
+  system (printing_cmd, target);
+  ::remove (target);
 }
 
+#ifdef THISISTHEPREVIOUSCODE_IJUSTLEFTITHEREINCASE
+void
+edit_main_rep::print_buffer (string first, string last) {
+  // in Qt this is the main entry point to the printing subsystem.
+  // the other routines (print_to_file, ...) are overriden since all fine tuning 
+  // is made here via the Qt print dialog
+  bool to_file, landscape;
+  url name = url_none();
+  string printer;
+  string paper_type;
+  if (qt_print (to_file, landscape, printer, name, first, last, paper_type)) {
+      if (!to_file) name = url_temp (".ps");
+      print (name, false, as_int (first), as_int (last));
+      if (!to_file) {
+        string cmd = printing_cmd * " -P" * printer;
+        system (cmd, name);  
+        ::remove (name);
+      }
+  }
+}
+#endif
+
 void
 edit_main_rep::export_ps (url name, string first, string last) {
   print (name, true, as_int (first), as_int (last));
@@ -235,7 +349,8 @@ edit_main_rep::print_snippet (url name, tree t) {
   bool ps= suffix (name) == "ps" || suffix (name) == "eps";
   typeset_prepare ();
   int dpi= as_int (printing_dpi);
-  if (!ps) t= tree (WITH, MAGNIFICATION, "2", PAGE_WIDTH, "40cm", t);
+  //if (!ps) t= tree (WITH, MAGNIFICATION, "2", PAGE_WIDTH, "40cm", t);
+  if (!ps) t= tree (WITH, MAGNIFICATION, "1.6", PAGE_WIDTH, "40cm", t);
   box b= typeset_as_box (env, t, path ());
   if (b->x4 - b->x3 >= 5*PIXEL && b->y4 - b->y3 >= 5*PIXEL) {
     if (ps) make_eps (name, b, dpi);
@@ -259,7 +374,7 @@ void
 edit_main_rep::footer_eval (string s) {
   // s= unslash (s); // FIXME: dirty fix; should not be necessary
   string r= object_to_string (eval (s));
-  set_message (r, "evaluate expression");
+  set_message (verbatim (r), "evaluate expression");
 }
 
 tree
@@ -293,6 +408,11 @@ edit_main_rep::the_path () {
   return copy (tp);
 }
 
+path
+edit_main_rep::the_shifted_path () {
+  return shift (et, tp, 1);
+}
+
 /******************************************************************************
 * Miscellaneous
 ******************************************************************************/
@@ -337,7 +457,16 @@ void
 edit_main_rep::edit_special () {
 }
 
+#ifdef UNCOMMENTED
+void test_commute ();
+void test_invert ();
+#endif
+
 void
 edit_main_rep::edit_test () {
   cout << "Test !\n";
+#ifdef UNCOMMENTED
+  test_commute();
+  test_invert();
+#endif
 }
