@@ -4,10 +4,9 @@
 * DESCRIPTION: interface between the editor and the window manager
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include "Interface/edit_interface.hpp"
@@ -19,8 +18,8 @@
 #include "data_cache.hpp"
 #include "drd_mode.hpp"
 #include "message.hpp"
+#include "tree_traverse.hpp"
 #ifdef EXPERIMENTAL
-#include "../../Style/Memorizer/clean_copy.hpp"
 #include "../../Style/Evaluate/evaluate_main.hpp"
 #endif
 
@@ -34,7 +33,7 @@ MODE_LANGUAGE (string mode) {
   else if (mode == "prog") return PROG_LANGUAGE;
   else if (mode == "src") return LANGUAGE;
   cerr << "Mode = " << mode << "\n";
-  fatal_error ("invalid mode", "MODE_LANGUAGE", "edit_interface.cpp");
+  FAILED ("invalid mode");
   return LANGUAGE;
 }
 
@@ -47,8 +46,9 @@ edit_interface_rep::edit_interface_rep ():
   last_change (texmacs_time()), last_update (last_change-1),
   do_animate (false), next_animate (last_change-1),
   full_screen (false), got_focus (false),
-  sh_s (""), sh_len (0),
+  sh_s (""), sh_mark (0), pre_edit_s (""), pre_edit_mark (0),
   popup_win (),
+  message_l (""), message_r (""), last_l (""), last_r (""),
   sfactor (sv->get_default_shrinking_factor ()),
   pixel (sfactor*PIXEL), copy_always (),
   last_click (0), last_x (0), last_y (0), dragging (false),
@@ -89,8 +89,9 @@ edit_interface_rep::resume () {
   got_focus= true;
   SERVER (menu_main ("(horizontal (link texmacs-menu))"));
   SERVER (menu_icons (0, "(horizontal (link texmacs-main-icons))"));
-  SERVER (menu_icons (1, "(horizontal (link texmacs-context-icons))"));
-  SERVER (menu_icons (2, "(horizontal (link texmacs-extra-icons))"));
+  SERVER (menu_icons (1, "(horizontal (link texmacs-mode-icons))"));
+  SERVER (menu_icons (2, "(horizontal (link texmacs-focus-icons))"));
+  SERVER (menu_icons (3, "(horizontal (link texmacs-extra-icons))"));
   cur_sb= 2;
   tp= make_cursor_accessible (tp, true);
   notify_change (THE_FOCUS + THE_EXTENTS + THE_CURSOR);
@@ -197,7 +198,7 @@ edit_interface_rep::cursor_visible () {
 	  if (new_xt != old_xt && is_accessible (obtain_ip (old_xt))) {
 	    object fun= symbol_object ("tree-set");
 	    object cmd= list_object (fun, old_xt, new_xt);
-	    eval_delayed (cmd);
+	    exec_delayed (scheme_cmd (cmd));
 	    temp_invalid_cursor= true;
 	  }
 	}
@@ -218,7 +219,7 @@ edit_interface_rep::cursor_visible () {
 	  if (new_yt != old_yt && is_accessible (obtain_ip (old_yt))) {
 	    object fun= symbol_object ("tree-set");
 	    object cmd= list_object (fun, old_yt, new_yt);
-	    eval_delayed (cmd);
+	    exec_delayed (scheme_cmd (cmd));
 	    temp_invalid_cursor= true;
 	  }
 	}
@@ -264,7 +265,6 @@ is_graphical (tree t) {
 
 void
 edit_interface_rep::compute_env_rects (path p, rectangles& rs, bool recurse) {
-  p= path_up (p);
   if (p == rp) return;
   tree st= subtree (et, p);
   if (is_atomic (st) ||
@@ -277,7 +277,7 @@ edit_interface_rep::compute_env_rects (path p, rectangles& rs, bool recurse) {
       (is_func (st, WITH) && is_func (st[N(st)-1], TEXT_AT)) ||
       (is_compound (st, "math", 1) &&
        is_compound (subtree (et, path_up (p)), "input")))
-    compute_env_rects (p, rs, recurse);
+    compute_env_rects (path_up (p), rs, recurse);
   else {
     int new_mode= DRD_ACCESS_NORMAL;
     if (get_init_string (MODE) == "src") new_mode= DRD_ACCESS_SOURCE;
@@ -293,10 +293,11 @@ edit_interface_rep::compute_env_rects (path p, rectangles& rs, bool recurse) {
       if (is_func (st, CELL)) { q1= p1; q2= p2; }
       else selection_correct (et, p1, p2, q1, q2);
       selection sel= eb->find_check_selection (q1, q2);
-      rs << outline (sel->rs, pixel);
+      if (N(focus_get ()) >= N(p))
+        rs << outline (sel->rs, pixel);
     }
     set_access_mode (old_mode);
-    if (recurse) compute_env_rects (p, rs, recurse);
+    if (recurse) compute_env_rects (path_up (p), rs, recurse);
   }
 }
 
@@ -308,6 +309,8 @@ void
 edit_interface_rep::notify_change (int change) {
   env_change= env_change | change;
   needs_update ();
+  if ((change & (THE_TREE | THE_SELECTION | THE_CURSOR)) != 0)
+    manual_focus_set (path (), (change & THE_TREE) != 0);
 }
 
 bool
@@ -338,25 +341,27 @@ edit_interface_rep::apply_changes () {
   //cout << HRULE << "\n";
   if (env_change == 0) {
     if (last_change-last_update > 0 &&
-	idle_time (INTERRUPTED_EVENT) >= 1000/6)
-      {
-	SERVER (menu_main ("(horizontal (link texmacs-menu))"));
-	SERVER (menu_icons (0, "(horizontal (link texmacs-main-icons))"));
-	SERVER (menu_icons (1, "(horizontal (link texmacs-context-icons))"));
-	SERVER (menu_icons (2, "(horizontal (link texmacs-extra-icons))"));
-	set_footer ();
-	if (!get_renderer (this) -> interrupted ()) drd_update ();
-	cache_memorize ();
-	last_update= last_change;
-      }
+        idle_time (INTERRUPTED_EVENT) >= 1000/6)
+    {
+      SERVER (menu_main ("(horizontal (link texmacs-menu))"));
+      SERVER (menu_icons (0, "(horizontal (link texmacs-main-icons))"));
+      SERVER (menu_icons (1, "(horizontal (link texmacs-mode-icons))"));
+      SERVER (menu_icons (2, "(horizontal (link texmacs-focus-icons))"));
+      SERVER (menu_icons (3, "(horizontal (link texmacs-extra-icons))"));
+      set_footer ();
+      if (!get_renderer (this) -> interrupted ()) drd_update ();
+      cache_memorize ();
+      last_update= last_change;
+    }
     return;
   }
-
+  
   // cout << "Applying changes " << env_change << " to " << get_name() << "\n";
-
+  // time_t t1= texmacs_time ();
+  
   // cout << "Always\n";
   update_visible ();
-
+  
   // cout << "Handling automatic resizing\n";
   int sb= 1;
   if (is_attached (this) && get_init_string (PAGE_MEDIUM) == "automatic") {
@@ -374,10 +379,12 @@ edit_interface_rep::apply_changes () {
   }
   if (sb != cur_sb) {
     cur_sb= sb;
-    tm_window win= get_server () -> get_window ();
-    win -> set_scrollbars (sb);
+    if (get_server() -> has_window()) {
+      tm_window win= get_server () -> get_window ();
+      win -> set_scrollbars (sb);
+    }
   }
-
+  
   // window decorations (menu bar, icon bars, footer)
   int wb= 2;
   if (is_attached (this)) {
@@ -388,17 +395,24 @@ edit_interface_rep::apply_changes () {
     if (wb != cur_wb) {
       cur_wb= wb;
       if (wb != 2) {
-	get_server () -> show_header (wb);
-	get_server () -> show_footer (wb);
+        get_server () -> show_header (wb);
+        get_server () -> show_footer (wb);
       }
     }
   }
-
+  
   // cout << "Handling selection\n";
   if (env_change & (THE_TREE+THE_ENVIRONMENT+THE_SELECTION)) {
-    if (made_selection) invalidate (selection_rects);
+    if (made_selection) {
+      invalidate (selection_rects);
+      if (!selection_active_any ()) {
+        made_selection= false;
+        set_selection (tp, tp);
+        selection_rects= rectangles ();
+      }
+    }
   }
-
+  
   // cout << "Handling environment\n";
   if (env_change & THE_ENVIRONMENT)
     typeset_invalidate_all ();
@@ -406,73 +420,95 @@ edit_interface_rep::apply_changes () {
   // cout << "Handling tree\n";
   if (env_change & (THE_TREE+THE_ENVIRONMENT)) {
     typeset_invalidate_env ();
-    if (input_mode == INPUT_NORMAL) {
-      made_selection= false;
-      set_selection (tp, tp);
-    }
-    selection_rects= rectangles ();
     SI x1, y1, x2, y2;
     typeset (x1, y1, x2, y2);
     invalidate (x1- 2*pixel, y1- 2*pixel, x2+ 2*pixel, y2+ 2*pixel);
     // check_data_integrety ();
     the_ghost_cursor()= eb->find_check_cursor (tp);
   }
-
+  
 #ifdef EXPERIMENTAL
   if (env_change & THE_ENVIRONMENT)
     environment_update ();
   if (env_change & THE_TREE) {
-    //global_trace_subtree (rp);
-    tree raw= global_get_subtree (rp);
     cout << HRULE;
-    mem= evaluate (ste, raw);
+    mem= evaluate (ste, cct);
     tree rew= mem->get_tree ();
-    //cout << HRULE;
-    //print_tree (mem);
     cout << HRULE;
     cout << tree_to_texmacs (rew) << LF;
     //print_tree (rew);
   }
 #endif
-
+  
   // cout << "Handling extents\n";
   if (env_change & (THE_TREE+THE_ENVIRONMENT+THE_EXTENTS))
     set_extents (eb->x1, eb->y1, eb->x2, eb->y2);
-
+  
   // cout << "Cursor\n";
   temp_invalid_cursor= false;
   if (env_change & (THE_TREE+THE_ENVIRONMENT+THE_EXTENTS+
-		    THE_CURSOR+THE_FOCUS)) {
+                    THE_CURSOR+THE_SELECTION+THE_FOCUS)) {
     SI /*P1= pixel,*/ P2= 2*pixel, P3= 3*pixel;
     int THE_CURSOR_BAK= env_change & THE_CURSOR;
     go_to_here ();
     env_change= (env_change & (~THE_CURSOR)) | THE_CURSOR_BAK;
     if (env_change & (THE_TREE+THE_ENVIRONMENT+THE_EXTENTS+THE_CURSOR))
       if (!inside_active_graphics ())
-	cursor_visible ();
-
+        cursor_visible ();
+    
     cursor cu= get_cursor();
     rectangle ocr (oc->ox+ ((SI) (oc->y1*oc->slope))- P3, oc->oy+ oc->y1- P3,
-		   oc->ox+ ((SI) (oc->y2*oc->slope))+ P2, oc->oy+ oc->y2+ P3);
+                   oc->ox+ ((SI) (oc->y2*oc->slope))+ P2, oc->oy+ oc->y2+ P3);
     copy_always= rectangles (ocr, copy_always);
     invalidate (ocr->x1, ocr->y1, ocr->x2, ocr->y2);
     rectangle ncr (cu->ox+ ((SI) (cu->y1*cu->slope))- P3, cu->oy+ cu->y1- P3,
-		   cu->ox+ ((SI) (cu->y2*cu->slope))+ P2, cu->oy+ cu->y2+ P3);
+                   cu->ox+ ((SI) (cu->y2*cu->slope))+ P2, cu->oy+ cu->y2+ P3);
     invalidate (ncr->x1, ncr->y1, ncr->x2, ncr->y2);
     copy_always= rectangles (ncr, copy_always);
     oc= copy (cu);
+   
+    // set hot spot in the gui
+    send_cursor (this, (cu->ox-sfactor+1)/sfactor, (cu->oy-sfactor+1)/sfactor);
 
     rectangles old_rects= env_rects;
     env_rects= rectangles ();
-    compute_env_rects (path_up (tp), env_rects, true);
+    path pp= path_up (tp);
+    tree pt= subtree (et, pp);
+    if (none_accessible (pt));
+    else pp= path_up (pp);
+    compute_env_rects (pp, env_rects, true);
     if (env_rects != old_rects) {
       invalidate (old_rects);
       invalidate (env_rects);
     }
     else if (env_change & THE_FOCUS) invalidate (env_rects);
+    
+    old_rects= sem_rects;
+    bool old_correct= sem_correct;
+    sem_rects= rectangles ();
+    sem_correct= true;
+    if (semantic_active (path_up (tp))) {
+      path p1= tp, p2= tp;
+      sem_correct= semantic_select (path_up (tp), p1, p2, 2);
+      if (!sem_correct) {
+        path sr= semantic_root (path_up (tp));
+        p1= start (et, sr);
+        p2= end (et, sr);
+      }
+      path q1, q2;
+      selection_correct (et, p1, p2, q1, q2);
+      selection sel= eb->find_check_selection (q1, q2);
+      sem_rects << outline (sel->rs, pixel);
+    }
+    if (sem_rects != old_rects || sem_correct != old_correct) {
+      invalidate (old_rects);
+      invalidate (sem_rects);
+    }
+    else if (env_change & THE_FOCUS) invalidate (sem_rects);
+    
     invalidate_graphical_object ();
   }
-
+  
   // cout << "Handling selection\n";
   if (env_change & THE_SELECTION) {
     made_selection= selection_active_any ();
@@ -480,15 +516,15 @@ edit_interface_rep::apply_changes () {
       table_selection= selection_active_table ();
       selection sel; selection_get (sel);
       /*
-      selection_rects=
-	simplify (::correct (thicken (sel->rs, pixel, pixel) - sel->rs));
-      */
+       selection_rects=
+       simplify (::correct (thicken (sel->rs, pixel, pixel) - sel->rs));
+       */
       selection_rects= simplify (::correct (thicken (sel->rs, pixel, 3*pixel) -
-					    thicken (sel->rs, 0, 2*pixel)));
+                                            thicken (sel->rs, 0, 2*pixel)));
       invalidate (selection_rects);
     }
   }
-
+  
   // cout << "Handling locus highlighting\n";
   if (env_change & (THE_TREE+THE_ENVIRONMENT+THE_EXTENTS))
     update_active_loci ();
@@ -499,7 +535,7 @@ edit_interface_rep::apply_changes () {
       locus_rects= locus_new_rects;
     }
   }
-
+  
   // cout << "Handling backing store\n";
   if (!is_nil (stored_rects)) {
     if (env_change & (THE_TREE+THE_ENVIRONMENT+THE_SELECTION+THE_EXTENTS))
@@ -510,18 +546,21 @@ edit_interface_rep::apply_changes () {
     if (find_graphical_region (gx1, gy1, gx2, gy2)) {
       rectangle gr= rectangle (gx1, gy1, gx2, gy2);
       if (!is_nil (gr - stored_rects))
-	invalidate (gx1, gy1, gx2, gy2);
+        invalidate (gx1, gy1, gx2, gy2);
     }
   }
-
+  
   // cout << "Handling environment changes\n";
   if (env_change & THE_ENVIRONMENT)
     send_invalidate_all (this);
-
+  
   // cout << "Applied changes\n";
+  // time_t t2= texmacs_time ();
+  // if (t2 - t1 >= 10) cout << "apply_changes took " << t2-t1 << "ms\n";
   env_change  = 0;
   last_change = texmacs_time ();
   last_update = last_change-1;
+  manual_focus_release ();
 }
 
 /******************************************************************************
@@ -554,13 +593,14 @@ edit_interface_rep::full_screen_mode (bool flag) {
 
 void
 edit_interface_rep::before_menu_action () {
-  mark_undo_blocks ();
+  start_editing ();
   set_input_normal ();
 }
 
 void
 edit_interface_rep::after_menu_action () {
   notify_change (THE_DECORATIONS);
+  end_editing ();
 }
 
 /******************************************************************************
