@@ -10,6 +10,7 @@
 ******************************************************************************/
 
 #include "edit_dynamic.hpp"
+#include "tree_analyze.hpp"
 
 /******************************************************************************
 * Constructors and destructors
@@ -53,7 +54,8 @@ edit_dynamic_rep::is_multi_paragraph_macro (tree t) {
   int n= arity (t);
   if (is_document (t) || is_func (t, PARA) || is_func (t, SURROUND))
     return true;
-  if (is_func (t, MACRO) || is_func (t, WITH) || is_func (t, LOCUS) ||
+  if (is_func (t, MACRO) || is_func (t, WITH) ||
+      is_func (t, LOCUS) ||
       is_func (t, CANVAS) || is_func (t, ORNAMENT))
     return is_multi_paragraph_macro (t [n-1]);
   if (is_extension (t) && (!is_compound (t, "footnote"))) {
@@ -82,7 +84,8 @@ contains_table_format (tree t, tree var) {
 
 void
 edit_dynamic_rep::make_compound (tree_label l, int n= -1) {
-  // cout << "Make compound " << as_string (l) << "\n";
+  //cout << "Make compound " << as_string (l) << ", " << n << "\n";
+  eval ("(use-modules (generic generic-edit))");
   if (n == -1) {
     for (n=0; true; n++) {
       if (drd->correct_arity (l, n) &&
@@ -93,7 +96,13 @@ edit_dynamic_rep::make_compound (tree_label l, int n= -1) {
 
   tree t (l, n);
   path p (0, 0);
+  int  acc=0;
+  for (; acc<n; acc++)
+    if (drd->is_accessible_child (t, acc))
+      break;
+  if (acc<n) p->item= acc;
   if (n == 0) insert_tree (t, 1);
+  else if (is_with_like (t) && as_bool (call ("with-like-check-insert", t)));
   else {
     tree f= get_env_value (as_string (l));
     bool block_macro= (N(f) == 2) && is_multi_paragraph_macro (f);
@@ -109,22 +118,22 @@ edit_dynamic_rep::make_compound (tree_label l, int n= -1) {
 	t[0]= tree (DOCUMENT, "");
 	p   = path (0, 0, 0);
       }
-    if ((!drd->all_accessible (l)) && (!in_source ())) {
+    if ((!drd->all_accessible (l)) && get_init_string (MODE) != "src") {
       t= tree (INACTIVE, t);
       p= path (0, p);
     }
     insert_tree (t, p);
     if (table_macro) make_table (1, 1);
     if (sel != "") insert_tree (sel, end (sel));
-
-    string mess;
+    
+    tree mess= concat ();
     if (drd->get_arity_mode (l) != ARITY_NORMAL)
-      mess= "A-right: insert argument";
+      mess= concat (kbd ("A-right"), ": insert argument");
     if (!drd->all_accessible (l)) {
       if (mess != "") mess << ", ";
-      mess << "return: activate";
+      mess << kbd ("return") << ": activate";
     }
-    if (mess == "") mess= "Move to the right when finished";
+    if (mess == concat ()) mess= "Move to the right when finished";
     set_message (mess, drd->get_name (l));
   }
 }
@@ -138,12 +147,16 @@ edit_dynamic_rep::activate () {
   if (is_func (st, COMPOUND) && is_atomic (st[0])) {
     tree u (make_tree_label (st[0]->label));
     u << A (st (1, N(st)));
-    st= u;
+    assign (p, u);
+    go_to (end (et, p));
+    correct (path_up (p));
+  }
+  else {
+    bool acc= (p < path_up (tp) && drd->is_accessible_child (st, tp[N(p)]));
+    remove_node (p * 0);
+    if (!acc) go_to (end (et, p));
+    correct (path_up (p));
   }
-
-  assign (p, st);
-  go_to (end (et, p));
-  correct (path_up (p));
 }
 
 /******************************************************************************
@@ -173,6 +186,10 @@ void
 edit_dynamic_rep::insert_argument (path p, bool forward) {
   tree t= subtree (et, path_up (p));
   int i= last_item (p), n= N(t), d= 1;
+  if (is_func (t, WITH) ||
+      is_func (t, STYLE_WITH) ||
+      is_func (t, VAR_STYLE_WITH))
+    if (i == n-1) i--;
   if ((!in_source ()) || drd->contains (as_string (L(t)))) {
     if (forward) do i++; while ((i<=n) && (!drd->insert_point (L(t), i, n)));
     else while ((i>=0) && (!drd->insert_point (L(t), i, n))) i--;
@@ -196,7 +213,7 @@ edit_dynamic_rep::insert_argument (bool forward) {
 }
 
 void
-edit_dynamic_rep::remove_argument (path p, bool forward) {
+edit_dynamic_rep::remove_empty_argument (path p, bool forward) {
   tree t= subtree (et, path_up (p));
   int i= last_item (p), j, d, n= N(t);
   bool src_flag= in_source () && (!drd->contains (as_string (L(t))));
@@ -259,12 +276,7 @@ edit_dynamic_rep::remove_argument (path p, bool forward) {
 }
 
 void
-edit_dynamic_rep::remove_argument (bool forward) {
-  path p= find_dynamic (tp);
-  if (is_nil (p)) return;
-  if (p == tp) p= find_dynamic (path_up (tp));
-  if (is_nil (p)) return;
-
+edit_dynamic_rep::remove_argument (path p, bool forward) {
   tree t= subtree (et, path_up (p));
   int i= last_item (p), n= N(t), d= 1;
   if ((!in_source ()) || drd->contains (as_string (L(t)))) {
@@ -283,6 +295,15 @@ edit_dynamic_rep::remove_argument (bool forward) {
   go_to_argument (q, forward);
 }
 
+void
+edit_dynamic_rep::remove_argument (bool forward) {
+  path p= find_dynamic (tp);
+  if (is_nil (p)) return;
+  if (p == tp) p= find_dynamic (path_up (tp));
+  if (is_nil (p)) return;
+  remove_argument (p, forward);
+}
+
 /******************************************************************************
 * Backspace and delete
 ******************************************************************************/
@@ -325,7 +346,7 @@ edit_dynamic_rep::back_in_general (tree t, path p, bool forward) {
       go_to_end (p);
       return;
     }
-  remove_argument (p, forward);
+  remove_empty_argument (p, forward);
 }
 
 /******************************************************************************
@@ -478,7 +499,8 @@ edit_dynamic_rep::make_hybrid () {
   path p= end (t, path (0));
   if (in_source ()) insert_tree (t, p);
   else insert_tree (tree (INACTIVE, t), path (0, p));
-  set_message ("return: activate symbol or macro", "hybrid");
+  set_message (concat (kbd ("return"), ": activate symbol or macro"),
+	       "hybrid");
 }
 
 bool
@@ -498,7 +520,8 @@ edit_dynamic_rep::activate_latex () {
       if (N(st) == 2) insert_tree (copy (st[1]));
       return true;
     }
-    set_message ("Error: not a command name", "activate latex command");
+    set_message ("Error: not a command name",
+		 "activate latex command");
   }
   return false;
 }
@@ -551,7 +574,8 @@ edit_dynamic_rep::activate_hybrid (bool with_args_hint) {
     make_compound (make_tree_label (name), with_args_hint? 1: 0);
     if (N(st) == 2) insert_tree (st[1]);
   }
-  else set_message ("Error: unknown command", "activate hybrid command");
+  else set_message ("Error: unknown command",
+		    "activate hybrid command");
   env_locked= old_locked;
 }
 
@@ -580,18 +604,6 @@ edit_dynamic_rep::activate_symbol () {
   correct (path_up (p));
 }
 
-void
-edit_dynamic_rep::activate_compound () {
-  path p= search_upwards (COMPOUND);
-  if (!is_nil (p)) {
-    tree st= subtree (et, p);
-    tree u (make_tree_label (st[0]->label));
-    u << A (st (1, N(st)));
-    assign (p, u);
-    go_to (end (et, p));
-  }
-}
-
 /******************************************************************************
 * Temporary fixes for block structures
 ******************************************************************************/
