@@ -93,9 +93,12 @@ protected:
   virtual void   typeset (SI& x1, SI& y1, SI& x2, SI& y2) = 0;
 
   /* protected subroutines for deletion of content */
+  virtual void back_around (tree t, path p, bool forward) = 0;
   virtual void back_prime (tree t, path p, bool forward) = 0;
+  virtual void back_in_around (tree t, path p, bool forward) = 0;
   virtual void back_in_wide (tree t, path p, bool forward) = 0;
   virtual void back_in_tree (tree t, path p, bool forward) = 0;
+  virtual void pre_remove_around (path p) = 0;
   virtual void back_table (path p, bool forward) = 0;
   virtual void back_in_table (tree t, path p, bool forward) = 0;
   virtual void back_monolithic (path p) = 0;
@@ -104,10 +107,11 @@ protected:
   virtual void back_in_general (tree t, path p, bool forward) = 0;
 
   /* other protected subroutines */
+  virtual tree kbd (string s) = 0;
+  virtual tree kbd_shortcut (string s) = 0;
   virtual path tree_path (path sp, SI x, SI y, SI delta) = 0;
   virtual void apply_changes () = 0;
   virtual void animate () = 0;
-  virtual void correct_concat (path p, int done=0) = 0;
   virtual path search_format (int& row, int& col) = 0;
   virtual void table_bound (path fp, int& i1, int& j1, int& i2, int& j2) = 0;
   virtual tree table_get_subtable (path p, int i1, int j1, int i2, int j2) = 0;
@@ -147,7 +151,8 @@ public:
   virtual bool complete_try () = 0;
   virtual void complete_start (string prefix, array<string> compls) = 0;
   virtual bool complete_keypress (string key) = 0;
-  virtual bool session_complete_try (tree t) = 0;
+  virtual string session_complete_command (tree t) = 0;
+  virtual void custom_complete (tree t) = 0;
   virtual void mouse_any (string s, SI x, SI y, int mods, time_t t) = 0;
   virtual void mouse_click (SI x, SI y) = 0;
   virtual bool mouse_extra_click (SI x, SI y) = 0;
@@ -159,7 +164,7 @@ public:
   virtual cursor get_cursor () = 0;
   virtual void set_pointer (string name) = 0;
   virtual void set_pointer (string curs_name, string mask_name) = 0;
-  virtual void set_message (string l, string r= "", bool temp= false) = 0;
+  virtual void set_message (tree l, tree r= "", bool temp= false) = 0;
   virtual void recall_message () = 0;
 
   /* public routines from edit_cursor */
@@ -293,6 +298,7 @@ public:
 
   /* public routines from edit_text */
   virtual void correct (path p) = 0;
+  virtual void correct_concat (path p, int done=0) = 0;
   virtual bool insert_return () = 0;
   virtual void remove_return (path p) = 0;
   virtual void insert_tree (tree t, path p_in_t) = 0;
@@ -312,15 +318,11 @@ public:
   virtual void make_vspace_after (string s) = 0;
   virtual void make_vspace_after (string smin, string sdef, string smax) = 0;
   virtual void make_htab (string spc) = 0;
-  virtual void make_move (string x, string y) = 0;
-  virtual void make_resize (string x1, string y1, string x2, string y2) = 0;
-  virtual void make_postscript (string file_name, bool link,
-				string w, string h,
-				string x1, string y1,
-				string x2, string y2) = 0;
+  virtual void make_image (string file_name, bool link,
+			   string w, string h, string x, string y) = 0;
 
   /* public routines from edit_math */
-  virtual void make_group () = 0;
+  virtual void make_rigid () = 0;
   virtual void make_lprime (string s) = 0;
   virtual void make_rprime (string s) = 0;
   virtual void make_below () = 0;
@@ -349,7 +351,7 @@ public:
   virtual int    table_which_column () = 0;
   virtual path   table_search_cell (int row, int col) = 0;
   virtual void   table_go_to (int row, int col) = 0;
-  virtual void   table_set_format (string var, string val) = 0;
+  virtual void   table_set_format (string var, tree val) = 0;
   virtual string table_get_format (string var) = 0;
   virtual void   table_del_format (string var= "") = 0;
   virtual void   table_format_center () = 0;
@@ -358,7 +360,7 @@ public:
   virtual void   table_correct_block_content () = 0;
   virtual void   set_cell_mode (string mode) = 0;
   virtual string get_cell_mode () = 0;
-  virtual void   cell_set_format (string var, string val) = 0;
+  virtual void   cell_set_format (string var, tree val) = 0;
   virtual string cell_get_format (string var) = 0;
   virtual void   cell_del_format (string var= "") = 0;
   virtual void   table_test () = 0;
@@ -371,6 +373,7 @@ public:
   virtual void go_to_argument (path p, bool start_flag) = 0;
   virtual void insert_argument (path p, bool forward) = 0;
   virtual void insert_argument (bool forward) = 0;
+  virtual void remove_empty_argument (path p, bool forward) = 0;
   virtual void remove_argument (path p, bool forward) = 0;
   virtual void remove_argument (bool forward) = 0;
   virtual void make_with (string var, string val) = 0;
@@ -382,7 +385,6 @@ public:
   virtual bool activate_latex () = 0;
   virtual void activate_hybrid (bool with_args_hint) = 0;
   virtual void activate_symbol () = 0;
-  virtual void activate_compound () = 0;
   virtual bool make_return_before () = 0;
   virtual bool make_return_after () = 0;
   virtual void temp_proof_fix () = 0;
@@ -396,6 +398,10 @@ public:
   virtual bool get_save_aux () = 0;
 
   /* public routines from edit_select */
+  virtual path semantic_root (path p) = 0;
+  virtual bool semantic_active (path p) = 0;
+  virtual bool semantic_select (path p, path& q1, path& q2, int mode) = 0;
+
   virtual void select (path p) = 0;
   virtual void select (path start, path end) = 0;
   virtual void select_all () = 0;
@@ -426,6 +432,7 @@ public:
   virtual void selection_set (tree t) = 0;
   virtual void selection_set_start (path p= path()) = 0;
   virtual void selection_set_end (path p= path()) = 0;
+  virtual void selection_set_paths (path start, path end) = 0;
   virtual void selection_copy (string key= "primary") = 0;
   virtual void selection_paste (string key= "primary") = 0;
   virtual void selection_clear (string key= "primary") = 0;
@@ -441,6 +448,10 @@ public:
   virtual void selection_move () = 0;
   virtual void cut (path p) = 0;
   virtual void cut (path start, path end) = 0;
+  virtual path manual_focus_get () = 0;
+  virtual void manual_focus_set (path p, bool force= true) = 0;
+  virtual void manual_focus_release () = 0;
+  virtual path focus_get (bool skip_flag= true) = 0;
 
   /* public routines from edit_replace */
   virtual bool inside (string what) = 0;
@@ -487,6 +498,7 @@ public:
   virtual tree the_buffer () = 0;
   virtual tree the_subtree (path p) = 0;
   virtual path the_path () = 0;
+  virtual path the_shifted_path () = 0;
   virtual path the_buffer_path () = 0;
   virtual void show_tree () = 0;
   virtual void show_env () = 0;
