@@ -4,10 +4,9 @@
 * DESCRIPTION: Messages to and between widgets
 * COPYRIGHT  : (C) 2007  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #ifndef MESSAGE_H
@@ -22,7 +21,7 @@
 * Slots should really be defined in message.hpp
 ******************************************************************************/
 
-typedef enum slot_id {
+enum slot_id {
   SLOT_IDENTIFIER,
   SLOT_WINDOW,
   SLOT_RENDERER,
@@ -49,13 +48,16 @@ typedef enum slot_id {
   SLOT_SCROLLBARS_VISIBILITY,
   SLOT_SCROLL_POSITION,
   SLOT_CANVAS,
+  SLOT_CURSOR,
 
   SLOT_HEADER_VISIBILITY,
   SLOT_MAIN_MENU,
   SLOT_MAIN_ICONS_VISIBILITY,
   SLOT_MAIN_ICONS,
-  SLOT_CONTEXT_ICONS_VISIBILITY,
-  SLOT_CONTEXT_ICONS,
+  SLOT_MODE_ICONS_VISIBILITY,
+  SLOT_MODE_ICONS,
+  SLOT_FOCUS_ICONS_VISIBILITY,
+  SLOT_FOCUS_ICONS,
   SLOT_USER_ICONS_VISIBILITY,
   SLOT_USER_ICONS,
   SLOT_FOOTER_VISIBILITY,
@@ -71,23 +73,28 @@ typedef enum slot_id {
   SLOT_INPUT_PROPOSAL,
   SLOT_FILE,
   SLOT_DIRECTORY
+#ifdef _MBD_EXPERIMENTAL_PRINTER_WIDGET
+  , SLOT_PRINTER_SETTINGS
+#endif
 };
 
 class slot {
 public:
-  slot_id id;
-  inline slot (slot_id id2): id (id2) {}
-  inline slot (const slot& s): id (s.id) {}
-  inline slot& operator = (slot s) { id= s.id; return *this; }
-  inline operator slot_id () { return id; }
-  inline bool operator == (slot_id id2) { return id == id2; }
-  inline bool operator != (slot_id id2) { return id != id2; }
-  inline bool operator == (slot s) { return id == s.id; }
-  inline bool operator != (slot s) { return id != s.id; }
-  inline friend ostream& operator << (ostream& out, slot s) {
-    return out << s.id; }
+  slot_id sid;
+  inline slot (slot_id sid2): sid (sid2) {}
+  inline slot (const slot& s): sid (s.sid) {}
+  inline slot& operator = (slot s) { sid= s.sid; return *this; }
+  inline operator slot_id () { return sid; }
+  inline bool operator == (slot_id sid2) { return sid == sid2; }
+  inline bool operator != (slot_id sid2) { return sid != sid2; }
+  inline bool operator == (slot s) { return sid == s.sid; }
+  inline bool operator != (slot s) { return sid != s.sid; }
+  inline friend tm_ostream& operator << (tm_ostream& out, slot s) {
+    return out << s.sid; }
 };
 
+const char * slot_name (const slot s);
+
 //extern bool* slot_state_table;
 //inline bool is_state_slot (slot s) { return slots_state_table[s]; }
 
@@ -119,7 +126,7 @@ send (widget w, slot s, T1 val1, T2 val2, T3 val3) {
 
 template<class T1, class T2, class T3, class T4> void
 send (widget w, slot s, T1 val1, T2 val2, T3 val3, T4 val4) {
-  typedef quadruple<T1,T2,T3,T4> T;
+  typedef quartet<T1,T2,T3,T4> T;
   w->send (s, close_box<T> (T (val1, val2, val3, val4)));
 }
 
@@ -150,7 +157,7 @@ query (widget w, slot s, T1& val1, T2& val2, T3& val3) {
 
 template<class T1, class T2, class T3, class T4> void
 query (widget w, slot s, T1& val1, T2& val2, T3& val3, T4& val4) {
-  typedef quadruple<T1,T2,T3,T4> T;
+  typedef quartet<T1,T2,T3,T4> T;
   T q= open_box<T> (w->query (s, type_helper<T>::id));
   val1= q.x1; val2= q.x2; val3= q.x3; val4= q.x4;
 }
@@ -448,6 +455,12 @@ set_canvas (widget w, widget cv) {
   write (w, SLOT_CANVAS, cv);
 }
 
+inline void
+send_cursor (widget w, SI x1, SI y1) {
+  // send current cursor coordinates
+  send<SI,SI> (w, SLOT_CURSOR, x1, y1);
+}
+
 /******************************************************************************
 * Top-level window related messages (also behaves as a canvas)
 ******************************************************************************/
@@ -489,21 +502,39 @@ set_main_icons (widget w, widget bar) {
 }
 
 inline void
-set_context_icons_visibility (widget w, bool visible) {
-  // set visibility of context icons bar
-  send<bool> (w, SLOT_CONTEXT_ICONS_VISIBILITY, visible);
+set_mode_icons_visibility (widget w, bool visible) {
+  // set visibility of mode dependent icons bar
+  send<bool> (w, SLOT_MODE_ICONS_VISIBILITY, visible);
 }
 
 inline bool
-get_context_icons_visibility (widget w) {
-  // get visibility of context icons bar
-  return query<bool> (w, SLOT_CONTEXT_ICONS_VISIBILITY);
+get_mode_icons_visibility (widget w) {
+  // get visibility of mode dependent icons bar
+  return query<bool> (w, SLOT_MODE_ICONS_VISIBILITY);
 }
 
 inline void
-set_context_icons (widget w, widget bar) {
-  // set context icons bar
-  write (w, SLOT_CONTEXT_ICONS, bar);
+set_mode_icons (widget w, widget bar) {
+  // set mode dependent icons bar
+  write (w, SLOT_MODE_ICONS, bar);
+}
+
+inline void
+set_focus_icons_visibility (widget w, bool visible) {
+  // set visibility of focus dependent icons bar
+  send<bool> (w, SLOT_FOCUS_ICONS_VISIBILITY, visible);
+}
+
+inline bool
+get_focus_icons_visibility (widget w) {
+  // get visibility of focus dependent icons bar
+  return query<bool> (w, SLOT_FOCUS_ICONS_VISIBILITY);
+}
+
+inline void
+set_focus_icons (widget w, widget bar) {
+  // set focus dependent icons bar
+  write (w, SLOT_FOCUS_ICONS, bar);
 }
 
 inline void
