@@ -18,6 +18,7 @@
 #include "iterator.hpp"
 #include "merge_sort.hpp"
 #include "scheme.hpp"
+#include "image_files.hpp"
 
 string PS_CLIP_PUSH ("gsave");
 string PS_CLIP_POP ("grestore");
@@ -59,7 +60,7 @@ printer_rep::printer_rep (
   
   prologue   << "%!PS-Adobe-2.0\n"
 	     << "%%Creator: TeXmacs-" TEXMACS_VERSION "\n"
-	     << "%%Title: " << as_string (ps_file_name) << "\n"
+	     << "%%Title: " << as_string (tail (ps_file_name)) << "\n"
 	     << "%%Pages: " << as_string (nr_pages) << "\n"
 	     << "%%PageOrder: Ascend\n";
   if (page_type != "user")
@@ -328,9 +329,9 @@ printer_rep::make_tex_char (string name, unsigned char c, glyph gl) {
       if ((i<gl->width) && (gl->get_x(i,j)>0)) cur++;
       count++;
       if (count==4) {
-	hex_code << hex_string[cur];
-	cur  = 0;
-	count= 0;
+        hex_code << hex_string[cur];
+        cur  = 0;
+        count= 0;
       }
     }
 
@@ -371,6 +372,80 @@ find_ps_font_name (string name, string s) {
   return name;
 }
 
+
+#define HEX_PER_LINE 30
+
+static SI parse_length (string pfb, int& pos) {
+  QN c4= (QN) pfb[pos++];
+  QN c3= (QN) pfb[pos++];
+  QN c2= (QN) pfb[pos++];
+  QI c1= (QI) pfb[pos++];
+  return (((((((SI) c1)<<8)+ ((SI) c2))<<8)+ ((SI) c3))<<8)+ c4;
+}
+
+static string pfb_to_pfa (url file) {
+  //cout << "pfb_to_pfa :" << file << LF;
+  string pfb, pfa;
+  QN magic, type = 0;
+  SI length;
+  
+  (void) load_string (file, pfb, true);
+  int pos = 0, size = N(pfb);
+  while ((pos < size) && (type != 3)) {
+    parse (pfb, pos, magic);
+    //cout << "magic:" << as_hexadecimal(magic,2) << LF ;
+    if (magic != 128) {
+      FAILED ("Not a pfb file");
+    }
+    parse (pfb, pos, type);
+    //cout << "type:" << as_hexadecimal(type,2) << LF;
+    switch (type) {
+        
+      case 1 :
+        // plain text
+        length = parse_length (pfb, pos);
+        // parse (pfb, pos, length);
+        //cout << "plain text of size " << length << LF;
+        for (int i=0; i <length; i++) {
+          QI ch;
+          parse(pfb, pos, ch);
+          if (ch == '\r') pfa << "\n";
+          else pfa << ch;
+        }
+        break;
+        
+      case 2 :
+        // binary data
+        length = parse_length (pfb, pos);
+        //        parse (pfb, pos, length);
+        //cout << "binary data of size " << length << LF;
+        for (int i=0; i <length; i++) {
+          QI ch;
+          parse(pfb, pos, ch);
+          pfa << as_hexadecimal (ch, 2);
+          if ((i+1) % HEX_PER_LINE == 0) pfa << "\n"; 
+        }
+        break;
+        
+      case 3 :
+        //cout << "end of file"  << LF;
+        // end of file
+        break;
+        
+      default : 
+        FAILED ("Unknown field type while reading PFB file");
+        break;
+        
+    }
+  }
+  return pfa;
+}
+
+#undef HEX_PER_LINE
+
+
+
+
 void
 printer_rep::generate_tex_fonts () {
   hashset<string> done;
@@ -390,10 +465,7 @@ printer_rep::generate_tex_fonts () {
       root= fn_name (0, pos2);
       url u= tt_font_find (root);
       if (suffix (u) == "pfb") {
-	url v= url_temp (".pfa");
-	system ("pfb2pfa", u, v);
-	(void) load_string (v, ttf, true);
-	remove (v);
+        ttf = pfb_to_pfa (u);
       }
     }
 #endif
@@ -410,10 +482,10 @@ printer_rep::generate_tex_fonts () {
       array<string> cum;
       cum << "{}" * as_string (N(a));
       for (i=0; i<N(a); i++) {
-	string w= tex_width [fn_name * "-" * as_string (a[i])];
-	d= (i==0? a[0]: (a[i]-a[i-1]-1));
-	if (d>0) cum << as_string (d) * "[";
-	cum << w * " ";
+        string w= tex_width [fn_name * "-" * as_string (a[i])];
+        d= (i==0? a[0]: (a[i]-a[i-1]-1));
+        if (d>0) cum << as_string (d) * "[";
+        cum << w * " ";
       }
       d= 255-a[i-1];
       if (d>0) cum << as_string (d) * "[";
@@ -428,8 +500,8 @@ printer_rep::generate_tex_fonts () {
       for (i=N(cum)-1; i>=0; i--) fdef << cum[i];
       fdef= "/" * name * " " * fdef * " " * mag * " /" * ttf_name * " rf";
       for (i=0, l=0; i<N(fdef); i++, l++)
-	if ((l<70) || (fdef[i]!=' ')) prologue << fdef[i];
-	else { prologue << '\n'; l=-1; }
+        if ((l<70) || (fdef[i]!=' ')) prologue << fdef[i];
+        else { prologue << '\n'; l=-1; }
       prologue << "\n";
     }
     else {
@@ -437,24 +509,24 @@ printer_rep::generate_tex_fonts () {
 	       << " " << as_string (N(a))
 	       << " " << as_string (a[N(a)-1]+1) << " df\n";
       for (i=0; i<N(a); i++) {
-	int end;
-	string hex_code= tex_chars [fn_name * "-" * as_string (a[i])];
-	for (end=1; end < N(hex_code); end++)
-	  if (hex_code[end-1]=='>') break;
-	string after= hex_code (end, N(hex_code));
-	if ((i>0) && (a[i]==(a[i-1]+1))) after << "I";
-	else after << as_string (a[i]) << " D";
-	if (i==(N(a)-1)) after << " E";
-	hex_code= hex_code (0, end);
+        int end;
+        string hex_code= tex_chars [fn_name * "-" * as_string (a[i])];
+        for (end=1; end < N(hex_code); end++)
+          if (hex_code[end-1]=='>') break;
+        string after= hex_code (end, N(hex_code));
+        if ((i>0) && (a[i]==(a[i-1]+1))) after << "I";
+        else after << as_string (a[i]) << " D";
+        if (i==(N(a)-1)) after << " E";
+        hex_code= hex_code (0, end);
       
-	int j, l, n= N(hex_code);
-	for (j=0; j<n; j+=79) {
-	  if (n < (j+79)) prologue << hex_code (j, n);
-	  else prologue << hex_code (j, j+79) << "\n";
-	}
-	l= 79-(n%79);
-	if (l<N(after)) prologue << "\n";
-	prologue << after << "\n";
+        int j, l, n= N(hex_code);
+        for (j=0; j<n; j+=79) {
+          if (n < (j+79)) prologue << hex_code (j, n);
+          else prologue << hex_code (j, j+79) << "\n";
+        }
+        l= 79-(n%79);
+        if (l<N(after)) prologue << "\n";
+        prologue << after << "\n";
       }
     }
   }
@@ -594,6 +666,7 @@ printer_rep::fill_arc (SI x1, SI y1, SI x2, SI y2, int alpha, int delta) {
 
 void
 printer_rep::polygon (array<SI> x, array<SI> y, bool convex) {
+  (void) convex;
   int i, n= N(x);
   if ((N(y) != n) || (n<1)) return;
   print (x[0], y[0]);
