@@ -21,12 +21,20 @@
   (version-before? version_inf (bool string string))
   (os-win32? os_win32 (bool))
   (os-mingw? os_mingw (bool))
+  (os-macos? os_macos (bool))
   (x-gui? gui_is_x (bool))
   (qt-gui? gui_is_qt (bool))
+  (default-look-and-feel default_look_and_feel (string))
+  (default-chinese-font default_chinese_font_name (string))
+  (default-japanese-font default_japanese_font_name (string))
+  (default-korean-font default_korean_font_name (string))
+  (tm-output tm_output (void string))
+  (tm-errput tm_errput (void string))
   (win32-display win32_display (void string))
   (scheme-dialect scheme_dialect (string))
   (get-texmacs-path get_texmacs_path (string))
   (plugin-list plugin_list (scheme_tree))
+  (set-fast-environments set_fast_environments (void bool))
   (get-font-type get_font_type (int))
   (set-font-type set_font_type (void int))
   (font-exists-in-tt? tt_font_exists (bool string))
@@ -44,12 +52,27 @@
   (math-symbol-type math_symbol_type (string string))
   (object->command as_command (command object))
   (exec-delayed exec_delayed (void object))
+  (exec-delayed-pause exec_delayed_pause (void object))
+  (notify-preferences-loaded notify_preferences_loaded (void))
   (set-input-language set_input_language (void string))
   (get-input-language get_input_language (string))
   (set-output-language gui_set_output_language (void string))
   (get-output-language get_output_language (string))
-  (translate translate (string string string string))
+  (translate translate (string content))
+  (translate-from-to translate (string content string string))
+  (tree-translate tree_translate (tree content))
+  (tree-translate-from-to tree_translate (tree content string string))
   (color named_color (int string))
+  (new-author new_author (double))
+  (set-author set_author (void double))
+  (get-author get_author (double))
+  (debug-set debug_set (void string bool))
+  (debug-get debug_get (bool string))
+  (cout-buffer cout_buffer (void))
+  (cout-unbuffer cout_unbuffer (string))
+
+  ;; routines for images
+  (image->psdoc image_to_psdoc (string url))
 
   ;; routines for trees
   (tree->stree tree_to_scheme_tree (scheme_tree tree))
@@ -76,12 +99,26 @@
   (tree-label-extension? is_extension (bool tree_label))
   (tree-multi-paragraph? is_multi_paragraph (bool tree))
   (tree-simplify simplify_correct (tree tree))
+  (tree-minimal-arity minimal_arity (int tree))
+  (tree-maximal-arity maximal_arity (int tree))
+  (tree-possible-arity? correct_arity (bool tree int))
+  (tree-insert_point insert_point (int tree int))
+  (tree-is-dynamic? is_dynamic (bool tree))
   (tree-accessible-child? is_accessible_child (bool tree int))
   (tree-accessible-children accessible_children (array_tree tree))
+  (tree-all-accessible? all_accessible (bool content))
+  (tree-none-accessible? none_accessible (bool content))
+  (tree-name get_name (string content))
+  (tree-long-name get_long_name (string content))
+  (tree-child-name get_child_name (string content int ))
+  (tree-child-long-name get_child_long_name (string content int))
+  (tree-child-type get_child_type (string content int))
   (tree-load-inclusion load_inclusion (tree url))
-  (tree-as-string var_as_string (string tree))
+  (tree-as-string tree_as_string (string content))
   (tree-extents tree_extents (tree content))
   (tree-empty? is_empty (bool content))
+  (tree-is-buffer? admits_edit_observer (bool tree))
+  (tree-search-sections search_sections (array_tree tree))
 
   (tree-assign tree_assign (tree tree content))
   (tree-var-insert tree_insert (tree tree int content))
@@ -92,6 +129,24 @@
   (tree-insert-node tree_insert_node (tree tree int content))
   (tree-remove-node tree_remove_node (tree tree int))
 
+  (cpp-tree-correct-node correct_node (void tree))
+  (cpp-tree-correct-downwards correct_downwards (void tree))
+  (cpp-tree-correct-upwards correct_upwards (void tree))
+
+  ;; extra routines for content
+  (concat-tokenize-math concat_tokenize (array_tree content))
+  (concat-decompose concat_decompose (array_tree content))
+  (concat-recompose concat_recompose (tree array_tree))
+  (with-like? is_with_like (bool content))
+  (with-same-type? with_same_type (bool content content))
+  (with-similar-type? with_similar_type (bool content content))
+  (with-correct with_correct (tree content))
+  (with-correct-superfluous superfluous_with_correct (tree content))
+  (invisible-correct-superfluous superfluous_invisible_correct (tree content))
+  (invisible-correct-missing missing_invisible_correct (tree content int))
+  (automatic-correct automatic_correct (tree content string))
+  (manual-correct manual_correct (tree content))
+
   ;; paths
   (path-inf? path_inf (bool path path))
   (path-inf-eq? path_inf_eq (bool path path))
@@ -113,6 +168,7 @@
 				   (path content path scheme_tree))
   (path-next-argument next_argument (path content path))
   (path-previous-argument previous_argument (path content path))
+  (path-previous-section previous_section (path content path))
 
   ;; links
   (tree->ids get_ids (list_string tree))
@@ -139,8 +195,6 @@
   (locase-all locase_all (string string))
   (string-union string_union (string string string))
   (string-minus string_minus (string string string))
-  (string-encode tm_encode (string string))
-  (string-decode tm_decode (string string))
   (escape-generic escape_generic (string string))
   (escape-verbatim escape_verbatim (string string))
   (escape-shell escape_sh (string string))
@@ -154,6 +208,30 @@
   (xml-name->tm xml_name_to_tm (string string))
   (old-xml-cdata->tm old_xml_cdata_to_tm (string string))
   (xml-unspace xml_unspace (string string bool bool))
+  (integer->hexadecimal as_hexadecimal (string int))
+  (integer->padded-hexadecimal as_hexadecimal (string int int))
+  (hexadecimal->integer from_hexadecimal (int string))
+
+  ;; routines for strings in the TeXmacs encoding
+  (string->tmstring tm_encode (string string))
+  (tmstring->string tm_decode (string string))
+  (tmstring-length tm_string_length (int string))
+  (tmstring-ref tm_forward_access (string string int))
+  (tmstring-reverse-ref tm_backward_access (string string int))
+  (tmstring->list tm_tokenize (array_string string))
+  (list->tmstring tm_recompose (string array_string))
+  (string-next tm_char_next (int string int))
+  (string-previous tm_char_previous (int string int))
+
+  ;; Packrat grammar and parsing tools
+  (define-grammar-rule define_grammar_rule (void tree tree))
+  (grammar-parse grammar_parse (int tree string))
+  (packrat-define packrat_define (void string string tree))
+  (packrat-property packrat_property (void string string string string))
+  (packrat-inherit packrat_inherit (void string string))
+  (packrat-parse packrat_parse (path string string content))
+  (packrat-correct? packrat_correct (bool string string content))
+  (packrat-context packrat_context (object string string content path))
 
   ;; further conversion routines for trees and strings
   (parse-texmacs texmacs_document_to_tree (tree string))
@@ -169,9 +247,12 @@
   (parse-latex parse_latex (tree string))
   (parse-latex-document parse_latex_document (tree string))
   (latex->texmacs latex_to_tree (tree tree))
+  (latex-document->texmacs latex_document_to_tree (tree string))
   (parse-xml parse_xml (scheme_tree string))
   (parse-html parse_html (scheme_tree string))
+  (parse-bib parse_bib (tree string))
   (upgrade-tmml tmml_upgrade (tree scheme_tree))
+  (upgrade-mathml upgrade_mathml (tree content))
 
   ;; routines for urls
   (string->url url (url string))
@@ -286,6 +367,7 @@
   ;; connections to extern systems
   (connection-start connection_start (string string string))
   (connection-status connection_status (int string string))
+  (connection-write-string connection_write (void string string string))
   (connection-write connection_write (void string string content))
   (connection-cmd connection_cmd (tree string string string))
   (connection-eval connection_eval (tree string string content))
@@ -293,17 +375,48 @@
   (connection-stop connection_stop (void string string))
 
   ;; widgets
+  (widget-printer printer_widget (widget command url))
+  (widget-color-picker color_picker_widget (widget command bool array_tree))
+  (widget-extend extend (widget widget array_widget))
   (widget-hmenu horizontal_menu (widget array_widget))
   (widget-vmenu vertical_menu (widget array_widget))
   (widget-tmenu tile_menu (widget array_widget int))
+  (widget-minibar-menu minibar_menu (widget array_widget))
   (widget-separator menu_separator (widget bool))
-  (widget-menu-group menu_group (widget string string))
+  (widget-menu-group menu_group (widget string int))
   (widget-pulldown-button pulldown_button (widget widget promise_widget))
   (widget-pullright-button pullright_button (widget widget promise_widget))
-  (widget-menu-button menu_button (widget widget command string string bool))
+  (widget-menu-button menu_button (widget widget command string string int))
   (widget-balloon balloon_widget (widget widget widget))
   (widget-empty empty_widget (widget))
-  (widget-text text_widget (widget string int bool string))
+  (widget-text text_widget (widget string int int bool))
+  (widget-input input_text_widget
+		(widget command string array_string int string))
   (widget-xpm xpm_widget (widget url))
   (widget-box box_widget (widget scheme_tree string int bool bool))
-  (object->promise-widget as_promise_widget (promise_widget object)))
+  (widget-glue glue_widget (widget bool bool int int))
+  (widget-color glue_widget (widget content bool bool int int))
+  (widget-hlist horizontal_list (widget array_widget))
+  (widget-vlist vertical_list (widget array_widget))
+  (object->promise-widget as_promise_widget (promise_widget object))
+
+  ;; windows
+  (window-handle window_handle (int))
+  (window-create window_create (void int widget string bool))
+  (window-delete window_delete (void int))
+  (window-show window_show (void int))
+  (window-hide window_hide (void int))
+
+  ;; routines for BibTeX
+  (bib-add-period bib_add_period (scheme_tree scheme_tree))
+  (bib-upcase-first bib_upcase_first (scheme_tree scheme_tree))
+  (bib-locase bib_locase (scheme_tree scheme_tree))
+  (bib-upcase bib_upcase (scheme_tree scheme_tree))
+  (bib-default bib_default (scheme_tree scheme_tree))
+  (bib-purify bib_purify (string scheme_tree))
+  (bib-text-length bib_text_length (int scheme_tree))
+  (bib-prefix bib_prefix (string scheme_tree int))
+  (bib-empty? bib_empty (bool scheme_tree string))
+  (bib-field bib_field (scheme_tree scheme_tree string))
+  (bib-abbreviate bib_abbreviate
+		  (scheme_tree scheme_tree scheme_tree scheme_tree)))
