@@ -23,6 +23,7 @@
 #include "widget.hpp" 
 #include "message.hpp"
 #include "promise.hpp"
+#include "analyze.hpp"
 
 #include "aqua_basic_widgets.h"
 
@@ -104,7 +105,7 @@ aqua_view_widget_rep::send (slot s, blackbox val) {
     break;
   case SLOT_INVALIDATE_ALL:
     {
-      if (!is_nil (val));
+      ASSERT (is_nil (val), "type mismatch");
       [view setNeedsDisplay:YES];
     }
     break;
@@ -124,141 +125,9 @@ aqua_view_widget_rep::send (slot s, blackbox val) {
       if (open_box<bool>(val)) the_keyboard_focus = this;
     }
     break;
-#if 0
-  case SLOT_EXTENTS:
-    {
-      TYPE_CHECK (type_box (val) == type_helper<coord4>::id);
-      coord4 p= open_box<coord4> (val);
-      NSRect rect = to_nsrect(p);
-      //			[[view window] setContentSize:rect.size];
-      [[(NSScrollView*)view documentView] setFrameSize: rect.size];
-    }
-    break;
-    
-  case SLOT_HEADER_VISIBILITY:
-    //			send_bool (THIS, "header", val);
-    break;
-  case SLOT_MAIN_ICONS_VISIBILITY:
-    //			send_bool (THIS, "main icons", val);
-    break;
-  case SLOT_CONTEXT_ICONS_VISIBILITY:
-    //			send_bool (THIS, "context icons", val);
-    break;
-  case SLOT_USER_ICONS_VISIBILITY:
-    //			send_bool (THIS, "user icons", val);
-    break;
-  case SLOT_FOOTER_VISIBILITY:
-    //			send_bool (THIS, "footer flag", val);
-    break;
-    
-  case SLOT_SCROLL_POSITION:
-    {
-      TYPE_CHECK (type_box (val) == type_helper<coord2>::id);
-      coord2 p= open_box<coord2> (val);
-      NSPoint pt = to_nspoint(p);
-      [[(NSScrollView*)view documentView] scrollPoint:pt];
-      //			[[(NSScrollView*)view documentView] scrollRectToVisible:NSMakeRect(pt.x,pt.y,1.0,1.0)];
-    }
-    break;
-  case SLOT_SHRINKING_FACTOR:
-    {
-      TYPE_CHECK (type_box (val) == type_helper<int>::id);
-      //w << set_integer (key, open_box<int> (val));
-      //FIXME: handle sf
-    }
-    break;
-    
-  case SLOT_SIZE:
-    {
-      TYPE_CHECK (type_box (val) == type_helper<coord2>::id);
-      coord2 p= open_box<coord2> (val);
-      NSWindow *win = [view window];
-      if (win) {
-	NSRect frame = [win frame];
-	NSSize s = to_nssize(p);
-	frame.size = s;
-	[win setFrame:frame display:YES];
-      }
-    }
-    break;
-  case SLOT_POSITION:
-    {
-      TYPE_CHECK (type_box (val) == type_helper<coord2>::id);
-      coord2 p= open_box<coord2> (val);
-      NSWindow *win = [view window];
-      if (win) {
-	[win setFrameTopLeftPoint:to_nspoint(p)];
-      }
-    }
-    break;
-  case SLOT_VISIBILITY:
-    {	
-      check_type<bool> (val, "SLOT_VISIBILITY");
-      bool flag = open_box<bool> (val);
-      NSWindow *win = [view window];
-      if (win) {
-	if (flag)
-	  [win makeKeyAndOrderFront:nil] ;
-	else 
-	  [win orderOut:nil]  ;
-      }
-    }	
-    break;
-  case SLOT_IDENTIFIER:
-    check_type<int> (val, "SLOT_IDENTIFIER");
-    THIS << emit_attach_window (get_window (open_box<int> (val)));
-    break;
-  case SLOT_FULL_SCREEN:
-    check_type<bool> (val, "SLOT_FULL_SCREEN");
-    win->set_full_screen (open_box<bool> (val));
-    break;
-  case SLOT_UPDATE:
-    send_update (THIS, val);
-    break;
-  case SLOT_KEYBOARD:
-    send_keyboard (THIS, val);
-    break;
-  case SLOT_MOUSE:
-    send_mouse (THIS, val);
-    break;
-  case SLOT_REPAINT:
-    send_repaint (THIS, val);
-    break;
-  case SLOT_DELAYED_MESSAGE:
-    send_delayed_message (THIS, val);
-    break;
-  case SLOT_DESTROY:
-    send_destroy (THIS, val);
-    break;
-    
-  case SLOT_EXTENTS:
-    send_coord4 (THIS, "extents", val);
-    break;
-  case SLOT_SCROLLBARS_VISIBILITY:
-    send_int (THIS, "scrollbars", val);
-    break;
-    
-  case SLOT_INTERACTIVE_MODE:
-    send_bool (THIS, "interactive mode", val);
-    break;
-    
-  case SLOT_STRING_INPUT:
-    send_string (THIS, "input", val);
-    break;
-  case SLOT_INPUT_TYPE:
-    send_string (THIS, "type", val);
-    break;
-  case SLOT_INPUT_PROPOSAL:
-    send_string (THIS, "default", val);
-    break;
-  case SLOT_FILE:
-    send_string (THIS, "file", val);
-    break;
-  case SLOT_DIRECTORY:
-    send_string (THIS, "directory", val);
-    break;
-#endif			
+	
   default:
+    cout << "slot type= " << slot_name (s) << "\n";
     FAILED ("cannot handle slot type");
   }
 }
@@ -272,19 +141,10 @@ aqua_view_widget_rep::query (slot s, int type_id) {
   switch (s) {
   case SLOT_IDENTIFIER:
     TYPE_CHECK (type_id == type_helper<int>::id);
-    return close_box<int> ((int)[view window] ? 1 : 0);
+    return close_box<int> ([view window] ? 1 : 0);
   case SLOT_RENDERER:
     TYPE_CHECK (type_id == type_helper<renderer>::id);
     return close_box<renderer> ((renderer) the_aqua_renderer());
-#if 0
-  case SLOT_VISIBLE_PART:
-    {
-      TYPE_CHECK (type_id == type_helper<coord4>::id);
-      NSRect rect = [view visibleRect];
-      coord4 c = from_nsrect(rect);
-      return close_box<coord4> (c);
-    }
-#endif
   case SLOT_POSITION:  
     {
       typedef pair<SI,SI> coord2;
@@ -522,6 +382,7 @@ void aqua_tm_widget_rep::layout()
 
 void aqua_tm_widget_rep::updateVisibility()
 {
+  //FIXME: this implementation is from the Qt port. to be adapted.
 #if 0
   mainToolBar->setVisible (visibility[1] && visibility[0]);
   contextToolBar->setVisible (visibility[2] && visibility[0]);
@@ -545,7 +406,7 @@ aqua_tm_widget_rep::send (slot s, blackbox val) {
       NSRect rect = to_nsrect(p);
       NSSize ws = [sv contentSize];
       NSSize sz = rect.size;
-      sz.height = max (sz.height, 7.0 * ws.height / 8.0);
+      sz.height = max (sz.height, ws.height );
       //			[[view window] setContentSize:rect.size];
       [[sv documentView] setFrameSize: sz];
     }
@@ -595,7 +456,7 @@ aqua_tm_widget_rep::send (slot s, blackbox val) {
     {
       TYPE_CHECK (type_box (val) == type_helper<string>::id);
       string msg = open_box<string> (val);
-      [leftField setStringValue:to_nsstring_utf8(aqua_translate(msg))];
+      [leftField setStringValue:to_nsstring_utf8 (tm_var_encode (msg))];
       [leftField displayIfNeeded];
     }
     break;
@@ -603,7 +464,7 @@ aqua_tm_widget_rep::send (slot s, blackbox val) {
     {
       TYPE_CHECK (type_box (val) == type_helper<string>::id);
       string msg = open_box<string> (val);
-      [rightField setStringValue:to_nsstring_utf8(aqua_translate(msg))];
+      [rightField setStringValue:to_nsstring_utf8 (tm_var_encode (msg))];
       [rightField displayIfNeeded];
     }
     break;
@@ -614,12 +475,9 @@ aqua_tm_widget_rep::send (slot s, blackbox val) {
       coord2 p= open_box<coord2> (val);
       NSPoint pt = to_nspoint(p);
       NSSize sz = [[sv contentView] bounds].size;
-      pt.x += sz.width/2;
-      pt.y += sz.height/2;
-      //cout << "scroll position :" << pt.x << "," << pt.y << LF;
-      
+      if (DEBUG_EVENTS) cout << "Scroll position :" << pt.x << "," << pt.y << LF;
       [[sv documentView] scrollPoint:pt];
-      //			[[(NSScrollView*)view documentView] scrollRectToVisible:NSMakeRect(pt.x,pt.y,1.0,1.0)];
+ //     [[sv documentView] scrollRectToVisible:NSMakeRect(pt.x,pt.y,1.0,1.0)];
     }
     break;
     
@@ -632,28 +490,34 @@ aqua_tm_widget_rep::send (slot s, blackbox val) {
     {
       TYPE_CHECK (type_box (val) == type_helper<bool>::id);
       if (open_box<bool>(val) == true) {
-	do_interactive_prompt();
+        //FIXME: to postpone once we return to the runloop
+	    do_interactive_prompt();
       }
     }
     break;
     
   case SLOT_SHRINKING_FACTOR:
-    TYPE_CHECK (type_box (val) == type_helper<int>::id);
-    simple_widget_rep *w = (simple_widget_rep *)[(TMView*)[sv documentView] widget];
-    if (w) {
-      int new_sf = open_box<int> (val);
-      if (DEBUG_EVENTS) cout << "New shrinking factor :" << new_sf << LF;
-      w->handle_set_shrinking_factor (new_sf);
+    {
+      TYPE_CHECK (type_box (val) == type_helper<int>::id);
+      simple_widget_rep *w = (simple_widget_rep *)[(TMView*)[sv documentView] widget];
+      if (w) {
+        int new_sf = open_box<int> (val);
+        if (DEBUG_EVENTS) cout << "New shrinking factor :" << new_sf << LF;
+        w->handle_set_shrinking_factor (new_sf);
+      }
+      break;
     }
-#if 0        
-    if (QTMWidget* tmw= qobject_cast<QTMWidget*> (tm_canvas())) {
-      int new_sf = open_box<int> (val);
-      if (DEBUG_EVENTS) cout << "New shrinking factor :" << new_sf << LF;
-      tmw->tm_widget()->handle_set_shrinking_factor (new_sf);
+
+  case SLOT_FILE:
+    {
+      TYPE_CHECK (type_box (val) == type_helper<string>::id);
+      string file = open_box<string> (val);
+      if (DEBUG_EVENTS) cout << "File: " << file << LF;
+//      view->window()->setWindowFilePath(to_qstring(file));
     }
-#endif
-    break;
-        
+      break;
+      
+      
   default:
     aqua_view_widget_rep::send(s,val);
   }
@@ -665,7 +529,9 @@ aqua_tm_widget_rep::query (slot s, int type_id) {
   case SLOT_SCROLL_POSITION:
     {
       TYPE_CHECK (type_id == type_helper<coord2>::id);
-      NSPoint pt = [[sv contentView] frame].origin;
+      NSPoint pt = [[sv documentView] frame].origin;
+      if (DEBUG_EVENTS)
+        cout << "Position (" << pt.x << "," << pt.y << ")\n"; 
       return close_box<coord2> (from_nspoint(pt));
     }
         
@@ -674,7 +540,11 @@ aqua_tm_widget_rep::query (slot s, int type_id) {
       TYPE_CHECK (type_id == type_helper<coord4>::id);
       NSRect rect= [[sv documentView] frame];
       coord4 c= from_nsrect (rect);
-      //     if (DEBUG_EVENTS) cout << "Canvas geometry " << rect << LF;
+      if (DEBUG_EVENTS) cout << "Canvas geometry (" << rect.origin.x 
+        << "," << rect.origin.y
+        << "," << rect.size.width
+        << "," << rect.size.height
+        << ")" << LF;
       return close_box<coord4> (c);
     }
         
@@ -684,7 +554,11 @@ aqua_tm_widget_rep::query (slot s, int type_id) {
       TYPE_CHECK (type_id == type_helper<coord4>::id);
       NSRect rect= [sv documentVisibleRect];
       coord4 c= from_nsrect (rect);
-      //    if (DEBUG_EVENTS) cout << "Visible Region " << rect << LF;
+      if (DEBUG_EVENTS) cout << "Visible region (" << rect.origin.x 
+        << "," << rect.origin.y
+        << "," << rect.size.width
+        << "," << rect.size.height
+        << ")" << LF;
       return close_box<coord4> (c);
     }
 
@@ -718,7 +592,7 @@ aqua_tm_widget_rep::query (slot s, int type_id) {
   case SLOT_INTERACTIVE_MODE:
     {
       TYPE_CHECK (type_id == type_helper<bool>::id);
-      return close_box<bool> (false);
+      return close_box<bool> (false);  // FIXME: who needs this info?
     }
     
     
@@ -884,8 +758,8 @@ aqua_window_widget_rep::send (slot s, blackbox val) {
       coord2 p= open_box<coord2> (val);
       NSWindow *win = [wc window];
       if (win) {
-	NSSize size = to_nssize(p);
-	[win setContentSize:size];
+        NSSize size = to_nssize(p);
+        [win setContentSize:size];
       }
     }
     break;
@@ -895,7 +769,7 @@ aqua_window_widget_rep::send (slot s, blackbox val) {
       coord2 p= open_box<coord2> (val);
       NSWindow *win = [wc window];
       if (win) { 
-	[win setFrameOrigin:to_nspoint(p)];
+        [win setFrameOrigin:to_nspoint(p)];
       }
     }
     break;
@@ -905,8 +779,8 @@ aqua_window_widget_rep::send (slot s, blackbox val) {
       bool flag = open_box<bool> (val);
       NSWindow *win = [wc window];
       if (win) {
-	if (flag) [win makeKeyAndOrderFront:nil] ;
-	else [win orderOut:nil]  ;
+        if (flag) [win makeKeyAndOrderFront:nil] ;
+        else [win orderOut:nil]  ;
       }
     }	
     break;
@@ -916,13 +790,14 @@ aqua_window_widget_rep::send (slot s, blackbox val) {
       string name = open_box<string> (val);
       NSWindow *win = [wc window];
       if (win) {
-	NSString *title = to_nsstring(name);
-	[win setTitle:title];
+        NSString *title = to_nsstring(name);
+        [win setTitle:title];
       }
     }
     break;
   case SLOT_FULL_SCREEN:
     check_type<bool> (val, "SLOT_FULL_SCREEN");
+    //FIXME: Implement fullscreen mode
     // win->set_full_screen (open_box<bool> (val));
     break;
   case SLOT_UPDATE:
@@ -941,7 +816,7 @@ aqua_window_widget_rep::query (slot s, int type_id) {
   switch (s) {
   case SLOT_IDENTIFIER:
     TYPE_CHECK (type_id == type_helper<int>::id);
-    return close_box<int> ((int)[wc window] ? 1 : 0);
+    return close_box<int> ((intptr_t) [wc window] ? 1 : 0);
   case SLOT_POSITION:  
     {
       typedef pair<SI,SI> coord2;
@@ -983,36 +858,6 @@ aqua_window_widget_rep::read (slot s, blackbox index) {
 void
 aqua_window_widget_rep::write (slot s, blackbox index, widget w) {
   switch (s) {
-#if 0
-  case SLOT_CANVAS:
-    check_type_void (index, "SLOT_CANVAS");
-    //			[(NSScrollView*)view setDocumentView: concrete (w)->get_nsview()];
-    break;
-  case SLOT_MAIN_MENU:
-    check_type_void (index, "SLOT_MAIN_MENU");
-    //			THIS << set_widget ("menu bar", concrete (w));
-    break;
-  case SLOT_MAIN_ICONS:
-    check_type_void (index, "SLOT_MAIN_ICONS");
-    //			THIS << set_widget ("main icons bar", concrete (w));
-    break;
-  case SLOT_CONTEXT_ICONS:
-    check_type_void (index, "SLOT_CONTEXT_ICONS");
-    //			THIS << set_widget ("context icons bar", concrete (w));
-    break;
-  case SLOT_USER_ICONS:
-    check_type_void (index, "SLOT_USER_ICONS");
-    //			THIS << set_widget ("user icons bar", concrete (w));
-    break;
-  case SLOT_INTERACTIVE_PROMPT:
-    check_type_void (index, "SLOT_INTERACTIVE_PROMPT");
-    //			THIS << set_widget ("interactive prompt", concrete (w));
-    break;
-  case SLOT_INTERACTIVE_INPUT:
-    check_type_void (index, "SLOT_INTERACTIVE_INPUT");
-    //			THIS << set_widget ("interactive input", concrete (w));
-    break;
-#endif
   default:
     FAILED ("cannot handle slot type");
   }
@@ -1188,6 +1033,18 @@ glue_widget (bool hx, bool vx, SI w, SI h) {
   return tm_new <aqua_view_widget_rep> ([[[NSView alloc] initWithFrame:NSMakeRect(0, 0, 50, 50)] autorelease]);
 }
 
+widget
+glue_widget (tree col, bool hx, bool vx, SI w, SI h) {
+  (void) col;
+  return glue_widget (hx, vx, w, h);
+}
+
+widget
+extend (widget w, array<widget> a) {
+  (void) a;
+  return w;
+}
+
 widget
 wait_widget (SI width, SI height, string message) { 
   // a widget of a specified width and height, displaying a wait message
