@@ -55,7 +55,8 @@ tt_locate (string name) {
 	   !starts (name, "bbm") &&
 	   !starts (name, "stmary") &&
 	   !starts (name, "rsfs") &&
-	   !starts (name, "grmn")
+	   !starts (name, "grmn") &&
+	   !starts (name, "mac-")
 	   // FIXME: better caching of missed tt_locates would be better
 	   )
     {
@@ -68,23 +69,57 @@ tt_locate (string name) {
 	    return url (s (start, i));
 	  start= i+1;
 	}
-      url tt_path= search_sub_dirs ("$TEXMACS_HOME_PATH/fonts/truetype");
-      return resolve (tt_path * name);
     }
-  return url_none ();
+
+  url tt_path=
+    search_sub_dirs ("$TEXMACS_HOME_PATH/fonts/truetype") |
+#if defined __MINGW32__
+    search_sub_dirs ("$windir/Fonts");
+#elif defined OS_MACOS
+    search_sub_dirs ("$HOME/Library/Fonts") |
+    search_sub_dirs ("/Library/Fonts") |
+    search_sub_dirs ("/System/Library/Fonts");
+#else
+    search_sub_dirs ("/usr/share/fonts/truetype") |
+    search_sub_dirs ("/usr/local/share/fonts/truetype");
+#endif
+  //cout << "Resolve " << name << " in " << tt_path << "\n";
+  return resolve (tt_path * name);
 }
 
 url
-tt_font_find (string name) {
+tt_font_find_sub (string name) {
+  //cout << "tt_font_find " << name << "\n";
   url u= tt_locate (name * ".pfb");
   //if (!is_none (u)) cout << name << " -> " << u << "\n";
   if (!is_none (u)) return u;
   u= tt_locate (name * ".ttf");
   //if (!is_none (u)) cout << name << " -> " << u << "\n";
   //else cout << name << " -> ???\n";
+  if (!is_none (u)) return u;
+  u= tt_locate (name * ".ttc");
+  if (!is_none (u)) return u;
+  u= tt_locate (name * ".otf");
   return u;
 }
 
+url
+tt_font_find (string name) {
+  string s= "ttf:" * name;
+  if (is_cached ("font_cache.scm", s)) {
+    string r= cache_get ("font_cache.scm", s) -> label;
+    if (r == "") return url_none ();
+    url u= url_system (r);
+    if (exists (u)) return u;
+    cache_reset ("font_cache.scm", s);
+  }
+
+  url r= tt_font_find_sub (name);
+  if (is_none (r)) cache_set ("font_cache.scm", s, "");
+  else cache_set ("font_cache.scm", s, as_string (r));
+  return r;
+}
+
 bool
 tt_font_exists (string name) {
   //cout << "tt_font_exists? " << name << "\n";
