@@ -152,12 +152,13 @@ initkeymap () {
 
 
 QTMWidget::QTMWidget (simple_widget_rep *_wid) 
-  : QTMScrollView (), backingPixmap() {
+  : QTMScrollView (), backingPixmap(), imwidget(NULL) {
   setObjectName("A QTMWidget");
   setProperty ("texmacs_widget", QVariant::fromValue ((void*) _wid));
   QAbstractScrollArea::viewport()->setMouseTracking (true);
-  QAbstractScrollArea::viewport()->setFocusPolicy (Qt::StrongFocus);
+  setFocusPolicy (Qt::StrongFocus);
   backing_pos = origin;
+  setAttribute(Qt::WA_InputMethodEnabled);
 }
 
 
@@ -167,7 +168,9 @@ QTMWidget::~QTMWidget () {
 
 void 
 QTMWidget::invalidate_rect (int x1, int y1, int x2, int y2) {
-  rectangle r = rectangle (x1, y1, x2, y2);
+  // larger rectangle is needed to avoid artifacts during cursor movements
+  // see bug #28664
+  rectangle r = rectangle (x1-3, y1-1, x2+4, y2+1);
   // cout << "invalidating " << r << LF;
   invalid_regions = invalid_regions | rectangles (r);
 }
@@ -427,7 +430,7 @@ QTMWidget::keyPressEvent (QKeyEvent* event) {
     } else {
       QString nss = event->text();
       unsigned short unic= nss.data()[0].unicode();
-      if (unic < 32 && key < 128) {
+      if (unic < 32 && key < 128 && key > 0) {
         if (((char) key) >= 'A' && ((char) key) <= 'Z') {
           if ((mods & Qt::ShiftModifier) == 0)
             key= (int) (key + ((int) 'a') - ((int) 'A'));
@@ -471,15 +474,15 @@ QTMWidget::keyPressEvent (QKeyEvent* event) {
 #ifdef Q_WS_MAC
     if (mods & Qt::ShiftModifier) r= "S-" * r;
     if (mods & Qt::MetaModifier) r= "C-" * r;        // The "Control" key
-    if (mods & Qt::ControlModifier) r= "Mod1-" * r;  // The "Command" key
-    //if (mods & Qt::KeypadModifier) r= "Mod3-" * r;
-    if (mods & Qt::AltModifier) r= "Mod4-" * r;
+    if (mods & Qt::AltModifier) r= "A-" * r;
+    if (mods & Qt::ControlModifier) r= "M-" * r;  // The "Command" key
+    //if (mods & Qt::KeypadModifier) r= "K-" * r;
 #else
     if (mods & Qt::ShiftModifier) r= "S-" * r;
     if (mods & Qt::ControlModifier) r= "C-" * r;
-    if (mods & Qt::AltModifier) r= "Mod1-" * r;
-    //if (mods & Qt::KeypadModifier) r= "Mod3-" * r;
-    if (mods & Qt::MetaModifier) r= "Mod4-" * r;     // The "Windows" key
+    if (mods & Qt::AltModifier) r= "A-" * r;
+    if (mods & Qt::MetaModifier) r= "M-" * r;     // The "Windows" key
+    //if (mods & Qt::KeypadModifier) r= "K-" * r;
 #endif
 
     if (DEBUG_QT)
@@ -532,6 +535,167 @@ mouse_decode (unsigned int mstate) {
   return "unknown";
 }
 
+#if 0 // NOT USED
+static void setRoundedMask(QWidget *widget)
+{
+  QPixmap pixmap(widget->size());
+  QPainter painter(&pixmap);
+  painter.fillRect(pixmap.rect(), Qt::white);
+  painter.setBrush(Qt::black);
+#if (QT_VERSION >= 0x040400)
+  painter.drawRoundedRect(pixmap.rect(),8,8, Qt::AbsoluteSize);
+#else
+  painter.drawRect(pixmap.rect());
+#endif
+  widget->setMask(pixmap.createMaskFromColor(Qt::white));
+}
+#endif
+
+
+#if 0 
+// OLD INPUT METHOD PREVIEW
+void
+QTMWidget::inputMethodEvent (QInputMethodEvent* event) {
+  if (! imwidget) {   
+    imwidget = new QLabel(this);
+    imwidget->setWindowFlags(Qt::Tool | Qt::FramelessWindowHint);
+  //  imwidget->setAttribute(Qt::WA_TranslucentBackground);
+//    imwidget->setAutoFillBackground(false);
+       imwidget->setAutoFillBackground(true);
+    imwidget->setWindowOpacity(0.5);
+    imwidget->setFocusPolicy(Qt::NoFocus);
+    QPalette pal = imwidget->palette();
+//    pal.setColor(QPalette::Window, QColor(0,0,255,80));
+    pal.setColor(QPalette::Window, QColor(0,0,255,255));
+    pal.setColor(QPalette::WindowText, Qt::white);
+    imwidget->setPalette(pal);
+    QFont f = imwidget->font();
+    f.setPointSize(30);
+    imwidget->setFont(f);
+    imwidget->setMargin(5);
+  }
+
+  QString const & preedit_string = event->preeditString();
+  QString const & commit_string = event->commitString();
+
+  if (preedit_string.isEmpty()) {
+    imwidget->hide();
+  } else {
+    if (DEBUG_QT)
+      cout << "IM preediting :" << preedit_string.toUtf8().data() << LF;
+    imwidget->setText(preedit_string);
+    imwidget->adjustSize();
+    QSize sz = size();
+    QRect g = imwidget->geometry();
+    QPoint c = mapToGlobal(cursor_pos);
+    c += QPoint(5,5);
+    // g.moveCenter(QPoint(sz.width()/2,sz.height()/2));
+    g.moveTopLeft(c);
+    if (DEBUG_QT)
+      cout << "IM hotspot: " << cursor_pos.x() << "," << cursor_pos.y() << LF;
+    imwidget->setGeometry(g);
+    // setRoundedMask(imwidget);
+    imwidget->show();
+#ifdef QT_MAC_USE_COCOA
+    // HACK: we unexplicably loose the focus even when showing the small window,
+    // so we need to restore it manually.....
+    // The following fixes the problem (but I do not really understand why it 
+    // happens)
+    // Maybe this is a Qt/Cocoa bug.
+    this->window()->activateWindow();
+#endif    
+  }
+  
+  if (!commit_string.isEmpty()) {
+    if (DEBUG_QT)
+      cout << "IM committing :" << commit_string.toUtf8().data() << LF;
+
+    int key = 0;
+#if 1
+    for (int i = 0; i < commit_string.size(); ++i) {
+      QKeyEvent ev(QEvent::KeyPress, key, Qt::NoModifier, commit_string[i]);
+      keyPressEvent(&ev);
+    }
+#else
+    QKeyEvent ev(QEvent::KeyPress, key, Qt::NoModifier, commit_string);
+    keyPressEvent(&ev);
+#endif
+  }
+  
+  event->accept();
+
+}  
+
+QVariant 
+QTMWidget::inputMethodQuery ( Qt::InputMethodQuery query ) const {
+  switch (query) {
+    case Qt::ImMicroFocus :
+      return QVariant(QRect(cursor_pos + QPoint(10,10),QSize(20,40)));
+    default:
+      return QVariant();
+  }
+}
+
+#else
+
+// NEW INPUT METHOD PREVIEW
+void
+QTMWidget::inputMethodEvent (QInputMethodEvent* event) {
+  
+  QString const & preedit_string = event->preeditString();
+  QString const & commit_string = event->commitString();
+  
+  if (!commit_string.isEmpty()) {
+    if (DEBUG_QT)
+      cout << "IM committing :" << commit_string.toUtf8().data() << LF;
+    
+    int key = 0;
+#if 1
+    for (int i = 0; i < commit_string.size(); ++i) {
+      QKeyEvent ev(QEvent::KeyPress, key, Qt::NoModifier, commit_string[i]);
+      keyPressEvent(&ev);
+    }
+#else
+    QKeyEvent ev(QEvent::KeyPress, key, Qt::NoModifier, commit_string);
+    keyPressEvent(&ev);
+#endif
+  }
+  
+  if (!preedit_string.isEmpty()) {
+    if (DEBUG_QT)
+      cout << "IM preediting :" << preedit_string.toUtf8().data() << LF;
+
+    // find cursor position in the preedit string
+    QList<QInputMethodEvent::Attribute> attrs = event->attributes();
+    int pos = preedit_string.count();
+    for(int i=0; i< attrs.count(); i++) 
+      if (attrs[i].type == QInputMethodEvent::Cursor) 
+        pos = attrs[i].start;
+    
+    string r = "pre-edit:" * as_string(pos) * ":" 
+                           * from_qstring(preedit_string);
+    simple_widget_rep *wid =  tm_widget();
+    if (wid)
+      the_gui -> process_keypress (wid, r, texmacs_time());
+  }
+  event->accept();
+}  
+
+QVariant 
+QTMWidget::inputMethodQuery ( Qt::InputMethodQuery query ) const {
+  switch (query) {
+    case Qt::ImMicroFocus :
+      return QVariant(QRect(cursor_pos ,QSize(5,5)));
+    default:
+      return QVariant();
+  }
+}
+
+
+#endif // input method variants
+
+
+
 void
 QTMWidget::mousePressEvent (QMouseEvent* event) {
   simple_widget_rep *wid= tm_widget ();
