@@ -13,14 +13,17 @@
 #define QT_BASIC_WIDGETS_HPP
 #include "qt_widget.hpp"
 
+class QTMInputTextWidgetHelper;
+
 class qt_text_widget_rep: public qt_widget_rep {
 public:
-  string str, lan;
+  string str;
+  int style;
   color col;
   bool tsp;
-
-  inline qt_text_widget_rep (string _s, color _col, bool _tsp, string _lan):
-    str (_s), lan (_lan), col (_col), tsp (_tsp) {}
+  
+  inline qt_text_widget_rep (string _s, int _style, color _col, bool _tsp):
+    str (_s), style(_style), col (_col), tsp (_tsp) {}
   virtual QAction* as_qaction ();
 };
 
@@ -47,10 +50,14 @@ public:
   string type;
   array<string> def;
   string text;
+  string width;
+  
+  QTMInputTextWidgetHelper *helper;
+  
+  qt_input_text_widget_rep (command _cmd, string _type, array<string> _def, string _width);
+  ~qt_input_text_widget_rep();
 
-  inline qt_input_text_widget_rep
-    (command _cmd, string _type, array<string> _def):
-      cmd (_cmd), type (_type), def (_def), text ("") {}
+  virtual QAction* as_qaction ();
 };
 
 #endif // QT_BASIC_WIDGETS_HPP
