@@ -81,7 +81,7 @@ class message {
 };
 CONCRETE_CODE(message);
 
-ostream& operator << (ostream& out, message m);
+tm_ostream& operator << (tm_ostream& out, message m);
 
 /******************************************************************************
 * The x_gui class
@@ -127,12 +127,13 @@ public:
   list<Window>                 windows_l;
   hashmap<string,tree>         selection_t;
   hashmap<string,string>       selection_s;
+  Window                       selection_w;
 
   Atom XA_CLIPBOARD;
   Atom XA_TARGETS;
 
 public:
-  x_gui_rep (int argc, char** argv);
+  x_gui_rep (int& argc, char** argv);
   ~x_gui_rep ();
 
   /******************************* Colors ************************************/
@@ -152,8 +153,8 @@ public:
   /******************************** Fonts ************************************/
   void set_shrinking_factor (int sfactor);
   void set_default_font (string name);
-  font default_font_sub (bool tt);
-  font default_font (bool tt= false);
+  font default_font_sub (bool tt, bool mini);
+  font default_font (bool tt= false, bool mini= false);
   void get_ps_char (Font fn, int c, metric& ex, glyph& gl);
   void load_system_font (string family, int size, int dpi,
 			 font_metric& fnm, font_glyphs& fng);
