@@ -13,7 +13,7 @@
 #include "X11/x_window.hpp"
 #include "iterator.hpp"
 #include "converter.hpp"
-#include "socket_server.hpp"
+#include "tm_link.hpp"
 #include "message.hpp"
 
 #ifdef OS_WIN32
@@ -26,8 +26,14 @@
 #include <unistd.h>
 #endif
 
+#if defined(X11TEXMACS) && defined(MACOSX_EXTENSIONS)
+#include "MacOS/mac_app.h"
+#endif
+
 extern hashmap<Window,pointer> Window_to_window;
 int  nr_windows= 0;
+int  alt_mask= 0;
+int  meta_mask= 0;
 
 static int  kbd_count= 0;
 static bool request_partial_redraw= false;
@@ -36,13 +42,13 @@ static bool request_partial_redraw= false;
 * Hack for getting the remote time
 ******************************************************************************/
 
-static bool time_initialized= false;
-static long time_difference = 0;
+static bool   time_initialized= false;
+static time_t time_difference = 0;
 
 static void
 synchronize_time (Time t) {
   if (time_initialized && time_difference == 0) return;
-  long d= texmacs_time () - ((time_t) t);
+  time_t d= texmacs_time () - ((time_t) t);
   if (time_initialized) {
     if (d < time_difference)
       time_difference= d;
@@ -85,12 +91,9 @@ x_gui_rep::look_up_key (XKeyEvent* ev) {
 
   /* other keyboard modifiers */
   if (N(s)==0) return s;
-  if (ev->state&4) s= "C-" * s;
-  if (ev->state&8) s= "Mod1-" * s;
-  if (ev->state&16) s= "Mod2-" * s;
-  if (ev->state&32) s= "Mod3-" * s;
-  if (ev->state&64) s= "Mod4-" * s;
-  if (ev->state&128) s= "Mod5-" * s;
+  if (ev->state & 4) s= "C-" * s;
+  if (ev->state & alt_mask) s= "A-" * s;
+  if (ev->state & meta_mask) s= "M-" * s;
   // cout << "key press: " << s << LF;
   return s;
 }
@@ -310,7 +313,8 @@ x_gui_rep::process_event (x_window win, XEvent* ev) {
   case SelectionClear:
     {
       XSelectionClearEvent& req= ev->xselectionclear;
-      if (req.selection == XA_PRIMARY) clear_selection ("mouse");
+      if (selection_w == req.window) selection_w= (Window) 0;
+      else if (req.selection == XA_PRIMARY) clear_selection ("mouse");
       else if (req.selection == XA_CLIPBOARD) clear_selection ("primary");
     }
     break;
@@ -368,6 +372,7 @@ x_gui_rep::event_loop () {
 	  report.type == NoExpose) continue;
 
     // Wait for events on all channels and interpose
+    //time_t t1= texmacs_time ();
     if (wait) {
       struct timeval tv;
       tv.tv_sec  = delay/1000;
@@ -379,6 +384,8 @@ x_gui_rep::event_loop () {
     else wait= true;
     if (the_interpose_handler != NULL) the_interpose_handler ();
     if (nr_windows == 0) continue;
+    //time_t t2= texmacs_time ();
+    //if (t2 - t1 >= 10) cout << "interpose took " << t2-t1 << "ms\n";
 
     // Popup help balloons
     if (!is_nil (balloon_wid))
@@ -387,6 +394,7 @@ x_gui_rep::event_loop () {
 	  map_balloon ();
 
     // Redraw invalid windows
+    //time_t t3= texmacs_time ();
     if (XPending (dpy) == 0 || request_partial_redraw) {
       interrupted= false;
       interrupt_time= texmacs_time () + (100 / (XPending (dpy) + 1));
@@ -401,6 +409,8 @@ x_gui_rep::event_loop () {
 	if (!win->has_focus) win->repaint_invalid_regions();
       }
     }
+    //time_t t4= texmacs_time ();
+    //if (t4 - t3 >= 10) cout << "redraw took " << t4-t3 << "ms\n";
 
     // Handle alarm messages
     if (!is_nil (messages)) {
@@ -414,5 +424,11 @@ x_gui_rep::event_loop () {
       }
       messages= not_ready;
     }
+    
+#if defined(X11TEXMACS) && defined (MACOSX_EXTENSIONS)
+    process_mac_events ();
+#endif
+    
+    
   }
 }
