@@ -14,6 +14,7 @@
 #include "file.hpp"
 #include "convert.hpp"
 #include "merge_sort.hpp"
+#include "drd_std.hpp"
 
 /******************************************************************************
 * Loading files
@@ -25,7 +26,8 @@ tm_data_rep::load_tree (url u, string fm) {
   string action= "load " * fm * " file";
   u= resolve (u);
   if (is_none (u) || load_string (u, s, false)) {
-    set_message ("Error: file#" * as_string (u) * "#not found", action);
+    tree vname= verbatim (as_string (u));
+    set_message (concat ("Error: file ", vname, " not found"), action);
     return "error";
   }
   if ((fm == "generic") || (fm == "help")) fm= get_format (s, suf);
@@ -40,6 +42,7 @@ void
 tm_data_rep::load_buffer (url u, string fm, int where, bool autosave_flag) {
   // cout << "Load= " << u << ", " << fm << "\n";
   string name= as_string (tail (u));
+  tree vname= verbatim (name);
   string action= "load " * fm * " file";
   if (fm == "generic")
     action= "load " * suffix_to_format (suffix (u)) * " file";
@@ -60,7 +63,7 @@ tm_data_rep::load_buffer (url u, string fm, int where, bool autosave_flag) {
 	}
       }
     if (!no_bufs ())
-      set_message ("Error: file#" * name * "#not found", action);
+      set_message (concat ("Error: file ", vname, " not found"), action);
     return;
   }
 
@@ -134,6 +137,122 @@ load_inclusion (url name) {
   return doc;
 }
 
+/******************************************************************************
+* Get environment and drd of style files
+******************************************************************************/
+
+static hashmap<string,bool> style_busy (false);
+static hashmap<string,tree> style_void (UNINIT);
+static hashmap<tree,hashmap<string,tree> > style_cached (style_void);
+static drd_info drd_void ("void");
+static hashmap<tree,drd_info> drd_cached (drd_void);
+
+bool
+compute_env_and_drd (tree style) {
+  ASSERT (is_tuple (style), "style tuple expected");
+  bool busy= false;
+  for (int i=0; i<N(style); i++)
+    busy= busy || style_busy->contains (as_string (style[i]));
+  hashmap<string,bool> old_busy= copy (style_busy);
+  for (int i=0; i<N(style); i++)
+    style_busy (as_string (style[i]))= true;
+
+  //cout << "Get environment of " << style << INDENT << LF;
+  hashmap<string,tree> H;
+  drd_info drd ("none", std_drd);
+  url none= url ("$PWD/none");
+  hashmap<string,tree> lref;
+  hashmap<string,tree> gref;
+  hashmap<string,tree> laux;
+  hashmap<string,tree> gaux;
+  edit_env env (drd, none, lref, gref, laux, gaux);
+  if (!busy) {
+    tree t;
+    bool ok;
+    get_server () -> style_get_cache (style, H, t, ok);
+    if (ok) {
+      env->patch_env (H);
+      ok= drd->set_locals (t);
+    }
+    if (!ok) {
+      env->exec (tree (USE_PACKAGE, A (style)));
+      env->read_env (H);
+      drd->heuristic_init (H);
+    }
+    style_cached (style)= H;
+    drd_cached (style)= drd;
+  }
+  //cout << UNINDENT << "Got environment of " << style << LF;
+
+  style_busy= old_busy;
+  return !busy;
+}
+
+hashmap<string,tree>
+get_style_env (tree style) {
+  if (style_cached->contains (style))
+    return style_cached [style];
+  else if (compute_env_and_drd (style))
+    return style_cached [style];
+  else {
+    //cout << "Busy style: " << style << "\n";
+    return hashmap<string,tree> ();
+  }
+}
+
+drd_info
+get_style_drd (tree style) {
+  init_std_drd ();
+  if (drd_cached->contains (style))
+    return drd_cached [style];
+  else if (compute_env_and_drd (style))
+    return drd_cached [style];
+  else {
+    //cout << "Busy drd: " << style << "\n";
+    return std_drd;
+  }
+}
+
+tree
+get_document_preamble (tree t) {
+  if (is_atomic (t)) return "";
+  else if (is_compound (t, "hide-preamble", 1)) return t[0];
+  else if (is_compound (t, "show-preamble", 1)) return t[0];
+  else {
+    int i, n= N(t);
+    for (i=0; i<n; i++) {
+      tree p= get_document_preamble (t[i]);
+      if (p != "") return p;
+    }
+    return "";
+  }
+}
+
+drd_info
+get_document_drd (tree doc) {
+  tree style= extract (doc, "style");
+  if (extract (doc, "TeXmacs") == "")
+    style= tree (TUPLE, "generic");
+  //cout << "style= " << style << "\n";
+  drd_info drd= get_style_drd (style);
+  tree p= get_document_preamble (doc);
+  if (p != "") {
+    drd= drd_info ("preamble", drd);
+    url none= url ("$PWD/none");
+    hashmap<string,tree> lref;
+    hashmap<string,tree> gref;
+    hashmap<string,tree> laux;
+    hashmap<string,tree> gaux;
+    edit_env env (drd, none, lref, gref, laux, gaux);
+    hashmap<string,tree> H;
+    env->exec (tree (USE_PACKAGE, A (style)));
+    env->exec (p);
+    env->read_env (H);
+    drd->heuristic_init (H);
+  }
+  return drd;
+}
+
 /******************************************************************************
 * Saving files
 ******************************************************************************/
@@ -193,6 +312,7 @@ tm_data_rep::make_document (tm_view vw, string fm) {
 void
 tm_data_rep::save_buffer (url u, string fm) {
   string name= as_string (u);
+  tree vname= verbatim (name);
   if (fm == "generic") {
     fm= suffix_to_format (suffix (u));
     if (fm == "generic") fm= "verbatim";
@@ -200,7 +320,7 @@ tm_data_rep::save_buffer (url u, string fm) {
   string action= "save " * fm * " file";
   u= resolve (u, "");
   if (is_none (u)) {
-    set_message ("Error: can't save file#" * name, action);
+    set_message (concat ("Error: can't save file ", vname), action);
     return;
   }
 
@@ -222,9 +342,9 @@ tm_data_rep::save_buffer (url u, string fm) {
   string s= tree_to_generic (doc, fm * "-document");
   if (s != "* error: unknown format *") {
     if (save_string (u, s))
-      set_message ("Error:#" * name * "#did not open", action);
+      set_message (concat ("Error: ", vname, " did not open"), action);
     else {
-      set_message ("saved " * name, action);
+      set_message (concat ("saved ", vname), action);
       if (fm == "texmacs") {
 	if (no_name () && exists (get_name_buffer ()))
 	  remove (get_name_buffer ());
@@ -244,12 +364,13 @@ tm_data_rep::auto_save () {
     tm_buffer buf= bufs[i];
     if ((buf->needs_to_be_autosaved () && (!buf->read_only))) {
       url name= buf->name;
+      tree vname= verbatim (as_string (name));
       if (!is_scratch (name))
 	name= glue (buf->name, "~");
       if (N(buf->vws)!=0) {
 	tree doc= make_document (buf->vws[0]);
 	if (save_string (name, tree_to_texmacs (doc)))
-	  set_message ("Error: " * as_string (name) * " did not open",
+	  set_message (concat ("Error: ", vname, " did not open"),
 		       "save TeXmacs file");
 	else
 	  call ("set-temporary-message",
