@@ -4,10 +4,9 @@
 * DESCRIPTION: Typesetting textual markup
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include "concater.hpp"
@@ -81,20 +80,22 @@ concater_rep::typeset_colored_substring
   }
 
 void
-concater_rep::typeset_text_string (string s, path ip) {
-  int    start, pos=0;
+concater_rep::typeset_text_string (tree t, path ip, int pos, int end) {
+  string s= t->label;
+  int    start;
   space  spc= env->fn->spc;
   space  extra= env->fn->extra;
 
   do {
     start= pos;
-    text_property tp= env->lan->advance (s, pos);
+    text_property tp= env->lan->advance (t, pos);
+    if (pos > end) pos= end;
     if ((pos>start) && (s[start]==' ')) { // spaces
       if (start==0) typeset_substring ("", ip, 0);
       penalty_min (tp->pen_after);
       PRINT_SPACE (tp->spc_before);
       PRINT_SPACE (tp->spc_after);
-      if ((pos==N(s)) || (s[pos]==' '))
+      if ((pos==end) || (s[pos]==' '))
 	typeset_substring ("", ip, pos);
     }
     else { // strings
@@ -104,25 +105,27 @@ concater_rep::typeset_text_string (string s, path ip) {
       penalty_min (tp->pen_after);
       PRINT_SPACE (tp->spc_after)
     }
-  } while (pos<N(s));
+  } while (pos<end);
 }
 
 void
-concater_rep::typeset_math_string (string s, path ip) {
-  int    start, pos=0;
+concater_rep::typeset_math_string (tree t, path ip, int pos, int end) {
+  string s= t->label;
+  int    start;
   space  spc= env->fn->spc;
   space  extra= env->fn->extra;
   bool   condensed= env->math_condensed;
 
   do {
     start= pos;
-    text_property tp= env->lan->advance (s, pos);
+    text_property tp= env->lan->advance (t, pos);
+    if (pos > end) pos= end;
     if ((pos>start) && (s[start]==' ')) { // spaces
       if (start==0) typeset_substring ("", ip, 0);
       penalty_min (tp->pen_after);
       PRINT_SPACE (tp->spc_before);
       PRINT_SPACE (tp->spc_after);
-      if ((pos==N(s)) || (s[pos]==' '))
+      if ((pos==end) || (s[pos]==' '))
 	typeset_substring ("", ip, pos);
     }
     else { // strings
@@ -135,25 +138,26 @@ concater_rep::typeset_math_string (string s, path ip) {
       if (condensed) PRINT_CONDENSED_SPACE (tp->spc_after)
       else PRINT_SPACE (tp->spc_after)
     }
-  } while (pos<N(s));
+  } while (pos<end);
 }
 
 void
-concater_rep::typeset_prog_string (tree t, path ip) {
+concater_rep::typeset_prog_string (tree t, path ip, int pos, int end) {
   string s= t->label;
-  int    start, pos=0;
+  int    start;
   space  spc= env->fn->spc;
   space  extra= env->fn->extra;
 
   do {
     start= pos;
-    text_property tp= env->lan->advance (s, pos);
+    text_property tp= env->lan->advance (t, pos);
+    if (pos > end) pos= end;
     if ((pos>start) && (s[start]==' ')) { // spaces
       if (start==0) typeset_substring ("", ip, 0);
       penalty_min (tp->pen_after);
       PRINT_SPACE (tp->spc_before);
       PRINT_SPACE (tp->spc_after);
-      if ((pos==N(s)) || (s[pos]==' '))
+      if ((pos==end) || (s[pos]==' '))
 	typeset_substring ("", ip, pos);
     }
     else { // strings
@@ -164,7 +168,7 @@ concater_rep::typeset_prog_string (tree t, path ip) {
       penalty_min (tp->pen_after);
       PRINT_SPACE (tp->spc_after)
     }
-  } while (pos<N(s));
+  } while (pos<end);
 }
 
 /******************************************************************************
@@ -198,6 +202,12 @@ concater_rep::typeset_concat (tree t, path ip) {
     typeset (t[i], descend (ip, i));
 }
 
+void
+concater_rep::typeset_rigid (tree t, path ip) {
+  box b= typeset_as_concat (env, t[0], descend (ip, 0));
+  print (STD_ITEM, move_box (ip, b, 0, 0, true));
+}
+
 /******************************************************************************
 * Typesetting space
 ******************************************************************************/
@@ -230,78 +240,45 @@ concater_rep::typeset_space (tree t, path ip) {
 
 void
 concater_rep::typeset_move (tree t, path ip) {
-  // IDEA: set left, right, bottom, top environment variables
-  //       and allow doing computations with them
-  box  b= typeset_as_concat (env, t[0], descend (ip, 0));
-  SI   x= env->as_length (env->exec (t[1]));
-  SI   y= env->as_length (env->exec (t[2]));
+  box  b  = typeset_as_concat (env, t[0], descend (ip, 0));
+  tree old= env->local_begin_extents (b);
+  SI   x  = (t[1] == ""? 0: env->as_length (env->exec (t[1]), "w"));
+  SI   y  = (t[2] == ""? 0: env->as_length (env->exec (t[2]), "h"));
+  env->local_end_extents (old);
   print (STD_ITEM, move_box (ip, b, x, y, true));
 }
 
-SI
-resize (edit_env env, SI old, SI minimum, SI maximum, tree new_size) {
-  if (!is_atomic (new_size)) return old;
-  string s= new_size->label;
-  if (N(s)<2) return old;
-  if (s[0] == '@') s= s (1, N(s));
-
-  bool flag;
-  SI   offset;
-  switch (s[0]) {
-  case 'l':
-  case 'b':
-    flag  = true;
-    offset= minimum;
-    break;
-  case 'c':
-    flag  = true;
-    offset= (minimum + maximum) >> 1;
-    break;
-  case 'r':
-  case 't':
-    flag  = true;
-    offset= maximum;
-    break;
-  default:
-    flag  = false;
-    offset= 0;
-  }
-
-  if (flag) {
-    SI arg= env->as_length (s (2, N(s)));
-    switch (s[1]) {
-    case '+': return offset + arg;
-    case '-': return offset - arg;
-    case '[': return min (offset, arg);
-    case ']': return max (offset, arg);
-    default : return arg;  
-    }
-  }
-  else return env->as_length (s);
+void
+concater_rep::typeset_shift (tree t, path ip) {
+  box  b  = typeset_as_concat (env, t[0], descend (ip, 0));
+  tree old= env->local_begin_extents (b);
+  SI   x  = (t[1] == ""? 0: env->as_length (env->exec (t[1]), "w"));
+  SI   y  = (t[2] == ""? 0: env->as_length (env->exec (t[2]), "h"));
+  env->local_end_extents (old);
+  print (STD_ITEM, shift_box (ip, b, x, y, true));
 }
 
 void
 concater_rep::typeset_resize (tree t, path ip) {
-  // IDEA: set left, right, bottom, top environment variables
-  //       and allow doing computations with them
   box  b = typeset_as_concat (env, t[0], descend (ip, 0));
-  SI   x1= resize (env, b->x1, b->x1, b->x2, env->exec (t[1]));
-  SI   y1= resize (env, b->y1, b->y1, b->y2, env->exec (t[2]));
-  SI   x2= resize (env, b->x2, b->x1, b->x2, env->exec (t[3]));
-  SI   y2= resize (env, b->y2, b->y2, b->y2, env->exec (t[4]));
-  bool fl= is_atomic (t[1]) && (N(t[1]->label)!=0) && (t[1]->label[0]=='@');
-  print (STD_ITEM, resize_box (ip, b, x1, y1, x2, y2, true, !fl));
+  tree old= env->local_begin_extents (b);
+  SI   x1 = (t[1] == ""? b->x1: env->as_length (env->exec (t[1]), "w"));
+  SI   y1 = (t[2] == ""? b->y1: env->as_length (env->exec (t[2]), "h"));
+  SI   x2 = (t[3] == ""? b->x2: env->as_length (env->exec (t[3]), "w"));
+  SI   y2 = (t[4] == ""? b->y2: env->as_length (env->exec (t[4]), "h"));
+  env->local_end_extents (old);
+  print (STD_ITEM, resize_box (ip, b, x1, y1, x2, y2, true, true));
 }
 
 void
 concater_rep::typeset_clipped (tree t, path ip) {
-  // IDEA: set left, right, bottom, top environment variables
-  //       and allow doing computations with them
-  box  b = typeset_as_concat (env, t[4], descend (ip, 4));
-  SI   x1= resize (env, b->x1, b->x1, b->x2, env->exec (t[0]));
-  SI   y1= resize (env, b->y1, b->y1, b->y2, env->exec (t[1]));
-  SI   x2= resize (env, b->x2, b->x1, b->x2, env->exec (t[2]));
-  SI   y2= resize (env, b->y2, b->y2, b->y2, env->exec (t[3]));
+  box  b = typeset_as_concat (env, t[0], descend (ip, 0));
+  tree old= env->local_begin_extents (b);
+  SI   x1 = (t[1] == ""? b->x1: env->as_length (env->exec (t[1]), "w"));
+  SI   y1 = (t[2] == ""? b->y1: env->as_length (env->exec (t[2]), "h"));
+  SI   x2 = (t[3] == ""? b->x2: env->as_length (env->exec (t[3]), "w"));
+  SI   y2 = (t[4] == ""? b->y2: env->as_length (env->exec (t[4]), "h"));
+  env->local_end_extents (old);
   print (STD_ITEM, clip_box (ip, b, x1, y1, x2, y2));
 }
 
@@ -359,7 +336,7 @@ void
 concater_rep::typeset_decorated_box (tree t, path ip) {
   (void) t; (void) ip;
   int n= N (env->decorated_boxes);
-  if ((n > 0) && (!nil (env->decorated_boxes [n-1]))) {
+  if ((n > 0) && (!is_nil (env->decorated_boxes [n-1]))) {
     print (STD_ITEM, env->decorated_boxes [n-1]);
     env->decorated_boxes [n-1]= box ();
   }
