@@ -4,10 +4,9 @@
 * DESCRIPTION: default values of the environment variables
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include "env.hpp"
@@ -22,15 +21,16 @@ initialize_default_env () {
   tree identity_m (MACRO, "x", tree (ARG, "x"));
   tree tabular_m (MACRO, "x", tree (TFORMAT, tree (ARG, "x")));
   tree the_page (MACRO, compound ("page-nr"));
-  tree gr_frame (TUPLE, "scale", "1cm", tree (TUPLE, "0.5par", "0cm"));
-  tree gr_clip (TUPLE, "clip",
-		tuple ("0par", "-0.3par"),
-		tuple ("1par", "0.3par"));
+
+  tree gr_geometry (TUPLE, "geometry", "1par", "0.6par", "center");
+  tree gr_frame (TUPLE, "scale", "1cm", tree (TUPLE, "0.5gw", "0.5gh"));
+
   tree gr_grid ("");
+  tree gr_edit_grid ("");
   tree gr_grid_aspect (TUPLE,
-		       tuple ("axes", "black"),
-		       tuple ("1", "grey"),
-		       tuple ("10", "light grey"));
+		       tuple ("axes", "#808080"),
+		       tuple ("1", "#c0c0c0"),
+		       tuple ("10", "#e0e0ff"));
 
   env (DPI)              = "600";       // resolution in dots per inch
   env (SFACTOR)          = "5";         // shrinking factor on screen
@@ -38,8 +38,13 @@ initialize_default_env () {
   env (SAVE_AUX)         = "true";      // save auxiliary data on disk ?
   env (MODE)             = "text";      // typesetting mode
   env (INFO_FLAG)        = "short";     // information about labels, etc.
+  env (WINDOW_BARS)      = "auto";      // override menu/icon bar settings
+  env (SCROLL_BARS)      = "true";      // allow scroll bars around canvas?
   env (IDENTITY)         = identity_m;  // identity macro
   env (TABULAR)          = tabular_m;   // tabular macro
+  env (THE_LABEL)        = "?";         // value of the next label
+  env (THE_TAGS)         = tree(TUPLE); // current tags
+  env (THE_MODULES)      = tree(TUPLE); // necessary modules and plug-ins
 
   env (FONT)             = "roman";     // the font name in text mode
   env (FONT_FAMILY)      = "rm";        // the font family in text mode
@@ -50,6 +55,8 @@ initialize_default_env () {
   env (MAGNIFICATION)    = "1";         // magnification (slides for instance)
   env (COLOR)            = "black";     // the color
   env (BG_COLOR)         = "white";     // the background color
+  env (LOCUS_COLOR)      = "global";    // the color of loci
+  env (VISITED_COLOR)    = "global";    // the color of visited loci
   env (LANGUAGE)         = "english";   // the language
   env (ATOM_DECORATIONS) = DATOMS;      // dots, underline, hyperlinks?, etc.
   env (LINE_DECORATIONS) = DLINES;      // boxed pars, nested envs, etc.
@@ -57,7 +64,7 @@ initialize_default_env () {
   env (XOFF_DECORATIONS) = "0tmpt";     // hor. placement of decorations
   env (YOFF_DECORATIONS) = "0tmpt";     // vert. placement of decorations
 
-  env (MATH_LANGUAGE)    = "texmath";   // the default mathematical language
+  env (MATH_LANGUAGE)    = "std-math";  // the default mathematical language
   env (MATH_FONT)        = "roman";     // the font name in math mode
   env (MATH_FONT_FAMILY) = "mr";        // the font family in math mode
   env (MATH_FONT_SERIES) = "medium";    // the font series in math mode
@@ -66,8 +73,11 @@ initialize_default_env () {
   env (MATH_DISPLAY)     = "false";     // true if we are in display style
   env (MATH_CONDENSED)   = "false";     // ignore spaces between operators ?
   env (MATH_VPOS)        = "0";         // used in fractions (-1, 0 or 1)
+  env (MATH_NESTING_MODE)= "off";       // color nested brackets?
+  env (MATH_NESTING_LEVEL)= "0";        // nesting level inside brackets
 
   env (PROG_LANGUAGE)    = "scheme";    // the default programming language
+  env (PROG_SCRIPTS)     = "scheme";    // the scripting language
   env (PROG_FONT)        = "roman";     // the font name in prog mode
   env (PROG_FONT_FAMILY) = "tt";        // the font family in prog mode
   env (PROG_FONT_SERIES) = "medium";    // the font series in prog mode
@@ -91,6 +101,7 @@ initialize_default_env () {
   env (PAR_COLUMNS_SEP)  = "2fn";       // separation between columns
 
   env (PAGE_MEDIUM)      = "papyrus";   // paper medium: paper, papyrus, auto
+  env (PAGE_PRINTED)     = "false";     // printed version?
   env (PAGE_TYPE)        = "a4";        // paper type (-> width & height)
   env (PAGE_ORIENTATION) = "portrait";  // paper orientation
   env (PAGE_WIDTH_MARGIN)  = "false";   // compute margins from par-width?
@@ -133,8 +144,8 @@ initialize_default_env () {
 
   env (TABLE_WIDTH)      = "";          // width of table
   env (TABLE_HEIGHT)     = "";          // height of table
-  env (TABLE_HMODE)      = "";          // width determination mode
-  env (TABLE_VMODE)      = "";          // height determination mode
+  env (TABLE_HMODE)      = "auto";      // width determination mode
+  env (TABLE_VMODE)      = "auto";      // height determination mode
   env (TABLE_HALIGN)     = "l";         // horizontal alignment
   env (TABLE_VALIGN)     = "f";         // vertical alignment (fraction height)
   env (TABLE_ROW_ORIGIN) = "0";         // row origin
@@ -161,8 +172,8 @@ initialize_default_env () {
   env (CELL_HEIGHT)      = "";          // height of cell
   env (CELL_HPART)       = "";          // take part of unused horizontal space
   env (CELL_VPART)       = "";          // take part of unused vertical space
-  env (CELL_HMODE)       = "exact";     // width determination mode
-  env (CELL_VMODE)       = "exact";     // height determination mode
+  env (CELL_HMODE)       = "auto";      // width determination mode
+  env (CELL_VMODE)       = "auto";      // height determination mode
   env (CELL_HALIGN)      = "l";         // horizontal alignment
   env (CELL_VALIGN)      = "B";         // vertical alignment
   env (CELL_LSEP)        = "0fn";       // left cell padding
@@ -175,6 +186,7 @@ initialize_default_env () {
   env (CELL_TBORDER)     = "0ln";       // top cell border width
   env (CELL_VCORRECT)    = "a";         // vertical limits correction
   env (CELL_HYPHEN)      = "n";         // horizontal hyphenation
+  env (CELL_BLOCK)       = "auto";      // cell contains block content?
   env (CELL_ROW_SPAN)    = "1";         // row span of cell
   env (CELL_COL_SPAN)    = "1";         // column span of cell
   env (CELL_ROW_NR)      = "1";         // row coordinate of cell
@@ -183,93 +195,177 @@ initialize_default_env () {
   env (POINT_STYLE)      = "disk";      // point style (square, circle...)
 
   env (LINE_WIDTH)       = "1ln";       // line width in graphics
-  env (LINE_STYLE)       = "solid";     // line style (dashes, etc.)
+  env (DASH_STYLE)       = "none";      // dash style
+  env (DASH_STYLE_UNIT)  = "5ln";       // dash style unit
   env (LINE_ARROWS)      = "none";      // arrows at end of lines
   env (LINE_CAPS)        = "normal";    // junctions in multilines
-  env (FILL_MODE)        = "both";      // show contour?
-  env (FILL_COLOR)       = "white";     // fill color
+  env (FILL_COLOR)       = "none";      // fill color
   env (FILL_STYLE)       = "plain";     // fill style
+  env (TEXTAT_HALIGN)    = "left";      // horizontal text-at alignment
+  env (TEXTAT_VALIGN)    = "base";      // vertical text-at alignment
 
+  env (GR_GEOMETRY)      = gr_geometry; // geometry of graphics
   env (GR_FRAME)         = gr_frame;    // coordinate frame for graphics
-  env (GR_CLIP)          = gr_clip;     // clipping of graphics
   env (GR_MODE)          = "line";      // graphical mode
   env (GR_COLOR)         = "default";   // color of new objects
+  env (GR_POINT_STYLE)   = "default";   // point style of new objects
   env (GR_LINE_WIDTH)    = "default";   // line width for new objects
-  env (GR_GRID)          = gr_grid;     // grid for graphics
-  env (GR_GRID_ASPECT)   = gr_grid_aspect; // grid aspect
+  env (GR_DASH_STYLE)      = "default"; // dash style for new objects
+  env (GR_DASH_STYLE_UNIT) = "default"; // dash style unit for new objects
+  env (GR_LINE_ARROWS)   = "default";   // line arrows for new objects
+  env (GR_FILL_COLOR)    = "default";   // fill color for new objects
+  env (GR_TEXTAT_HALIGN) = "default";   // horiz. alignment for new text-ats
+  env (GR_TEXTAT_VALIGN) = "default";   // vert. alignment for new text-ats
+  env (GR_GRID)             = gr_grid;        // grid for graphics
+  env (GR_GRID_ASPECT)      = gr_grid_aspect; // grid aspect
+  env (GR_EDIT_GRID)        = gr_edit_grid;   // edit grid
+  env (GR_EDIT_GRID_ASPECT) = gr_grid_aspect; // edit grid (subdivisions)
 
   env (SRC_STYLE)        = "angular";   // style for "source" tags
   env (SRC_SPECIAL)      = "normal";    // special treatment of certain tags
   env (SRC_COMPACT)      = "normal";    // compact inline/multi-paragraph tags?
   env (SRC_CLOSE)        = "compact";   // how to close long tags
 
+  env (CANVAS_TYPE)           = "plain";        // which kind of scrollbars
+  env (CANVAS_COLOR)          = "white";        // canvas colour
+  env (CANVAS_HPADDING)       = "0px";          // horizontal canvas padding
+  env (CANVAS_VPADDING)       = "0px";          // vertical canvas padding
+  env (CANVAS_BAR_WIDTH)      = "1em";          // width of scroll bars
+  env (CANVAS_BAR_PADDING)    = "0.25em";       // distance of scrollbars
+  env (CANVAS_BAR_COLOR)      = "grey";         // color of bar
+  env (ORNAMENT_SHAPE)        = "rectangular";  // shape of the ornament
+  env (ORNAMENT_BORDER)       = "1ln";          // border width
+  env (ORNAMENT_HPADDING)     = "0px";          // horizontal padding of body
+  env (ORNAMENT_VPADDING)     = "0px";          // vertical padding of body
+  env (ORNAMENT_COLOR)        = "light grey";   // background color
+  env (ORNAMENT_SUNNY_COLOR)  = "white";        // sunny color
+  env (ORNAMENT_SHADOW_COLOR) = "dark grey";    // shadow color
+
+  /* hiding and showing content */
+  env ("shown")= identity_m;
+  env ("ignore")=
+    tree (MACRO, "body", tree (HIDDEN, tree (ARG, "body")));
+
+  /* linking macros */
+  tree src_id (ID, tree (HARD_ID, tree (ARG, "body")));
+  tree ref_id (ID, tree (HARD_ID, tree (ARG, "Id")));
+  tree dest_url (URL, tree (ARG, "destination"));
+  tree dest_script (SCRIPT, tree (ARG, "destination"), tree (ARG, "where"));
+  tree dest_ref (URL, tree (MERGE, "#", tree (ARG, "Id")));
+  tree ln1 (LINK, "hyperlink", copy (src_id), copy (dest_url));
+  tree ln2 (LINK, "action", copy (src_id), copy (dest_script));
+  tree ln3 (LINK, "hyperlink", copy (ref_id), copy (dest_ref));
+  tree labflag (FLAG, tree (ARG, "Id"), "blue", "Id");
+  tree labtxt (SET_BINDING, tree (ARG, "Id"), tree (VALUE, THE_LABEL));
+  tree merged (MERGE, tree (VALUE, THE_TAGS), tuple (tree (ARG, "Id")));
+  tree tagflag (FLAG, tree (ARG, "Id"), "blue", "Id");
+  tree reftxt (GET_BINDING, tree (ARG, "Id"));
+  tree preftxt (GET_BINDING, tree (ARG, "Id"), "1");
+  env ("hlink")= tree (MACRO, "body", "destination",
+		       tree (LOCUS, copy (src_id), ln1,
+                             tree (ARG, "body")));
+  env ("action")= tree (MACRO, "body", "destination", "where",
+			tree (LOCUS, copy (src_id), ln2,
+                              tree (ARG, "body")));
+  env ("label")= tree (MACRO, "Id", tree (CONCAT, labflag, labtxt));
+  env ("tag")= tree (MACRO, "Id", "body",
+		     tree (WITH, "the-tags", merged,
+			   tree (SURROUND, tagflag, "",
+                                 tree (ARG, "body"))));
+  env ("reference")= tree (MACRO, "Id",
+			   tree (LOCUS, copy (ref_id), ln3, reftxt));
+  env ("pageref")= tree (MACRO, "Id",
+			 tree (LOCUS, copy (ref_id), copy (ln3), preftxt));
+
   /* further standard macros */
   env ("error")=
-    tree (MACRO, "x", tree (REWRITE_INACTIVE, tree (ARG, "x"), "error"));
+    tree (MACRO, "message",
+          tree (REWRITE_INACTIVE, tree (ARG, "message"), "error"));
   env ("style-only")=
-    tree (MACRO, "x", tree (ARG, "x"));
+    tree (MACRO, "body", tree (ARG, "body"));
   env ("style-only*")=
-    tree (MACRO, "x", tree (ARG, "x"));
+    tree (MACRO, "body", tree (ARG, "body"));
   env ("active")=
-    tree (MACRO, "x", tree (ARG, "x"));
+    tree (MACRO, "body", tree (ARG, "body"));
   env ("active*")=
-    tree (MACRO, "x", tree (ARG, "x"));
+    tree (MACRO, "body", tree (ARG, "body"));
   env ("inactive")=
-    tree (MACRO, "x", tree (REWRITE_INACTIVE, tree (ARG, "x"), "once"));
+    tree (MACRO, "body",
+          tree (REWRITE_INACTIVE, tree (ARG, "body"), "once"));
   env ("inactive*")=
-    tree (MACRO, "x", tree (REWRITE_INACTIVE, tree (ARG, "x"), "recurse"));
-  env ("indent")=
-    tree (MACRO, "x",
-	  tree (WITH, PAR_LEFT, tree (PLUS, tree (VALUE, PAR_LEFT), "1.5fn"),
-		tree (ARG, "x")));
-  env ("rightflush")=
+    tree (MACRO, "body",
+          tree (REWRITE_INACTIVE, tree (ARG, "body"), "recurse"));
+  env ("right-flush")=
     tree (MACRO, tree (HTAB, "0fn", "first"));
+  env ("indent*")=
+    tree (MACRO, "body",
+	  tree (WITH, PAR_LEFT, tree (PLUS, tree (VALUE, PAR_LEFT), "1.5fn"),
+		tree (ARG, "body")));
+  env ("indent")=
+    tree (MACRO, "body",
+	  tree (SURROUND, "", compound ("right-flush"),
+		compound ("indent*", tree (ARG, "body"))));
+  env ("math")=
+    tree (MACRO, "body", tree (WITH, MODE, "math", tree (ARG, "body")));
+  env ("text")=
+    tree (MACRO, "body", tree (WITH, MODE, "text", tree (ARG, "body")));
+  env ("pre-edit")=
+    tree (MACRO, "body", tree (WITH, COLOR, "#4040c0", tree (ARG, "body")));
   env ("mutator")=
-    tree (MACRO, "x", "y", tree (ARG, "x"));
+    tree (MACRO, "body", "y", tree (ARG, "body"));
 
   /* syntactic highlighting */
+  env ("src-regular")=
+    tree (MACRO, "body", tree (WITH, COLOR, "black", tree (ARG, "body")));
   env ("src-macro")=
-    tree (MACRO, "x", tree (WITH, COLOR, "blue",
-			    FONT_FAMILY, "ss", tree (ARG, "x")));
+    tree (MACRO, "body", tree (WITH, MODE, "src", COLOR, "blue",
+                               FONT_FAMILY, "ss", tree (ARG, "body")));
   env ("src-var")=
-    tree (MACRO, "x", tree (WITH, COLOR, "dark green",
-			    FONT_SHAPE, "italic", tree (ARG, "x")));
+    tree (MACRO, "body", tree (WITH, MODE, "src", COLOR, "dark green",
+                               FONT_SHAPE, "italic", tree (ARG, "body")));
   env ("src-arg")=
-    tree (MACRO, "x", tree (WITH, COLOR, "brown",
-			    FONT_SHAPE, "italic", tree (ARG, "x")));
+    tree (MACRO, "body", tree (WITH, MODE, "src", COLOR, "brown",
+                               FONT_SHAPE, "italic", tree (ARG, "body")));
   env ("src-tt")=
-    tree (MACRO, "x", tree (WITH, COLOR, "#224",
-			    FONT_FAMILY, "tt", tree (ARG, "x")));
-  env ("src-integer")=
-    tree (MACRO, "x", tree (WITH, COLOR, "dark grey", tree (ARG, "x")));
+    tree (MACRO, "body", tree (WITH, MODE, "src", COLOR, "#228",
+                               FONT_FAMILY, "tt", tree (ARG, "body")));
+  env ("src-numeric")=
+    tree (MACRO, "body", tree (WITH, MODE, "src", COLOR, "#848",
+                               tree (ARG, "body")));
+  env ("src-textual")=
+    tree (MACRO, "body", tree (WITH, MODE, "src", COLOR, "black",
+                               tree (ARG, "body")));
   env ("src-length")=
-    tree (MACRO, "x", tree (WITH, COLOR, "dark grey", tree (ARG, "x")));
+    tree (MACRO, "body", tree (WITH, MODE, "src", COLOR, "#288",
+                               tree (ARG, "body")));
+  env ("src-unknown")=
+    tree (MACRO, "body", tree (WITH, COLOR, "#C68", tree (ARG, "body")));
   env ("src-error")=
-    tree (MACRO, "x", tree (WITH, COLOR, "red", tree (ARG, "x")));
+    tree (MACRO, "body", tree (WITH, COLOR, "red", tree (ARG, "body")));
 
   /* for correct importation of style files and packages */
   env ("src-title")= identity_m;
   env ("src-style-file")=
-    tree (MACRO, "x", "y",
+    tree (MACRO, "style", "version",
 	  tree (ASSIGN,
-		tree (MERGE, tree (ARG, "x"), "-style"),
-		tree (ARG, "y")));
+		tree (MERGE, tree (ARG, "style"), "-style"),
+		tree (ARG, "version")));
   env ("src-package")=
-    tree (MACRO, "x", "y",
+    tree (MACRO, "package", "version",
 	  tree (CONCAT,
 		tree (ASSIGN,
-		      tree (MERGE, tree (ARG, "x"), "-package"),
-		      tree (ARG, "y")),
+		      tree (MERGE, tree (ARG, "package"), "-package"),
+		      tree (ARG, "version")),
 		tree (ASSIGN,
-		      tree (MERGE, tree (ARG, "x"), "-dtd"),
-		      tree (ARG, "y"))));
+		      tree (MERGE, tree (ARG, "package"), "-dtd"),
+		      tree (ARG, "version"))));
   env ("src-package-dtd")=
-    tree (MACRO, "x", "y", "a", "b",
+    tree (MACRO, "package", "version", "drd", "drd-version",
 	  tree (CONCAT,
 		tree (ASSIGN,
-		      tree (MERGE, tree (ARG, "x"), "-package"),
-		      tree (ARG, "y")),
+		      tree (MERGE, tree (ARG, "package"), "-package"),
+		      tree (ARG, "version")),
 		tree (ASSIGN,
-		      tree (MERGE, tree (ARG, "a"), "-dtd"),
-		      tree (ARG, "b"))));
+		      tree (MERGE, tree (ARG, "drd"), "-dtd"),
+		      tree (ARG, "drd-version"))));
 }
